/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:51 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: AbstractIndividualList.java,v $
 *
 */
package org.evolvica.core;

import java.util.Iterator;
import java.util.List;

/** Abstract base class for individual sets.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:57:51 $
 */
abstract public class AbstractIndividualList implements IIndividualSet {
	
	/**
	 * The underlying collection of individuals.
	 */
	protected List list = null;
	
	/**
	 * @see org.evolvica.core.IIndividualSet#add(org.evolvica.core.IIndividual)
	 */
	public void add( IIndividual ind ) {
		list.add( ind );
	}

	/**
	 * @see org.evolvica.core.IIndividualSet#add(org.evolvica.core.IIndividual)
	 */
	public void add( IIndividual[] indArray ) {
		for( int i = 0; i < indArray.length; i++ ) {
			list.add( indArray[i] );
		}
	}

	/**
	 * @see org.evolvica.core.IIndividualSet#clear()
	 */
	public void clear() {
		list.clear();
	}

	/**
	 * @see org.evolvica.core.IIndividualSet#contains(org.evolvica.core.IIndividual)
	 */
	public boolean contains(IIndividual ind) {
		return list.contains( ind );
	}

	/**
	 * @see org.evolvica.core.IIndividualSet#isEmpty()
	 */
	public boolean isEmpty() {
		return list.isEmpty();
	}

	/**
	 * @see org.evolvica.core.IIndividualSet#iterator()
	 */
	public ISetIterator iterator() {
		return new IndividualSetIterator( list.iterator() );
	}

	/**
	 * @see org.evolvica.core.IIndividualSet#remove(org.evolvica.core.IIndividual)
	 */
	public void remove( IIndividual ind ) {
		list.remove( ind );
	}

	/**
	 * @see org.evolvica.core.IIndividualSet#remove(org.evolvica.core.IIndividual)
	 */
	public void remove( IIndividual[] indArray ) {
		for ( int i = 0; i < indArray.length; i++ ) {
			list.remove( indArray[i] );
		}
	}

	/**
	 * @see org.evolvica.core.IIndividualSet#size()
	 */
	public int size() {
		return list.size();
	}

	/**
	 * @see org.evolvica.core.IIndividualSet#toArray()
	 */
	public IIndividual[] toArray() {
		return (IIndividual[]) list.toArray( new IIndividual[list.size()] );
	}

	private class IndividualSetIterator implements ISetIterator {
		
		private Iterator iterator;
		
		private IndividualSetIterator( Iterator iterator ) {
			this.iterator = iterator;
		}
		
		public IIndividual next() {
			return (IIndividual) iterator.next();
		}
		
		public boolean hasNext() {
			return iterator.hasNext();
		}
		
	}

}
