/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:51 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: BinaryGene.java,v $
 *
 */
package org.evolvica.core;

import org.evolvica.util.BitVector;

/** A gene of type binary string.
 * This gene contains one single string as a value.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:57:51 $
 */
public class BinaryGene extends AbstractGene {
	
	/**
	 * The binary string value of this gene.
	 */
	protected BitVector value;
	
	/**
	 * Constructor.
	 * @param value the value of this gene
	 */
	public BinaryGene( BitVector value ) {
		this.value = value;
	}
	
	/**
	 * @see org.evolvica.core.IGenotype#copy()
	 */
	public IGenotype copy() {
		return new BinaryGene( new BitVector( value ) );
	}
	
	/**
	 * Returns the value.
	 * @return float
	 */
	public BitVector getValue() {
		return value;
	}

	/**
	 * Sets the value.
	 * @param value The value to set
	 */
	public void setValue( BitVector value ) {
		this.value = value;
	}

	/**
	 * Returns a string representation of this gene.
	 * @return string representation
	 */
	public String toString() {
		return value.toString();
	}
	
}
