/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:52 $
 * $Revision: 1.4 $
 * $State: Exp $
 * $RCSfile: BuilderRepository.java,v $
 *
 */
package org.evolvica.core;

import java.util.HashMap;

import org.evolvica.engine.IDatatype;

/** Default implementation of a genotype builder repository.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.4 $ $Date: 2004/05/26 11:57:52 $
 */
public class BuilderRepository {
	
	private static HashMap map = new HashMap();
	
	private BuilderRepository() {
		// prevent instantiation
	}
	
	/**
	 * Returns a genotype builder for the given genotype identifier.
	 *
	 * @param id ID of the genotype
	 *
	 * @return builder for the given genotype or null if no builder can be found
	 */
	public static IGenotypeBuilder getBuilder( IDatatype id ) {
		return (IGenotypeBuilder) map.get( id );
	}

	/**
	 * Registers a genotype builder to this repository.
	 * 
	 * @param builder genotype builder to be registered
	 */
	public static void register( IGenotypeBuilder builder ) {
		map.put( builder.getType(), builder );
	}

}
