/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:52 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: DoubleArrayGene.java,v $
 *
 */
package org.evolvica.core;

import org.evolvica.util.Convert;

/** Gene holding an array of doubles.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:57:52 $
 */
public class DoubleArrayGene extends AbstractGene {
	
	/**
	 * Array of doubles. 
	 */
	protected double[] value = null;
	
	/**
	 * Constructor.
	 * @param value array of doubles
	 */
	public DoubleArrayGene( double[] value ) {
		setValue( value );
	}
	
	/**
	 * Returns the value.
	 * @return double[]
	 */
	public double[] getValue() {
		return value;
	}

	/**
	 * Sets the value.
	 * @param value The value to set
	 */
	public void setValue( double[] value ) {
		if ( value == null ) {
			this.value = new double[0];
		} else {
			this.value = value;
		}
	}
	
	/**
	 * @see org.evolvica.core.IGenotype#copy()
	 */
	public IGenotype copy() {
		double[] copy = new double[value.length];
		System.arraycopy( value, 0, copy, 0, value.length );
		return new DoubleArrayGene( copy );
	}
	
	/**
	 * Returns the size of the double array.
	 * @return size of the double array
	 */
	public int size() {
		return value.length;
	}
	
	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return Convert.toString( value );
	}
	
}
