/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:52 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: FloatArrayGene.java,v $
 *
 */
package org.evolvica.core;

import org.evolvica.util.Convert;

/** Gene holding an array of floats.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:57:52 $
 */
public class FloatArrayGene extends AbstractGene {
	
	/**
	 * Array of floats. 
	 */
	protected float[] value = null;
	
	/**
	 * Constructor.
	 * @param value array of floats
	 */
	public FloatArrayGene( float[] value ) {
		setValue( value );
	}
	
	/**
	 * Returns the value.
	 * @return float[]
	 */
	public float[] getValue() {
		return value;
	}

	/**
	 * Sets the value.
	 * @param value The value to set
	 */
	public void setValue( float[] value ) {
		if ( value == null ) {
			this.value = new float[0];
		} else {
			this.value = value;
		}
	}
	
	/**
	 * @see org.evolvica.core.IGenotype#copy()
	 */
	public IGenotype copy() {
		float[] copy = new float[value.length];
		System.arraycopy( value, 0, copy, 0, value.length );
		return new FloatArrayGene( copy );
	}
	
	/**
	 * Returns the size of the float array.
	 * @return size of the float array
	 */
	public int size() {
		return value.length;
	}
	
	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return Convert.toString( value );
	}
	
}
