/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:52 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: GenotypeID.java,v $
 *
 */
package org.evolvica.core;

import org.evolvica.engine.IDatatype;
import org.evolvica.util.Convert;

/** Default implementation of a genotype ID.
 * This class is not intended to be subclassed.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:57:52 $
 */
public class GenotypeID implements IDatatype {
	
	/**
	 * The referenced genotype class.
	 */
	protected Class clazz = null;
	
	/**
	 * Constructor.
	 * @param clazz genotype class
	 */
	public GenotypeID( Class clazz ) {
		this.clazz = clazz;
	}
	
	/**
	 * @see org.evolvica.engine.IDatatype#getName()
	 */
	public String getName() {
		return Convert.toClassname( clazz );
	}
	
	/**
	 * @see org.evolvica.engine.IDatatype#getReferencedClass()
	 */
	public Class getReferencedClass() {
		return clazz;
	}
	
	/**
	 * @see org.evolvica.engine.IDatatype#isEqualTo(org.evolvica.engine.IDatatype)
	 */
	public boolean isEqualTo( IDatatype otherDatatype ) {
		if ( this.clazz == otherDatatype.getReferencedClass() ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * @see org.evolvica.engine.IDatatype#isCompatibleTo(org.evolvica.engine.IDatatype)
	 */
	public boolean isCompatibleTo( IDatatype otherDatatype ) {
		if ( otherDatatype.getReferencedClass().isAssignableFrom( clazz ) ) {
			return true;
		} else {
			return false;
		}
	}
	
	
}
