/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:53 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: IGenotype.java,v $
 *
 */
package org.evolvica.core;

import java.io.Serializable;

import org.evolvica.engine.IDatatype;

/**
 * Interface for a genotype. A genotype holds the genetic information of an individual. Other
 * common expressions are gene or chromosome. Each individual holds exactly one genotype.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:57:53 $
 *
 * @since 0.4.0
 */
public interface IGenotype extends Serializable {
	
	/**
	 * Creates a complete copy of this genotype (optional operation).
	 * Note that this is an optional operation. Operators should not rely on this
	 * method and use a genotype builder for creating new genotypes.
	 *
	 * @return new genotype containing the same genetic information
	 */
	public IGenotype copy();

	/**
	 * Returns the type identifier of this genotype.
	 * Type identifiers are used for recognizing genotypes and for being able to perform validity
	 * checks in genetic operators.
	 *
	 * @return the identifier of the genotype
	 */
	public IDatatype getType();
	
	/**
	 * Returns a string representation of this genotype.
	 *
	 * @return string representation
	 */
	public String toString();
	
}
