/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:53 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: IIndividual.java,v $
 *
 */
package org.evolvica.core;

import java.io.Serializable;

/**
 * Interface for an individual.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:57:53 $
 *
 * @since 0.4.0
 */
public interface IIndividual extends Serializable {
	
	/**
	 * Gets the genotype that is assigned to this individual.
	 *
	 * @return genotype of this individual
	 */
	public IGenotype getGenotype();

	/**
	 * Sets a new genotype.
	 *
	 * @param genotype new genotype
	 */
	public void setGenotype( IGenotype genotype );

	/**
	 * Returns the phenotype of this individual (if any).
	 *
	 * @return associated phenotype or null
	 */
	public IPhenotype getPhenotype();

	/**
	 * Returns the score of this individual.
	 * This method may return null if no score has been assigned yet.
	 *
	 * @return score of the individual
	 */
	public IScore getScore();

	/**
	 * Sets a new to to this individual.
	 *
	 * @param score score to be set
	 */
	public void setScore( IScore score );

	/**
	 * Returns the fitness of this individual.
	 * This method may return null if no fitness has been assigned yet.
	 *
	 * @return fitness
	 */
	public IFitness getFitness();

	/**
	 * Sets a new fitness for this individual.
	 *
	 * @param fitness new fitness
	 */
	public void setFitness( IFitness fitness );
	
}
