/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:54 $
 * $Revision: 1.4 $
 * $State: Exp $
 * $RCSfile: IIndividualComparator.java,v $
 *
 */
 
package org.evolvica.core;

import java.util.Comparator;

/** Interface for an individual comparator.
 * Such comparators are used in genetic operators to determine which individuals
 * are better than others in terms of their target objective value, fitness or other
 * criteria.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.4 $ $Date: 2004/05/26 11:57:54 $
 */
public interface IIndividualComparator extends Comparator {
	
	/**
	 * Compares two individuals.
	 * This method compares two individuals and returns the following result:
	 * a number smaller than 0 if the individual is better than the second one,
	 * 0 if both individual are equal in terms of their quality and a number
	 * greater than 0 if the second one is better. 
	 * 
	 * @param i0 first individual
	 * @param i1 second individual
	 * @return result of the comparison according to the rules given above
	 */
	public int compare( IIndividual i0, IIndividual i1 );
	
}
