/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:54 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: IMultiGenotype.java,v $
 *
 */
package org.evolvica.core;


/**
 * Interface for a genotype containing multiple other genotypes. This interface is intended to be
 * used for special purposes only. For creating specialized genotypes implementing the IGenotype
 * interface is recommended. The contained genotypes are held together in an array-like structure.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:57:54 $
 *
 * @since 0.4.0
 */
public interface IMultiGenotype extends IGenotype {
	
	/**
	 * Gets a genotype from a specific position.
	 *
	 * @param index index of the genotype
	 * @return genotype at the given position
	 */
	public IGenotype getGenotype( int index );

	/**
	 * Sets a genotype at the given position.
	 *
	 * @param gene the genotype that is to be set
	 * @param index the index of the genotype
	 */
	public void setGenotype( IGenotype gene, int index );

}
