/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:54 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: IMultiScore.java,v $
 *
 */
package org.evolvica.core;

/**
 * Interface for a score that consists of other scores.
 * This can be used for multiobjective optimization problems. The contained scores
 * are held in an arraylike structure.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:57:54 $
 *
 * @since 0.4.0
 */
public interface IMultiScore extends IScore {
	
	/**
	 * Returns the array of scores that this multi score consists of.
	 * 
	 * @return array of scores
	 */
	public IScore[] getScores();
	
	/**
	 * Gets a single score from the multi score set.
	 *
	 * @param index index of the score
	 * @return score at the given index
	 * @deprecated use getScores() to access the array directly
	 */
	public IScore getScore( int index );
	
	/**
	 * Sets the scores of this multiscore.
	 * 
	 * @param scores array of scores
	 */
	public void setScores( IScore[] scores );
	
	/**
	 * Sets a new score at the given index.
	 *
	 * @param score score to set
	 * @param index index
	 * @deprecated use setScores() to set the array directly
	 */
	public void setScore( IScore score, int index );
	
	/**
	 * Returns the number of scores this multi score consists of.
	 *
	 * @return number of single scores this score contains
	 * @deprecated use getScores().length learn about the number of scores
	 */
	public int scoreSize();
	
}
