/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:54 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: IPhenotype.java,v $
 *
 */
package org.evolvica.core;

import java.io.Serializable;

/**
 * Interface for a phenotype.
 * Phenotypes can be associated genotypes to represent the actual value of the genotype.
 * By differentiating between genotypes and phenotypes it is possible to create individuals
 * that have different genetic representations but similiar values. Example: individuals
 * containing a bit string that encodes an integer value (phenotype) in binary code or in
 * Gray code (two different genotypes). Each individual must be assigned a genotype, a
 * phenotype is optional.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:57:54 $
 *
 * @since 0.4.0
 */
public interface IPhenotype extends Serializable {
	/**
	 * Creates a copy of this phenotype.
	 *
	 * @return copy of this phenotype
	 */
	public IPhenotype copy();
}
