/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:55 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: IndividualSetUtils.java,v $
 *
 */

package org.evolvica.core;


/**
 * Some utiility methods for individual set handling.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:57:55 $
 */
public class IndividualSetUtils {
	
	private IndividualSetUtils() {
		// prevent instantiation
	}
	
	/**
	 * Finds the best individual in a given set.
	 * 
	 * @param set individual set
	 * @param comparator comparator to be used
	 * @return best found individual
	 */
	public static final IIndividual findBest( IIndividualSet set, IIndividualComparator comparator ) {
		IIndividual best = null;
		for ( ISetIterator it = set.iterator(); it.hasNext(); ) {
			IIndividual ind = it.next();
			if ( best == null ) {
				best = ind;
			} else {
				int compare = comparator.compare( best, ind );
				if ( compare > 0 ) {
					best = ind;
				}
			}
		}
		return best;
	}
	
	/**
	 * Finds the worst individual in a given set.
	 * 
	 * @param set individual set
	 * @param comparator comparator to be used
	 * @return worst found individual
	 */
	public static final IIndividual findWorst( IIndividualSet set, IIndividualComparator comparator ) {
		IIndividual worst = null;
		for ( ISetIterator it = set.iterator(); it.hasNext(); ) {
			IIndividual ind = it.next();
			if ( worst == null ) {
				worst = ind;
			} else {
				int compare = comparator.compare( worst, ind );
				if ( compare < 0 ) {
					worst = ind;
				}
			}
		}
		return worst;
	}
	
}
