/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:55 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: IntegerArrayGene.java,v $
 *
 */
package org.evolvica.core;

import org.evolvica.util.Convert;

/** Gene holding an array of integers.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:57:55 $
 */
public class IntegerArrayGene extends AbstractGene {
	
	/**
	 * Array of integers. 
	 */
	protected int[] value = null;
	
	/**
	 * Constructor.
	 * @param value array of integers
	 */
	public IntegerArrayGene( int[] value ) {
		setValue( value );
	}
	
	/**
	 * Returns the value.
	 * @return int[]
	 */
	public int[] getValue() {
		return value;
	}

	/**
	 * Sets the value.
	 * @param value The value to set
	 */
	public void setValue( int[] value ) {
		if ( value == null ) {
			this.value = new int[0];
		} else {
			this.value = value;
		}
	}
	
	/**
	 * @see org.evolvica.core.IGenotype#copy()
	 */
	public IGenotype copy() {
		int[] copy = new int[value.length];
		System.arraycopy( value, 0, copy, 0, value.length );
		return new IntegerArrayGene( copy );
	}
	
	/**
	 * Returns the size of the integer array.
	 * @return int size of the integer array
	 */
	public int size() {
		return value.length;
	}
	
	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return Convert.toString( value );
	}
	
}
