/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:56 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: LongArrayGene.java,v $
 *
 */
package org.evolvica.core;

import org.evolvica.util.Convert;

/** Gene holding an array of longs.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:57:56 $
 */
public class LongArrayGene extends AbstractGene {
	
	/**
	 * Array of longs. 
	 */
	protected long[] value = null;
	
	/**
	 * Constructor.
	 * @param value array of longs
	 */
	public LongArrayGene( long[] value ) {
		setValue( value );
	}
	
	/**
	 * Returns the value.
	 * @return long[]
	 */
	public long[] getValue() {
		return value;
	}

	/**
	 * Sets the value.
	 * @param value The value to set
	 */
	public void setValue( long[] value ) {
		if ( value == null ) {
			this.value = new long[0];
		} else {
			this.value = value;
		}
	}
	
	/**
	 * @see org.evolvica.core.IGenotype#copy()
	 */
	public IGenotype copy() {
		long[] copy = new long[value.length];
		System.arraycopy( value, 0, copy, 0, value.length );
		return new LongArrayGene( copy );
	}
	
	/**
	 * Returns the size of the long array.
	 * @return size of the long array
	 */
	public int size() {
		return value.length;
	}
	
	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return Convert.toString( value );
	}
	
}
