/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:56 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: StringGene.java,v $
 *
 */
package org.evolvica.core;

/** A gene of type String.
 * This gene contains string as a value.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:57:56 $
 */
public class StringGene extends AbstractGene {
	
	/**
	 * The string value of this gene.
	 */
	protected String value;
	
	/**
	 * Constructor.
	 * @param value the value of this gene
	 */
	public StringGene( String value ) {
		this.value = value;
	}
	
	/**
	 * @see org.evolvica.core.IGenotype#copy()
	 */
	public IGenotype copy() {
		return new StringGene( value );
	}
	
	/**
	 * Returns the value.
	 * @return String
	 */
	public String getValue() {
		return value;
	}

	/**
	 * Sets the value.
	 * @param value The value to set
	 */
	public void setValue( String value ) {
		this.value = value;
	}

	/**
	 * Returns a string representation of this gene.
	 * @return string representation
	 */
	public String toString() {
		return value;
	}
	
}
