/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:57 $
 * $Revision: 1.9 $
 * $State: Exp $
 * $RCSfile: AbstractConduit.java,v $
 *
 */
package org.evolvica.core.common;

import org.evolvica.core.IIndividualSet;
import org.evolvica.engine.IConduit;

/**
 * Abstract base class for a conduit.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.9 $ $Date: 2004/05/26 11:57:57 $
 * @since 0.4.0
 */
abstract public class AbstractConduit extends AbstractGeneticOperator implements IConduit {
	
	/**
	 * Constructor.
	 */
	public AbstractConduit() {
		// this.descriptor.setInterface( 2, 2 );
	}
	
	/**
	 * @see org.evolvica.engine.IConduit#process(java.lang.Object[])
	 */
	public Object[] process( Object[] input ) {
		return process( (IIndividualSet[]) input );
	}

	/**
	 * @see org.evolvica.engine.IConduit#isActivatable(java.lang.Object[])
	 */
	public boolean isActivatable( Object[] input ) {
		return isActivatable( (IIndividualSet[]) input );
	}
	
	/**
	 * Processes an array of individual sets.
	 * 
	 * @param input input sets
	 * @return output sets
	 */
	abstract public IIndividualSet[] process( IIndividualSet[] input );
	
	/**
	 * Checks if this conduit can be activated by using the given individual set array as input.
	 * 
	 * @param input array of individual sets, the array may contain null-values
	 * @return true if this conduit can be activated, false instead
	 */
	abstract public boolean isActivatable( IIndividualSet[] input );
	
}
