/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:57 $
 * $Revision: 1.9 $
 * $State: Exp $
 * $RCSfile: AbstractMultiCaster.java,v $
 *
 */
 
package org.evolvica.core.common;

import org.evolvica.core.IIndividualSet;
import org.evolvica.engine.IFork;

/** Abstract base class for a multicaster.
 * A multicaster retrieves input data from its input and sends data
 * to all of its outputs. A multicaster may split up the data into smaller
 * portions or copy the data or do somthing similiar.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.9 $ $Date: 2004/05/26 11:57:57 $
 */
abstract public class AbstractMultiCaster extends AbstractGeneticOperator implements IFork {
	
	/**
	 * Constructor.
	 */
	public AbstractMultiCaster() {
		// this.descriptor.setInterface( 1, 2 );
	}
	
	/**
	 * @see org.evolvica.engine.IFork#fork(java.lang.Object)
	 */
	public Object[] fork( Object input ) {
		return disjoin( (IIndividualSet) input );
	}
	
	/**
	 * Splits up the given individual set.
	 * 
	 * @param input input set
	 * @return array of output sets
	 */
	abstract public IIndividualSet[] disjoin( IIndividualSet input );
	
}
