/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:57 $
 * $Revision: 1.10 $
 * $State: Exp $
 * $RCSfile: AbstractRouter.java,v $
 *
 */
package org.evolvica.core.common;

import org.evolvica.core.IIndividualSet;
import org.evolvica.engine.IFork;

/** Abstract base class for a router.
 * A router receives data on its input and sends the input data further to one
 * of its outputs, dependent on its internal state, some external condition(s)
 * or something similiar.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.10 $ $Date: 2004/05/26 11:57:57 $
 */
abstract public class AbstractRouter extends AbstractGeneticOperator implements IFork {
	
	/**
	 * Output objects.
	 */
	protected Object[] output;
	
	/**
	 * Constructor.
	 */
	public AbstractRouter() {
		// this.descriptor.setInterface( 1, 2 );
	}
	
	/**
	 * @see org.evolvica.engine.IFork#fork(java.lang.Object)
	 */
	public Object[] fork( Object input ) {
		int slot = route( (IIndividualSet) input );
		for (int i = 0; i < output.length; i++) {
			if ( i == slot ) {
				output[i] = input;
			} else {
				output[i] = null;
			}
		}
		return output;
	}
	
	/**
	 * Calculates the output to which the individual set should be routed.
	 * 
	 * @param input the individual set that should be routed
	 * @return the number of the output to which the individual set should is sent
	 */
	abstract public int route( IIndividualSet input );
	
}
