/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:57 $
 * $Revision: 1.10 $
 * $State: Exp $
 * $RCSfile: AbstractSink.java,v $
 *
 */
 
package org.evolvica.core.common;

import org.evolvica.core.IIndividual;
import org.evolvica.core.IIndividualSet;
import org.evolvica.core.ISetIterator;
import org.evolvica.engine.ISink;

/** Abstract sink.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.10 $ $Date: 2004/05/26 11:57:57 $
 */
abstract public class AbstractSink extends AbstractGeneticOperator implements ISink {

	/**
	 * Constructor.
	 */
	public AbstractSink() {
		// this.descriptor.setInterface( 1, 0 );
	}
	
	/**
	 * @see org.evolvica.engine.ISink#store(java.lang.Object)
	 */
	public void store( Object data ) {
		for ( ISetIterator it = ((IIndividualSet) data).iterator(); it.hasNext(); ) {
			store( (IIndividual) it.next() );
		}
	}
	
	/**
	 * @see org.evolvica.engine.ISink#contents()
	 */
	public Object[] contents() {
		return toArray();
	}
	
	/**
	 * Consumes a single individual
	 * 
	 * @param ind individual to consume
	 */
	abstract public void store( IIndividual ind );
	
	/**
	 * Returns the consumed individuals.
	 * 
	 * @return array with consumed individuals
	 */
	abstract public IIndividual[] toArray();
	
}
