
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:58 $
 * $Revision: 1.9 $
 * $State: Exp $
 * $RCSfile: BinaryFileWriter.java,v $
 *
 */

package org.evolvica.core.common;

import java.io.FileOutputStream;
import java.io.ObjectOutputStream;

import org.apache.commons.lang.SystemUtils;
import org.evolvica.core.IIndividualSet;
import org.evolvica.util.EVLog;
import org.evolvica.util.UniqueID;

/** Processor for saving individual sets to files in a binary format.
 * This processor performs a serialization on the individual set. The set
 * can be restored later using a BinaryFileInitializer}. Everytime the
 * write() method is invoked the set is written to a different file.
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.9 $ $Date: 2004/05/26 11:57:58 $
 */
public class BinaryFileWriter extends FileWriter {
	
	/**
	 * Constructor.
	 */
	public BinaryFileWriter() {
		this( "iset", "bin" );
	}
	
	/** Constructor with filename body.
	* 
	* @param body body of filename to write to
	* @param suffix suffix to be used
	*/
	public BinaryFileWriter( String body, String suffix ) {
		super( body, suffix );
	}
	
	/** Saves the individual set to a file.
	* The set is saved to a file named &lt;filename body&gt;_&lt;identifier&gt;.&lt;suffix&gt;.
	* The identifier is generated automatically to prevent overwriting old files. The default
	* suffix is .bin.
	* 
	* @param set individual set to be saved
	*/
	public void write( IIndividualSet set ) {
		String file = directory + SystemUtils.FILE_SEPARATOR + body + "_" + UniqueID.nextInt() + "." + suffix;
		try {
			ObjectOutputStream oos = new ObjectOutputStream( new FileOutputStream( file ) );
			oos.writeObject( set );
			oos.close();
		} catch ( Exception fnfe ) {
			EVLog.error( "saving individual collection to file " + file + " failed : unable to open file", fnfe );
		}
	}
	
}
