/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:58 $
 * $Revision: 1.10 $
 * $State: Exp $
 * $RCSfile: Collector.java,v $
 *
 */
 
package org.evolvica.core.common;

import org.evolvica.engine.IJoin;

/** An individual collector.
 * A collector collects individual sets from its inputs and sends them immediately to its output.
 * 
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.10 $ $Date: 2004/05/26 11:57:58 $
 */
public class Collector extends AbstractGeneticOperator implements IJoin {
	
	/**
	 * Constructor.
	 * Sets the default input size to 2.
	 */
	public Collector() {
		// this.descriptor.setInterface( 2, 1 );
	}
	
	/**
	 * @see org.evolvica.engine.IJoin#join(java.lang.Object[])
	 */
	public Object join( Object[] input ) {
		for ( int i = 0; i < input.length; i++ ) {
			if ( input[i] != null ) {
				return input[i];
			}
		}
		return null; // CHANGE: throw exception here, because this should not happen
	}

	/**
	 * @see org.evolvica.engine.IJoin#isActivatable(java.lang.Object[])
	 */
	public boolean isActivatable( Object[] input ) {
		for (int i = 0; i < input.length; i++) {
			if ( input[i] != null ) {
				return true;
			}
		}
		return false;
	}

	/**
	 * @return int
	 */
	public int getInputSize() {
		return descriptor.getInputSize();
	}

	/**
	 * Sets the inputSize.
	 * @param inputSize The inputSize to set
	 */
	public void setInputSize( int inputSize ) {
		descriptor.setInputSize( inputSize );
	}

}
