/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:58 $
 * $Revision: 1.11 $
 * $State: Exp $
 * $RCSfile: DefaultSink.java,v $
 *
 */
 
package org.evolvica.core.common;

import java.util.LinkedList;
import java.util.List;

import org.evolvica.core.*;
import org.evolvica.util.EVLog;

/** Default implementation of a sink.
 * This sink can have an optional comparator. If this comparator is set,
 * the sink will print the genotype and the score of the best individual.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.11 $ $Date: 2004/05/26 11:57:58 $
 */
public class DefaultSink extends AbstractSink {
	
	/**
	 * List with contained individuals.
	 */
	protected List list = new LinkedList();
	
	/**
	 * Comparator.
	 */
	protected IIndividualComparator comparator = null;
	
	/**
	 * @see org.evolvica.engine.ISink#store(java.lang.Object)
	 */
	public void store( Object data ) {
		for ( ISetIterator it = ((IIndividualSet) data).iterator(); it.hasNext(); ) {
			store( (IIndividual) it.next() );
		}
		if ( comparator != null ) {
			IIndividual best = IndividualSetUtils.findBest( (IIndividualSet) data, comparator );
			EVLog.info( "best individual: genotype = " + best.getGenotype() + ", score = " + best.getScore() );
		}
	}
	
	/**
	 * @see org.evolvica.core.common.AbstractSink#store(org.evolvica.core.IIndividual)
	 */
	public void store( IIndividual ind ) {
		list.add( ind );
	}
	
	/**
	 * @see org.evolvica.core.common.AbstractSink#toArray()
	 */
	public IIndividual[] toArray() {
		return (IIndividual[]) list.toArray( new IIndividual[list.size()] );
	}
	
	/**
	 * Returns the comparator.
	 * 
	 * @return comparator
	 */
	public IIndividualComparator getComparator() {
		return comparator;
	}

	/**
	 * Sets the comparator.
	 * 
	 * @param comparator new comparator
	 */
	public void setComparator( IIndividualComparator comparator ) {
		this.comparator = comparator;
	}

}
