/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:58 $
 * $Revision: 1.12 $
 * $State: Exp $
 * $RCSfile: Duplicator.java,v $
 *
 */
package org.evolvica.core.common;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;

import org.evolvica.core.*;
import org.evolvica.engine.IProcessor;

/** Duplicator for individuals.
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.12 $ $Date: 2004/05/26 11:57:58 $
 */
public class Duplicator extends AbstractGeneticOperator implements IProcessor {

	/**
	 * Number of copies.
	 * 
	 */
	protected int copies = 1;
	
	/**
	 * Builder to be used.
	 * 
	 */
	protected IIndividualBuilder builder = new DefaultIndividualBuilder();
	
	/**
	 * Constructor.
	 */
	public Duplicator() {
		// this.descriptor.setInterface( 1, 1 );
	}
	
	/**
	 * @see org.evolvica.engine.IProcessor#process(java.lang.Object)
	 */
	public Object process( Object input ) {
		IIndividualSet set = (IIndividualSet) input;
		IIndividualSet output = new IndividualArray( set.size() );
		for ( int i = 0; i < copies; i++ ) {
			for ( ISetIterator it = set.iterator(); it.hasNext(); ) {
				IIndividual copy = builder.newIndividual();
				copy.setGenotype( ((IIndividual) it.next()).getGenotype().copy() );
				output.add( copy );
			}
		}
		return output;
	}

	/**
	 * Returns the builder.
	 * 
	 * @return builder
	 */
	public IIndividualBuilder getBuilder() {
		return builder;
	}

	/**
	 * Sets the builder.
	 * @param builder The builder to set
	 */
	public void setBuilder(IIndividualBuilder builder) {
		this.builder = builder;
	}

	/**
	 * Returns the number of copies.
	 * 
	 * @return number of copies.
	 */
	public int getCopies() {
		return copies;
	}

	/**
	 * Sets the number of copies.
	 * 
	 * @param copies number of copies
	 * @exception PropertyVetoException thrown if property set fails
	 */
	public void setCopies( int copies ) throws PropertyVetoException {
		if ( copies < 1 ) {
			throw new PropertyVetoException( "parameter must be greater than 1",
				new PropertyChangeEvent( this, "copies", new Integer( this.copies ), new Integer( copies ) ) );
		} else {
			this.copies = copies;
		}
	}

}
