
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:58 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: FileWriter.java,v $
 *
 */

package org.evolvica.core.common;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.SystemUtils;
import org.evolvica.util.Directory;

/** Abstract base class for a print processor which saves an individual set into a file.
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:57:58 $
 */
abstract public class FileWriter extends AbstractWriter {
	
	/** Body of the filename.
	 * 
	 */
	protected String body = "iset";
	
	/** Filename suffix.
	 * 
	 */
	protected String suffix = "dat";
	
	/** Directory to write to.
	 * 
	 */
	protected Directory directory = new Directory( SystemUtils.USER_HOME );
	
	/** Default constructor.
	* The filename is set to "iset" and the suffix is set to "dat".
	* 
	*/
	public FileWriter() {
		this( "iset", "dat" );
	}
	
	/** Constructor with filename body and suffix.
	* 
	* @param body body of the filename to write to
	* @param suffix suffix of the filename
	*/
	public FileWriter( String body, String suffix ) {
		this.body = body;
		this.suffix = suffix;
	}
	
	/**
	 * Returns the body.
	 * 
	 * @return body.
	 */
	public String getBody() {
		return body;
	}

	/**
	 * Returns the directory.
	 * 
	 * @return directory.
	 */
	public Directory getDirectory() {
		return directory;
	}

	/**
	 * Returns the suffix.
	 * 
	 * @return suffix
	 */
	public String getSuffix() {
		return suffix;
	}

	/**
	 * Sets the body.
	 * 
	 * @param string new body
	 */
	public void setBody( String string ) throws PropertyVetoException {
		if ( StringUtils.isNotEmpty( string ) ) {
			this.body = string;
		} else {
			throw new PropertyVetoException( "body may not be empty",
				new PropertyChangeEvent( this, "body", this.body, string )
			);
		}
	}

	/**
	 * Stes the directory.
	 * 
	 * @param dir directory
	 */
	public void setDirectory( Directory dir ) throws PropertyVetoException {
		if ( dir != null ) {
			this.directory = dir;
		} else {
			throw new PropertyVetoException( "directory may not be empty",
				new PropertyChangeEvent( this, "driectory", this.directory, dir )
			);
		}
	}

	/**
	 * Sets the suffix.
	 * 
	 * @param string new suffix
	 */
	public void setSuffix( String string ) throws PropertyVetoException {
		if ( StringUtils.isNotEmpty( string ) ) {
			this.suffix = string;
		} else {
			throw new PropertyVetoException( "suffix may not be empty",
				new PropertyChangeEvent( this, "suffix", this.suffix, string )
			);
		}
	}

}
