
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:59 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: Log4JWriter.java,v $
 *
 */

package org.evolvica.core.common;

import org.apache.log4j.Category;
import org.evolvica.core.IIndividual;
import org.evolvica.core.IIndividualSet;
import org.evolvica.core.ISetIterator;

/** Print processor that converts individuals to log messages.
 * This class can be used for debugging purposes. Instead of simple
 * printing, individual are sent to a Log4J log category. The log level is set to INFO.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:57:59 $
 */
public class Log4JWriter extends AbstractWriter {
	
	/** Indicator if printing of genetic representation should be surpressed.
	 */
	protected boolean surpressRepresentation = true;
	
	/** Name of the logging category.
	 * The name is always preceded by "evolvica.log4j".
	 */
	protected String logCategoryName;
	
	/** The log category to send log messages to.
	 */
	protected Category logCategory;
	
	/** Defautl constructor.
	 */
	public Log4JWriter() {
		this( "default" );
	}
	
	/** Constructor with log category name.
	 * 
	 * @param categoryName log category name
	 */
	public Log4JWriter( String categoryName ) {
		this.logCategoryName = categoryName;
		logCategory = Category.getInstance( "evolvica.log4j." + this.logCategoryName );
	}
	
	/** Sends all individuals in a set to the internal log category.
	 * 
	 * @param set set of individuals to print
	 */
	public void write( IIndividualSet set ) {
		int index = 0;
		for( ISetIterator it = set.iterator(); it.hasNext(); ) {
			IIndividual ind = it.next();
			StringBuffer sb = new StringBuffer();
			sb.append( "I" + index++ );
			sb.append( " score " + ind.getScore() + ", fitness " + ind.getFitness() );
			if ( ! surpressRepresentation ) {
				sb.append( ", " + ind.getGenotype() );
			}
			logCategory.info( sb.toString() );
		}
	}
	
}
