
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:57:59 $
 * $Revision: 1.9 $
 * $State: Exp $
 * $RCSfile: XMLWriter.java,v $
 *
 */

package org.evolvica.core.common;

import java.io.FileOutputStream;

import org.apache.commons.lang.SystemUtils;
import org.evolvica.core.IIndividualSet;
import org.evolvica.util.EVLog;
import org.evolvica.util.UniqueID;

import JSX.ObjOut;

/** Processor for saving individual sets to files in a XML format.
 * This processor performs a XML serialization on the individual set. The set
 * can be restored later using a XMLFileInitializer.
 * Everytime the write() method is invoked the set is written to a different file.
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.9 $ $Date: 2004/05/26 11:57:59 $
 */
public class XMLWriter extends FileWriter {
	
	/**
	 * Constructor.
	 */
	public XMLWriter() {
		this( "iset", "xml" );
	}
	
	/** Constructor with filename body.
	* 
	* @param body body of filename to write to
	* @param suffix suffix to be used
	*/
	public XMLWriter( String body, String suffix ) {
		super( body, suffix );
	}
	
	/** Saves the individual set to a file.
	* The collection is saved to a file named &lt;filename body&gt;_&lt;identifier&gt;.&lt;suffix&gt;.
	* The identifier is generated automatically to prevent overwriting old files. The default suffix
	* is .xml.
	* 
	* @param set individual set to be saved
	*/
	public void write( IIndividualSet set ) {
		String file = directory + SystemUtils.FILE_SEPARATOR + body + "_" + UniqueID.nextLong() + "." + suffix;
		try {
			ObjOut out = new ObjOut( new FileOutputStream( file ) );
			out.writeObject( set );
			out.close();
		} catch ( Exception fnfe ) {
			EVLog.error( "saving individual collection to file " + file + " failed : unable to open file", fnfe );
		}
	}
	
}
