/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:00 $
 * $Revision: 1.11 $
 * $State: Exp $
 * $RCSfile: AbstractEvaluator.java,v $
 *
 */
package org.evolvica.core.eval;

import org.evolvica.core.IIndividual;
import org.evolvica.core.IIndividualSet;
import org.evolvica.core.ISetIterator;
import org.evolvica.engine.IProcessor;
import org.evolvica.core.common.AbstractGeneticOperator;

/** Abstract base class for an evaluator.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.11 $ $Date: 2004/05/26 11:58:00 $
 */
abstract public class AbstractEvaluator extends AbstractGeneticOperator implements IProcessor {
	
	/**
	 * Constructor.
	 */
	public AbstractEvaluator() {
		// this.descriptor.setInterface( 1, 1 );
	}
	
	/**
	 * Evaluates an individual.
	 * Evaluation means assigning a score or fitness value
	 * to the given individual
	 *
	 * @param ind individual to be evaluated
	 */
	abstract public void evaluate( IIndividual ind );

	/**
	 * @see org.evolvica.engine.IProcessor#process(java.lang.Object)
	 */
	public Object process( Object input ) {
		for ( ISetIterator it = ((IIndividualSet) input).iterator(); it.hasNext(); ) {
			evaluate( (IIndividual) it.next() );
		}
		return input;
	}

}
