/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:00 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: CompoundScore.java,v $
 *
 */
package org.evolvica.core.eval;

import org.evolvica.core.IMultiScore;
import org.evolvica.core.IScore;
import org.evolvica.util.Convert;

/**
 * Default implementation for a multi score.
 * A compound score consists of several other scores and can be used
 * in multi-objective algorithms.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:00 $
 * @since 0.4.0
 */

public class CompoundScore implements IMultiScore {
	
	/**
	 * Array of scores.
	 * 
	 */
	protected IScore[] scores = new IScore[0];
	
	/**
	 * Constructor.
	 * This creates a compound score with an array size of 0.
	 * 
	 */
	public CompoundScore() {
		this( 0 );
	}
	
	/**
	 * Constructor.
	 * 
	 * @param size number of scores
	 */
	public CompoundScore( int size ) {
		scores = new IScore[size];
	}
	
	/**
	 * Constructor with an array of scores.
	 * 
	 * @param scores array of scores
	 */
	public CompoundScore( IScore[] scores ) {
		this.scores = scores;
	}
	
	/**
	 * @see org.evolvica.core.IMultiScore#getScore(int)
	 */
	public IScore getScore( int index ) {
		return scores[index];
	}

	/**
	 * @see org.evolvica.core.IMultiScore#getScores()
	 */
	public IScore[] getScores() {
		return scores;
	}

	/**
	 * @see org.evolvica.core.IMultiScore#setScore(org.evolvica.core.IScore, int)
	 */
	public void setScore( IScore score, int index ) {
		scores[index] = score;
	}

	/** (Kein Javadoc)
	 * @see org.evolvica.core.IScore#getValueAsObject()
	 */
	public Object getValueAsObject() {
		return scores;
	}

	/**
	 * @see org.evolvica.core.IMultiScore#scoreSize()
	 */
	public int scoreSize() {
		return scores.length;
	}
	
	/**
	 * Returns a string representation.
	 * 
	 * @return string representation of this score in the form {1,2,...}
	 */
	public String toString() {
		return Convert.toString( scores );
	}

	/**
	 * @see org.evolvica.core.IMultiScore#setScores(org.evolvica.core.IScore[])
	 */
	public void setScores( IScore[] scores ) {
		this.scores = scores;
	}
	
}

