/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:01 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: ConvergentFloatScoreComparator.java,v $
 *
 */
 
package org.evolvica.core.eval;

import org.evolvica.core.IScore;

/** Comparator for scores of type float (convergence mode).
 * This comparator assumes that the score should "converge" to a certain best
 * value. This means the closer the score to this best value the better is the score.
 * 
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:58:01 $
 */
public class ConvergentFloatScoreComparator extends AbstractScoreComparator {
	
	/**
	 * Best value.
	 * 
	 */
	protected float best = 0.0f;
	
	/**
	 * @see org.evolvica.core.IScoreComparator#compare(org.evolvica.core.IScore, org.evolvica.core.IScore)
	 */
	public int compare( IScore score0, IScore score1 ) {
		float diff0 = Math.abs( ((FloatScore) score0).getValue() - best );
		float diff1 = Math.abs( ((FloatScore) score1).getValue() - best );
		if ( diff0 < diff1 ) {
			return -1;
		} else if ( diff0 == diff1 ) {
			return 0;
		} else {
			return 1;
		} 	}
	
	/**
	 * @return float
	 */
	public float getBest() {
		return best;
	}

	/**
	 * Sets the best value.
	 * @param best The best value to set
	 */
	public void setBest( float best ) {
		this.best = best;
	}

}
