/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:01 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: DoubleScore.java,v $
 *
 */
package org.evolvica.core.eval;

import org.evolvica.core.IScore;

/** Double score.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:58:01 $
 */
public class DoubleScore implements IScore {
	
	/** Score value.
	* 
	*/
	protected double score;
	
	/** Constructor.
	 * The score value is set to 0.
	 * 
	 */
	public DoubleScore() {
		this( 0.0 );
	}
	
	/** Constructor with initial score.
	* 
	* @param score score
	*/
	public DoubleScore( double score ) {
		this.score = score;
	}
	
	/** Gets the value of the score as dDouble.
	* 
	* @return value of score
	*/
	public double getValue() {
		return score;
	}
	
	/** Gets the value of the score as a Double object.
	* 
	* @return value of score as Double
	*/
	public Object getValueAsObject() {
		return new Double( score );
	}
	
	/** Sets the value of this score object.
	* 
	* @param value new score value
	*/
	public void setValue( double value ) {
		this.score = value;
	}
	
	/** Gets the value of the score as dDouble.
	* 
	* @return value of score
	* @deprecated use getValue()
	*/
	public double doubleValue() {
		return score;
	}
	
	/** Returns a string representation.
	* 
	* @return string representation
	*/
	public String toString() {
		return String.valueOf( score );
	}
	
}
