/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:01 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: FloatScore.java,v $
 *
 */
package org.evolvica.core.eval;

import org.evolvica.core.IScore;

/** Float score.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:58:01 $
 */
public class FloatScore implements IScore {
	
	/** Score value.
	* 
	*/
	protected float score;
	
	/** Constructor.
	 * The score value is set to 0.0.
	 * 
	 */
	public FloatScore() {
		this( 0.0f );
	}
	
	/** Constructor with initial score.
	* 
	* @param score score
	*/
	public FloatScore( float score ) {
		this.score = score;
	}
	
	/** Gets the value of the score as float.
	* 
	* @return value of score
	*/
	public float getValue() {
		return score;
	}
	
	/** Gets the value of the score as a Float object.
	* 
	* @return value of score as Float
	*/
	public Object getValueAsObject() {
		return new Float( score );
	}
	
	/** Sets the value of this score object.
	* 
	* @param value new score value
	*/
	public void setValue( float value ) {
		this.score = value;
	}
	
	/** Gets the value of the score as float.
	* 
	* @return value of score
	* @deprecated use getValue()
	*/
	public float floatValue() {
		return score;
	}
	
	/** Returns a string representation.
	* 
	* @return string representation
	*/
	public String toString() {
		return String.valueOf( score );
	}
	
}
