/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:02 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: IntegerScore.java,v $
 *
 */
package org.evolvica.core.eval;

import org.evolvica.core.IScore;

/** Integer score.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:58:02 $
 */
public class IntegerScore implements IScore {
	
	/** Score value.
	* 
	*/
	protected int score;
	
	/** Constructor.
	 * The score value is set to 0.
	* 
	*/
	public IntegerScore() {
		this( 0 );
	}
	
	/** Constructor with initial score.
	* 
	* @param score score
	*/
	public IntegerScore( int score ) {
		this.score = score;
	}
	
	/** Gets the value of the score as int.
	* 
	* @return value of score
	*/
	public int getValue() {
		return score;
	}
	
	/** Gets the value of the score as an Integer object.
	* 
	* @return value of score as Integer
	*/
	public Object getValueAsObject() {
		return new Integer( score );
	}
	
	/** Sets the value of this score object.
	* 
	* @param value new score value
	*/
	public void setValue( int value ) {
		this.score = value;
	}
	
	/** Gets the value of the score as int.
	* 
	* @return value of score
	* @deprecated use getValue()
	*/
	public int intValue() {
		return score;
	}
	
	/** Returns a string representation.
	* 
	* @return string representation
	*/
	public String toString() {
		return String.valueOf( score );
	}
	
}
