/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:02 $
 * $Revision: 1.3 $
 * $State: Exp $
 * $RCSfile: RastriginEvaluator.java,v $
 * 
 */
package org.evolvica.core.eval;

import org.evolvica.core.*;
import org.evolvica.core.DoubleArrayGene;
import org.evolvica.core.IGenotype;
import org.evolvica.core.IIndividual;

/** Evaluator for the Rastrigin function.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.3 $ $Date: 2004/05/26 11:58:02 $
 */
public class RastriginEvaluator extends AbstractEvaluator {
	
	private static final double a_d = 10.0;
	
	private static final double omega_d = 2.0 * Math.PI;
	
	private static final float a_f = 10.0f;
	
	private static final float omega_f = 2.0f * (float) Math.PI;
	
	/**
	 * @see org.evolvica.core.eval.AbstractEvaluator#evaluate(org.evolvica.core.IIndividual)
	 */
	public void evaluate( IIndividual ind ) {
		IGenotype genotype = ind.getGenotype();
		if ( genotype instanceof FloatArrayGene ) { // version for float[]
			float[] gene = ((FloatArrayGene) genotype).getValue();
			float sum = 0.0f;
			for ( int i = 0; i < gene.length; i++ ) {
				sum += gene[i] * gene[i] - a_f * Math.cos( omega_f * gene[i] );
			}
			float score = a_f * gene.length + sum;
			ind.setScore( new FloatScore( score ) );
		} else if ( genotype instanceof DoubleArrayGene ) { // version for double[]
			double[] gene = ((DoubleArrayGene) genotype).getValue();
			double sum = 0.0;
			for ( int i = 0; i < gene.length; i++ ) {
				sum += gene[i] * gene[i] - a_d * Math.cos( omega_d * gene[i] );
			}
			double score = a_d * gene.length + sum;
			ind.setScore( new DoubleScore( score ) );
		} else {
			ind.setScore( null );
		}
	}

}
