/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:03 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: RosenbrockEvaluator.java,v $
 * 
 */
package org.evolvica.core.eval;

import org.evolvica.core.*;

/** Evaluator for Rosenbrock's function.
 *
 * @author Andreas Rummler
 * @since 0.6.0
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:58:03 $
 */
public class RosenbrockEvaluator extends AbstractEvaluator {

	/**
	 * @see org.evolvica.core.eval.AbstractEvaluator#evaluate(org.evolvica.core.IIndividual)
	 */
	public void evaluate( IIndividual ind ) {
		IGenotype genotype = ind.getGenotype();
		if ( genotype instanceof FloatArrayGene ) { // version for float[]
			float[] gene = ((FloatArrayGene) genotype).getValue();
			float sum = 0.0f;
			for ( int i = 1; i < gene.length; i++ ) { // iteration only from 1 to n !
				float term_a = gene[i] - (gene[i-1] * gene[i-1]);
				float term_b = 1.0f - gene[i-1];
				sum += 100.0f * term_a * term_a + term_b * term_b;
			}
			ind.setScore( new FloatScore( sum ) );
		} else if ( genotype instanceof DoubleArrayGene ) { // version for double[]
			double[] gene = ((DoubleArrayGene) genotype).getValue();
			double sum = 0.0;
			for ( int i = 1; i < gene.length; i++ ) { // iteration only from 1 to n !
				double term_a = gene[i] - (gene[i-1] * gene[i-1]);
				double term_b = 1.0 - gene[i-1];
				sum += 100.0 * term_a * term_a + term_b * term_b;
			}
			ind.setScore( new DoubleScore( sum ) );
		} else {
			ind.setScore( null );
		}
	}

}
