/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:03 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: SchwefelEvaluator.java,v $
 * 
 */
package org.evolvica.core.eval;

import org.evolvica.core.*;

/** Evaluator for Schwefel's function.
 *
 * @author Andreas Rummler
 * @since 0.6.0
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:58:03 $
 */
public class SchwefelEvaluator extends AbstractEvaluator {

	/**
	 * @see org.evolvica.core.eval.AbstractEvaluator#evaluate(org.evolvica.core.IIndividual)
	 */
	public void evaluate( IIndividual ind ) {
		IGenotype genotype = ind.getGenotype();
		if ( genotype instanceof FloatArrayGene ) { // version for float[]
			float[] gene = ((FloatArrayGene) genotype).getValue();
			float sum = 0.0f;
			for ( int i = 0; i < gene.length; i++ ) {
				sum += - gene[i] * Math.sin( Math.sqrt( Math.abs( gene[i] ) ) );
			}
			ind.setScore( new FloatScore( sum ) );
		} else if ( genotype instanceof DoubleArrayGene ) { // version for double[]
			double[] gene = ((DoubleArrayGene) genotype).getValue();
			double sum = 0.0;
			for ( int i = 0; i < gene.length; i++ ) {
				sum += - gene[i] * Math.sin( Math.sqrt( Math.abs( gene[i] ) ) );
			}
			ind.setScore( new DoubleScore( sum ) );
		} else {
			ind.setScore( null );
		}
	}
	
}
