/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:04 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: SphereEvaluator.java,v $
 *
 */
package org.evolvica.core.eval;

import org.evolvica.core.*;
import org.evolvica.core.FloatArrayGene;
import org.evolvica.core.IGenotype;
import org.evolvica.core.IIndividual;

/**
 * Evaluator for the Sphere function.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:58:04 $
 * @since 0.4.0
 */
public class SphereEvaluator extends AbstractEvaluator {
	
	/**
	 * @see org.evolvica.core.eval.AbstractEvaluator#evaluate(org.evolvica.core.IIndividual)
	 */
	public void evaluate( IIndividual ind ) {
		IGenotype genotype = ind.getGenotype();
		if ( genotype instanceof FloatArrayGene ) { // version for float[]
			float[] array = ((FloatArrayGene) genotype).getValue();
			float sphere = 0.0f;
			for ( int i = 0; i < array.length; i++ ) {
				sphere += array[i] * array[i];
			}
			ind.setScore( new FloatScore( sphere ) );
		} else if ( genotype instanceof DoubleArrayGene ) {
			double[] array = ((DoubleArrayGene) genotype).getValue();
			double sphere = 0.0f;
			for ( int i = 0; i < array.length; i++ ) {
				sphere += array[i] * array[i];
			}
			ind.setScore( new DoubleScore( sphere ) );
		} else {
			ind.setScore( null );
		}
	}

}
