/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:04 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: BinaryInitializer.java,v $
 * 
 */
package org.evolvica.core.init;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;

import org.evolvica.core.BinaryGene;
import org.evolvica.core.IGenotype;
import org.evolvica.util.BitVector;

/** Initiatializer for binary strings.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:04 $
 */
public class BinaryInitializer extends AbstractInitializer {
	
	/**
	 * Length of the bitstring.
	 */
	protected int length = 10;
	
	/**
	 * @see org.evolvica.core.init.AbstractInitializer#createGenotype()
	 */
	protected IGenotype createGenotype() {
		BitVector vector = new BitVector( length );
		vector.randomize();
		return new BinaryGene( vector );
	}

	/**
	 * Returns the length of the bitstring.
	 * 
	 * @return length of the bitstring
	 */
	public int getLength() {
		return length;
	}

	/**
	 * Sets the length of the bitstring.
	 * 
	 * @param i length of the bitstring
	 * @throws PropertyVetoException thrown in case the property cannot be set
	 */
	public void setLength( int i ) throws PropertyVetoException {
		if ( i > 0 ) {
			length = i;
		} else {
			throw new PropertyVetoException( "bit vector size must be > 0",
				new PropertyChangeEvent( this, "length", new Integer( length ), new Integer( i ) )
			);
		}
	}
	
}
