/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:05 $
 * $Revision: 1.10 $
 * $State: Exp $
 * $RCSfile: FloatInitializer.java,v $
 *
 */
package org.evolvica.core.init;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;

import org.evolvica.core.IGenotype;
import org.evolvica.core.FloatGene;
import org.evolvica.util.RandomUtils;

/** Creation of individuals containing float chromosomes.
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.10 $ $Date: 2004/05/26 11:58:05 $
 */
public class FloatInitializer extends AbstractInitializer {

	/** Lower bound of generated float values.
	 * 
	 */
	protected float lowerBound;
	
	/** Upper bound of generated float values.
	 * 
	 */
	protected float upperBound;
	
	/**
	 * Constructor.
	 */
	public FloatInitializer() {
		this( 0.0f, 1.0f );
	}
	
	/** Constructor.
	 * 
	 * @param lowerbound lower bound of generated float values
	 * @param upperbound upper bound of generated float values
	 */
	public FloatInitializer( float lowerbound, float upperbound ) {
		this.lowerBound = lowerbound;
		this.upperBound = upperbound;
	}
	
	/**
	 * @see org.evolvica.core.init.AbstractInitializer#createGenotype()
	 */
	protected IGenotype createGenotype() {
		return new FloatGene( RandomUtils.randomFloat( lowerBound, upperBound ) );
	}

	/**
	 * @return float
	 */
	public float getLowerBound() {
		return lowerBound;
	}

	/**
	 * @return float
	 */
	public float getUpperBound() {
		return upperBound;
	}

	/**
	 * Sets the lowerBound.
	 * @param lowerBound The lowerBound to set
	 */
	public void setLowerBound( float lowerBound ) throws PropertyVetoException {
		if ( lowerBound < upperBound ) {
			this.lowerBound = lowerBound;
		} else {
			throw new PropertyVetoException( "lower bound must be smaller than value of upper bound",
			new PropertyChangeEvent( this, "lowerBound", new Float( this.lowerBound ), new Float( lowerBound ) )
			);
		}
	}

	/**
	 * Sets the upperBound.
	 * @param upperBound The upperBound to set
	 */
	public void setUpperBound( float upperBound ) throws PropertyVetoException {
		if ( upperBound > lowerBound ) {
			this.upperBound = upperBound;
		} else {
			throw new PropertyVetoException( "upper bound must be greater than value of lower bound",
			new PropertyChangeEvent( this, "upperBound", new Float( this.upperBound ), new Float( upperBound ) )
			);
		}
	}

}
