/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:05 $
 * $Revision: 1.13 $
 * $State: Exp $
 * $RCSfile: IntegerInitializer.java,v $
 *
 */
package org.evolvica.core.init;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;

import org.evolvica.core.IGenotype;
import org.evolvica.core.IntegerGene;
import org.evolvica.util.RandomUtils;

/** Creation of individuals containing integer chromosomes.
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.13 $ $Date: 2004/05/26 11:58:05 $
 */
public class IntegerInitializer extends AbstractInitializer {

	/** Lower bound of generated int values.
	 * 
	 */
	protected int lowerBound;
	
	/** Upper bound of generated int values.
	 * 
	 */
	protected int upperBound;
	
	/**
	 * Constructor.
	 */
	public IntegerInitializer() {
		this( 0, 10 );
	}
	
	/** Constructor.
	 * 
	 * @param lowerbound lower bound of generated int values
	 * @param upperbound upper bound of generated int values
	 */
	public IntegerInitializer( int lowerbound, int upperbound ) {
		this.lowerBound = lowerbound;
		this.upperBound = upperbound;
	}
	
	/**
	 * @see org.evolvica.core.init.AbstractInitializer#createGenotype()
	 */
	protected IGenotype createGenotype() {
		return new IntegerGene( RandomUtils.randomInt( lowerBound, upperBound ) );
	}

	/**
	 * @return int
	 */
	public int getLowerBound() {
		return lowerBound;
	}

	/**
	 * @return int
	 */
	public int getUpperBound() {
		return upperBound;
	}

	/**
	 * Sets the lowerBound.
	 * @param lowerBound The lowerBound to set
	 */
	public void setLowerBound( int lowerBound ) throws PropertyVetoException {
		if ( lowerBound < upperBound ) {
			this.lowerBound = lowerBound;
		} else {
			throw new PropertyVetoException( "lower bound must be smaller than value of upper bound",
			new PropertyChangeEvent( this, "lowerBound", new Integer( this.lowerBound ), new Integer( lowerBound ) )
			);
		}
	}

	/**
	 * Sets the upperBound.
	 * @param upperBound The upperBound to set
	 */
	public void setUpperBound( int upperBound ) throws PropertyVetoException {
		if ( upperBound > lowerBound ) {
			this.upperBound = upperBound;
		} else {
			throw new PropertyVetoException( "upper bound must be greater than value of lower bound",
			new PropertyChangeEvent( this, "upperBound", new Integer( this.upperBound ), new Integer( upperBound ) )
			);
		}
	}
	
}
