/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:05 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: IntegerSequenceInitializer.java,v $
 *
 */

package org.evolvica.core.init;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;

import org.evolvica.core.IGenotype;
import org.evolvica.core.IntegerArrayGene;
import org.evolvica.util.RandomUtils;

/** Creation of integer sequences.
 * This class creates an individual containing an array of integers.
 * Every integer number is taken from a sequence and is
 * contained exactly once.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:05 $
 */
public class IntegerSequenceInitializer extends AbstractInitializer {
	
	/** Size of the array to generated.
	* 
	*/
	protected int arraySize;
	
	/**
	 * Constructor.
	 */
	public IntegerSequenceInitializer() {
		this( 10 );
	}
	
	/** Constructor.
	* 
	* @param arraySize size of the array to be generated
	*/
	public IntegerSequenceInitializer( int arraySize ) {
		this.arraySize = arraySize;
	}
	
	/**
	 * @see org.evolvica.core.init.AbstractInitializer#createGenotype()
	 */
	protected IGenotype createGenotype() {
		return new IntegerArrayGene( RandomUtils.randomIntSequence( arraySize ) );
	}

	/**
	 * @return int
	 */
	public int getArraySize() {
		return arraySize;
	}

	/**
	 * Sets the arraySize.
	 * @param arraySize The arraySize to set
	 */
	public void setArraySize( int arraySize ) throws PropertyVetoException {
		if ( arraySize > 0 ) {
			this.arraySize = arraySize;
		} else {
			throw new PropertyVetoException( "array size must be > 0",
				new PropertyChangeEvent( this, "arraySize", new Integer( this.arraySize ), new Integer( arraySize ) )
			);
		}
	}

}
