/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:06 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: RandomStringInitializer.java,v $
 *
 */

package org.evolvica.core.init;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;

import org.apache.commons.lang.RandomStringUtils;
import org.evolvica.core.IGenotype;
import org.evolvica.core.StringGene;

/** Operator that creates random string chromosomes.
 * The purpose of this class is mainly the support of debugging.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:06 $
 */
public class RandomStringInitializer extends AbstractInitializer {
	
	/** Length of the generated strings.
	 */
	protected int length;
	
	/**
	 * Constructor.
	 */
	public RandomStringInitializer() {
		this( 10 );
	}
	
	/** Constructor.
	 * 
	 * @param stringLength length of the strings to be generated
	 */
	public RandomStringInitializer( int stringLength ) {
		this.length = stringLength;
	}
	
	/**
	 * @see org.evolvica.core.init.AbstractInitializer#createGenotype()
	 */
	protected IGenotype createGenotype() {
		return new StringGene( RandomStringUtils.randomAlphabetic( length ) );
	}

	/**
	 * @return int
	 */
	public int getLength() {
		return length;
	}

	/**
	 * Sets the length.
	 * @param length The length to set
	 */
	public void setLength( int length ) throws PropertyVetoException {
		if ( length > 0 ) {
			this.length = length;
		} else {
			throw new PropertyVetoException( "length size must be > 0",
				new PropertyChangeEvent( this, "length", new Integer( this.length ), new Integer( length ) )
			);
		}
	}

}
