
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:06 $
 * $Revision: 1.9 $
 * $State: Exp $
 * $RCSfile: XMLInitializer.java,v $
 *
 */

package org.evolvica.core.init;

import java.beans.PropertyVetoException;
import java.io.File;
import java.io.FileReader;

import org.evolvica.core.IGenotype;
import org.evolvica.core.IIndividualBuilder;
import org.evolvica.core.IIndividualSet;
import org.evolvica.util.EVLog;

import JSX.ObjIn;

/** This operator initializes an individual set with data from a XML file.
 * The file must contain an individual set which has been serialized with JSX.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.9 $ $Date: 2004/05/26 11:58:06 $
 */
public class XMLInitializer extends AbstractInitializer {
	
	/** File to read data from.
	 */
	protected File file = null;
	
	/**
	 * Constructor.
	 */
	public XMLInitializer() {
		// empty
	}
	
	public XMLInitializer( File file ) {
		this.file = file;
	}
	
	/** Constructor.
	 * 
	 * @param file name of the file to read from
	 */
	public XMLInitializer( String file ) {
		this( new File( file ) );
	}
	
	/** Reads the individual set from the given file.
	 * In case the file does not exist or is corrupt (or whatever),
	 * null is returned.
	 * 
	 * @return set of new individuals
	 */
	public IIndividualSet createIndividualSet() {
		try {
			ObjIn xmlInput = new ObjIn( new FileReader( file ) );
			IIndividualSet set = (IIndividualSet) xmlInput.readObject();
			return set;
		} catch ( Exception ex ) {
			EVLog.error( "initialization from file failed : file " + file + " did not contain an individual collection", ex );
			return null;
		}
	}
	
	/** Dummy method.
	 * 
	 * @return always null
	 */
	public IGenotype createGenotype() {
		// do nothing, we're overloading the createIndividualSet()-method !
		return null;
	}
	
	/**
	 * Returns the file.
	 * 
	 * @return file
	 */
	public File getFile() {
		return file;
	}

	/**
	 * Sets the file.
	 * 
	 * @param file file
	 */
	public void setFile( File file ) {
		this.file = file;
	}

	/**
	 * @see org.evolvica.core.init.AbstractInitializer#setIndividualBuilder(org.evolvica.core.IIndividualBuilder)
	 */
	public void setIndividualBuilder( IIndividualBuilder individualBuilder ) throws PropertyVetoException {
		// overloaded to do simply nothing
		this.individualBuilder = null;
	}

	/**
	 * @see org.evolvica.core.init.AbstractInitializer#setSetSize(int)
	 */
	public void setSetSize( int setSize ) throws PropertyVetoException {
		// overloaded to do simply nothing
		this.setSize = 0;
	}

}
