/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:06 $
 * $Revision: 1.13 $
 * $State: Exp $
 * $RCSfile: AbstractMutator.java,v $
 *
 */
package org.evolvica.core.mutate;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;

import org.evolvica.core.IIndividual;
import org.evolvica.core.IIndividualSet;
import org.evolvica.core.ISetIterator;
import org.evolvica.core.common.AbstractGeneticOperator;
import org.evolvica.engine.IProcessor;
import org.evolvica.util.RandomUtils;

/** Abstract base class for a mutator.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.13 $ $Date: 2004/05/26 11:58:06 $
 */
abstract public class AbstractMutator extends AbstractGeneticOperator implements IProcessor {
	
	/**
	 * Mutation propability.
	 */
	protected float propability = 0.1f;
	
	/**
	 * Constructor.
	 */
	public AbstractMutator() {
		// this.descriptor.setInterface( 1, 1 );
	}
	
	/**
	 * @see org.evolvica.engine.IProcessor#process(java.lang.Object)
	 */
	public Object process( Object input ) {
		for ( ISetIterator it = ((IIndividualSet) input).iterator(); it.hasNext(); ) {
			IIndividual individual = it.next();
			if ( RandomUtils.randomFloat( 1.0f ) <= propability ) {
				mutate( individual );
			}
		}
		return input;
	}

	/**
	 * @see org.evolvica.core.mutate.IMutator#mutate(org.evolvica.core.IIndividual)
	 */
	abstract public void mutate( IIndividual ind );

	/**
	 * Returns the propability.
	 * @return float
	 */
	public float getPropability() {
		return propability;
	}

	/**
	 * Sets the propability.
	 * @param propability The propability to set
	 * @exception PropertyVetoException thrown in case the property cannot be set
	 */
	public void setPropability( float propability ) throws PropertyVetoException {
		if ( propability >= 0.0f && propability <= 1.0f ) {
			this.propability = propability;
		} else {
			throw new PropertyVetoException( "propability must be greater than 0.0 and smaller than 1.0",
				new PropertyChangeEvent( this, "propability", new Float( this.propability ), new Float( propability ) ) );
		}
	}

}
