/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:07 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: RelativeRangeMutator.java,v $
 *
 */
package org.evolvica.core.mutate.Double;

import org.evolvica.util.RandomUtils;

/** Mutation of a double number with a mutation range.
 * This operator will add a random double number within the given range relative to
 * the given value.<br>
 * The mutated value = value + random( - value * range, + value * range ), range in percent
 * 
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:58:07 $
 */
public class RelativeRangeMutator extends DoubleMutator {
	
	/** Mutation range.
	* 
	*/
	protected float range;
	
	/** Constructor.
	* The mutation range is set to 10%.
	* 
	*/
	public RelativeRangeMutator() {
		this( 0.1f );
	}
	
	/** Constructor with mutation range.
	* 
	* @param range mutation range
	*/
	public RelativeRangeMutator( float range ) {
		this.range = range;
	}
	
	/**
	 * @see org.evolvica.core.mutate.Double.DoubleMutator#mutate(double)
	 */
	public double mutate( double value ) {
		double lowerBound = (double) - Math.abs( range * value );
		double upperBound = (double) Math.abs( range * value );
		return value + RandomUtils.randomDouble( lowerBound, upperBound );
	}
	
	/**
	 * Returns the mutation range.
	 * @return the mutation range
	 */
	public float getRange() {
		return range;
	}

	/**
	 * Sets a new mutation range.
	 * @param f new value for the mutation range
	 */
	public void setRange(float f) {
		range = f;
	}

}
