/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:07 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: RelativeFloatMutator.java,v $
 *
 */
package org.evolvica.core.mutate.Float;

import org.evolvica.util.RandomUtils;

/** Mutation of a float number with a mutation range (percentage).
 * This operator will add a random float number within the given range.
 * Example: if the value is 10.0 and the range is 0.1 (10%) the mutated
 * value will be between 9.0 and 11.0.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:07 $
 */
public class RelativeFloatMutator extends FloatMutator {
	
	/**
	* Mutation range.
	*/
	protected float range = 0.1f;
	
	/** Constructor.
	* 
	*/
	public RelativeFloatMutator() {
		this( 0.1f );
	}
	
	/**
	 * Constructor.
	 * 
	 * @param range range
	 */
	public RelativeFloatMutator( float range ) {
		this.range = range;
	}
	
	/**
	 * @see org.evolvica.core.mutate.Float.FloatMutator#mutate(float)
	 */
	public float mutate( float value ) {
		return value + RandomUtils.randomFloat( - Math.abs( range * value ), Math.abs( range * value ) );
	}
	
	/**
	 * @return float
	 */
	public float getRange() {
		return range;
	}

	/**
	 * Sets the range.
	 * @param range The range to set
	 */
	public void setRange( float range ) {
		this.range = range;
	}

}
