/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:08 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: AbsoluteRangeMutator.java,v $
 *
 */
package org.evolvica.core.mutate.Long;

import org.evolvica.util.RandomUtils;

/** Mutation of a long number with a mutation range.
 * This operator will add a random long number within the given range.<br>
 * The mutated value = value +/- random( lowerBound, upperBound )
 * 
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:58:08 $
 */
public class AbsoluteRangeMutator extends LongMutator {
	
	/** Lower mutation bound.
	* 
	*/
	protected long lowerBound = -10;
	
	/** Upper mutation bound.
	* 
	*/
	protected long upperBound = 10;
	
	/** Constructor.
	* 
	*/
	public AbsoluteRangeMutator() {
		this( -10, 10 );
	}
	
	/** Constructor with mutation range.
	* 
	* @param lowerBound lower bound of mutation range
	* @param upperBound upper bound of mutation range
	*/
	public AbsoluteRangeMutator( long lowerBound, long upperBound ) {
		this.lowerBound = lowerBound;
		this.upperBound = upperBound;
	}
	
	/**
	 * @see org.evolvica.core.mutate.Long.LongMutator#mutate(long)
	 */
	public long mutate( long value ) {
		return value + RandomUtils.randomLong( lowerBound, upperBound );
	}
	
	/**
	 * Returns the lower bound.
	 * @return the lower bound
	 */
	public long getLowerBound() {
		return lowerBound;
	}

	/**
	 * Returns the upper bound.
	 * @return the upper bound
	 */
	public long getUpperBound() {
		return upperBound;
	}

	/**
	 * Sets a new lower bound.
	 * @param i new value for the lower bound
	 */
	public void setLowerBound( long i ) {
		lowerBound = i;
	}

	/**
	 * Sets a new upper bound.
	 * @param i new value for the upper bound
	 */
	public void setUpperBound( long i ) {
		upperBound = i;
	}

}
