/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:09 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: StepMutator.java,v $
 *
 */
package org.evolvica.core.mutate.Long;

import org.evolvica.util.RandomUtils;

/** Mutation of an long number with a maximal stepsize.
 * This operator will add a random long number within the range from 0 to a given long number.
 * The given number can be either positive or negative.<br>
 * The mutated value = value + random( 0, stepSize )
 * 
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:58:09 $
 */
public class StepMutator extends LongMutator {
	
	/** Mutation step size.
	* 
	*/
	protected long stepSize;
	
	/** Constructor.
	* The mutation step size is set to 10.
	* 
	*/
	public StepMutator() {
		this( 10 );
	}
	
	/** Constructor with mutation step size.
	* 
	* @param stepSize mutation step size
	*/
	public StepMutator( long stepSize ) {
		this.stepSize = stepSize;
	}
	
	/**
	 * @see org.evolvica.core.mutate.Long.LongMutator#mutate(long)
	 */
	public long mutate( long value ) {
		return value + RandomUtils.randomLong( stepSize );
	}
	
	/**
	 * Returns the step size.
	 * @return step size
	 */
	public long getStepSize() {
		return stepSize;
	}

	/**
	 * Sets a new value for the step size.
	 * @param i new value for the step size
	 */
	public void setStepSize(long i) {
		stepSize = i;
	}

}
