/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:09 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: FloatMutationWrapper.java,v $
 *
 */
 
package org.evolvica.core.mutate.farray;

import org.evolvica.core.FloatArrayGene;
import org.evolvica.core.IIndividual;
import org.evolvica.core.mutate.AbstractMutator;
import org.evolvica.core.mutate.Float.FloatMutator;

/** Wrapper for a float mutator to be used as float array mutator.
 * The wrapped mutator is used for all float values in the array that
 * is to be mutated.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:09 $
 */
public class FloatMutationWrapper extends AbstractMutator {
	
	/**
	 * Underlying mutator.
	 */
	protected FloatMutator mutator = null;
	
	/**
	 * Constructor.
	 */
	public FloatMutationWrapper() {
		this( null );
	}
	
	/**
	 * Constructor.
	 * 
	 * @param mutator float mutator
	 */
	public FloatMutationWrapper( FloatMutator mutator ) {
		this.mutator = mutator;
	}
	
	/**
	 * @see org.evolvica.core.mutate.AbstractMutator#mutate(org.evolvica.core.IIndividual)
	 */
	public void mutate( IIndividual ind ) {
		FloatArrayGene gene = (FloatArrayGene) ind.getGenotype();
		float[] value = gene.getValue();
		for (int i = 0; i < value.length; i++) {
			value[i] = mutator.mutate( value[i] );
		}
	}

	/**
	 * @return FloatMutator
	 */
	public FloatMutator getMutator() {
		return mutator;
	}

	/**
	 * Sets the mutator.
	 * @param mutator The mutator to set
	 */
	public void setMutator( FloatMutator mutator ) {
		this.mutator = mutator;
	}

}
