
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:10 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: RotateMutator.java,v $
 *
 */

package org.evolvica.core.mutate.iarray;

import org.evolvica.util.EVLog;

/** Rotate mutation of an array.
 * This operator will rotate the contents of an array.
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:10 $
 */
public class RotateMutator extends IntegerArrayMutator {

	/** Rotate distance.
	*
	*/
	protected int rotateDistance = Integer.MAX_VALUE;

	/** Rotate distance (percentage value).
	*
	*/
	protected float rotatePercentage = -1.0f;

 	/** Constructor.
	*
	*/
	public RotateMutator() {
		this.rotatePercentage = -1.0f;
	}

	/** Constructor with number of rotated elements.
	*
	* @param percentage percentage value of the number of elements to be rotated
	*/
	public RotateMutator( float percentage ) {
		this.rotatePercentage = percentage;
	}

	/** Constructor with number of rotated elements.
	*
	* @param number number of elements to be rotated
	*/
	public RotateMutator( int number ) {
		this.rotateDistance = number;
	}

	/**
	 * @see org.evolvica.core.mutate.iarray.IntegerArrayMutator#mutate(int[])
	 */
	public int[] mutate( int[] array ) {
		// setup
		int distance; // rotate distance
		int length = array.length;
		// check if rotation distance has been set
		if ( rotateDistance == Integer.MAX_VALUE ) { // no
			// check if percentage value has been set
			if ( rotatePercentage >= 0.0 ) { // yes
				// calculate a valid distance
				distance = (int) Math.round( rotatePercentage * (float) length );
			} else {
				// use half of the length of the array
				distance = length / 2;
			}
		} else { // yes
			if ( rotateDistance < length ) { // check, if rotate distance is valid
				distance = rotateDistance;
			} else {
				distance = length / 2;
				EVLog.warn( "invalid value for rotation distance in RotateIntegerMutation (" + rotateDistance + "), using half of array length (" + distance + ")" );
			}
		}
		// save objects to be replaced
		int[] save = new int[distance];
		System.arraycopy( array, 0, save, 0, distance );
		// copy objects from outside the rotate distance
		int index = 0;
		for ( int i = distance; i < length; i++ ) {
			array[index++] = array[i];
		}
		// copy object from save list back to array
		for ( int i = 0; i < save.length; i++ ) {
			array[index++] = save[i];
		}
		return array;
	}
	
}
