
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:10 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: ScrambleMutator.java,v $
 *
 */

package org.evolvica.core.mutate.iarray;

import org.evolvica.util.RandomUtils;

/** Scramble mutation of an array.
 * This operator will scramble the sequence of randomly selected elements of an
 array.
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:10 $
 */
public class ScrambleMutator extends IntegerArrayMutator {

	/**
	* Scramble number.
	*/
	protected int scrambleNumber = Integer.MAX_VALUE;

	/**
	* Scamble number (percentage).
	*/
	protected float scramblePercentage = -1.0f;
	
 	/** Constructor.
	*
	*/
	public ScrambleMutator() {
		this.scramblePercentage = -1.0f;
	}

	/** Constructor with number of scrambled elements.
	*
	* @param percentage percentage value of the number of elements to be scrambled
	*/
	public ScrambleMutator( float percentage ) {
		this.scramblePercentage = percentage;
	}

	/** Constructor with number of scrambled elements.
	*
	* @param number number of elements to be scrambled
	*/
	public ScrambleMutator( int number ) {
		this.scrambleNumber = number;
	}

	/**
	 * @see org.evolvica.core.mutate.iarray.IntegerArrayMutator#mutate(int[])
	 */
	public int[] mutate( int[] array ) {
		// setup
		int number;
		int length = array.length;
		// check if scramble number have been set
		if ( scrambleNumber == Integer.MAX_VALUE ) { // no
			// check if percentage has been set
			if ( scramblePercentage > 0.0 ) { // yes
				// calculate a valid range
				number = (int) Math.round( scramblePercentage * (float) length );
			} else {
				// use whole length of list
				number = length;
			}
		} else { // yes
			if ( scrambleNumber <= length ) {
				number = scrambleNumber;
			} else {
				number = length;
			}
		}
		int[] pool = RandomUtils.randomIntegerSample( number, length, true );
		int[] al = new int[length];
		for ( int i = 0; i < pool.length; i++ ) {
			al[i] = array[pool[i]];
		}
		RandomUtils.scramble( pool );
		for ( int i = 0; i < pool.length; i++ ) {
			array[pool[i]] = al[i];
		}
		return array;
	}
	
}
