/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:11 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: DiscreteInsertion.java,v $
 *
 */
package org.evolvica.core.population;

import org.evolvica.core.IIndividual;
import org.evolvica.core.IIndividualSet;
import org.evolvica.core.IndividualArray;
import org.evolvica.util.RandomUtils;


/** Discrete reinsertion.
 * This operator works in the same way as {@link UniformInsertion UniformInsertion}
 * with the difference that discrete values for generation gap and reinsertion rate
 * instead of percentage values can be given.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:58:11 $
 */
public class DiscreteInsertion extends AbstractInsertionFilter {
	
	/** Number of individuals to be deleted from the population.
	* 
	*/
	protected int gapSize;
	
	/** Number of individuals to inserted from the reinsertion pool.
	* 
	*/
	protected int rateSize;
	
	/** Constructor.
	* 
	*/
	public DiscreteInsertion() {
		this( 2, 2 );
	}
	
	/** Constructor with insertion parameters.
	* 
	* @param gap number of individuals to be removed
	* @param rate number of individuals to inserted
	*/
	public DiscreteInsertion( int gap, int rate ) {
		this.gapSize = gap;
		this.rateSize = rate;
	}
	
	/** Gets the number of individuals to be removed.
	* 
	* @return number of individuals to be removed
	*/
	public int getGapSize() {
		return gapSize;
	}
	
	/** Sets the number of individuals to be removed.
	* 
	* @param size number of individuals to be removed
	*/
	public void setGapSize( int size ) {
		gapSize = size;
	}
	
	/** Sets the number of individuals to be inserted.
	* 
	* @param size number of individuals to be inserted
	*/
	public void setRateSize( int size ) {
		rateSize = size;
	}
	
	/** Gets the number of individuals to be inserted.
	* 
	* @return number of individuals to be inserted
	*/
	public int getRateSize() {
		return rateSize;
	}
	
	/**
	 * @see org.evolvica.core.population.IInsertionFilter#insert(org.evolvica.core.population.IPopulation, org.evolvica.core.IIndividualSet)
	 */
	public void insert( IPopulation population, IIndividualSet pool ) {
		IIndividual[] popArray = population.toArray();
		int[] survivedFromGate = RandomUtils.randomIntegerSample( popArray.length - gapSize, popArray.length, true );
		IIndividual[] poolArray = pool.toArray();
		int[] survivedFromCollection = RandomUtils.randomIntegerSample( rateSize, popArray.length, true );
		IIndividualSet insertSet = new IndividualArray();
		for ( int i = 0; i < survivedFromGate.length; i++ ) {
			insertSet.add( popArray[survivedFromGate[i]] );
		}
		for ( int i = 0; i < survivedFromCollection.length; i++ ) {
			insertSet.add( popArray[survivedFromCollection[i]] );
		}
		population.clear();
		population.add( insertSet.toArray() );
	}

}
