/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:12 $
 * $Revision: 1.11 $
 * $State: Exp $
 * $RCSfile: EliteInsertion.java,v $
 *
 */
package org.evolvica.core.population;

import java.util.Arrays;

import org.evolvica.core.IIndividual;
import org.evolvica.core.IIndividualComparator;
import org.evolvica.core.IIndividualSet;

/** Elite reinsertion.
 * According to the value of generationGap the worst individuals will
 * be discarded and replaced by the best individuals of the insertion pool
 * according to the value of insertionRate. With respective values a
 * growing or downsizing population can be set up.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.11 $ $Date: 2004/05/26 11:58:12 $
 */
public class EliteInsertion extends AbstractInsertionFilter {
	
	/** Individual comparator used for selecting the best individuals.
	* 
	*/
	protected IIndividualComparator comparator = null;
	
	/** Default constructor.
	* 
	*/
	public EliteInsertion() {
		this( 0.5f, 0.5f, null );
	}
	
	/** Constructor with insertion parameters and comparator.
	* 
	* @param gap generation gap
	* @param rate insertion rate
	* @param comparator individual comparator
	*/
	public EliteInsertion( float gap, float rate, IIndividualComparator comparator ) {
		this.generationGap = gap;
		this.insertionRate = rate;
		this.comparator = comparator;
	}
	
	/**
	 * @return IIndividualComparator
	 */
	public IIndividualComparator getComparator() {
		return comparator;
	}

	/**
	 * Sets the comparator.
	 * @param comparator The comparator to set
	 */
	public void setComparator(IIndividualComparator comparator) {
		this.comparator = comparator;
	}

	/**
	 * @see org.evolvica.core.population.IInsertionFilter#insert(org.evolvica.core.population.IPopulation, org.evolvica.core.IIndividualSet)
	 */
	public void insert( IPopulation population, IIndividualSet pool ) {
		IIndividual[] popArray = population.toArray();
		IIndividual[] poolArray = pool.toArray();
		Arrays.sort( popArray, comparator ); // IMPROVE: check if sorting must really be performed
		Arrays.sort( poolArray, comparator );
		int insertionSize = insertionRateSize( pool ); // to be inserted
		int surviveSize = population.size() - generationGapSize( population ); // to be removed
		for ( int i = surviveSize; i < popArray.length; i++ ) {
			population.remove( popArray[i] );
		}
		for ( int i = 0; i < insertionSize; i++ ) {
			population.add( poolArray[i] );
		}
	}

}
