/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:12 $
 * $Revision: 1.11 $
 * $State: Exp $
 * $RCSfile: ElitistInsertion.java,v $
 *
 */
package org.evolvica.core.population;

import java.util.Arrays;

import org.evolvica.core.IIndividual;
import org.evolvica.core.IIndividualComparator;
import org.evolvica.core.IIndividualSet;

/** Elite reinsertion variant 2.
 * The insertion pool and the population are merged, then the worst
 * individual according to the generation gap and the insertion rate
 * are discarded. The rest is the new population.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.11 $ $Date: 2004/05/26 11:58:12 $
 */
public class ElitistInsertion extends EliteInsertion {
	
	/** Default constructor.
	* 
	*/
	public ElitistInsertion() {
		this( 0.5f, 0.5f, null );
	}
	
	/** Constructor with insertion parameters and comparator.
	* 
	* @param gap generation gap
	* @param rate insertion rate
	* @param comparator individual comparator
	*/
	public ElitistInsertion( float gap, float rate, IIndividualComparator comparator ) {
		this.generationGap = gap;
		this.insertionRate = rate;
		this.comparator = comparator;
	}
	
	/**
	 * @see org.evolvica.core.population.IInsertionFilter#insert(org.evolvica.core.population.IPopulation, org.evolvica.core.IIndividualSet)
	 */
	public void insert( IPopulation population, IIndividualSet pool ) {
		IIndividual[] popArray = population.toArray();
		IIndividual[] poolArray = pool.toArray();
		IIndividual[] insertionArray = new IIndividual[ popArray.length + poolArray.length ];
		System.arraycopy( popArray, 0, insertionArray, 0, popArray.length );
		System.arraycopy( poolArray, 0, insertionArray, popArray.length, poolArray.length );
		Arrays.sort( insertionArray, comparator );
		int insertionSize = population.size() - generationGapSize( population ) + insertionRateSize( pool );
		population.clear();
		for ( int i = 0; i < insertionSize; i++ ) {
			population.add( insertionArray[i] );
		}
		
	}
}
