/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:12 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: IPopulation.java,v $
 *
 */
package org.evolvica.core.population;

import org.evolvica.core.IIndividual;
import org.evolvica.core.IIndividualSet;
import org.evolvica.engine.IProcessor;

/**
 * Interface for a population. A population is also a set of individuals therefore this interface
 * extends IIndividualSet. A population differs from a normal set only in some additional methods
 * concerning replacing individuals and setting insertion filters.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:12 $
 *
 * @since 0.4.0
 */
public interface IPopulation extends IIndividualSet, IProcessor {
	
	/**
	 * Inserts a set of indidividuals into this population.
	 * The individuals from the given set are inserted with applying the populations's
	 * insertion filter. The filter performs the actual insertion procedure and may remove
	 * older individuals from the population. The contents of the population are returned.
	 * 
	 *
	 * @param iset individual set to be inserted
	 * @return contents of the population after applying the insertion filter
	 */
	public IIndividualSet insert( IIndividualSet iset );
	
	/**
	 * Replaces an individual with a substitute individual.
	 *
	 * @param ind individual to be replaced
	 * @param subst substitute for the replaced individual
	 */
	public void replace( IIndividual ind, IIndividual subst );

	/**
	 * Replaces an array of individuals with several substitute indviduals.
	 *
	 * @param indArray array with individuals to be replaced
	 * @param substArray array with individual used as substitution
	 */
	public void replace( IIndividual[] indArray, IIndividual[] substArray );

	/**
	 * Sets an insertion filter to this population.
	 *
	 * @param filter new insertion filter
	 */
	public void setFilter( IInsertionFilter filter );

}
