/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:12 $
 * $Revision: 1.9 $
 * $State: Exp $
 * $RCSfile: SortedPopulation.java,v $
 *
 */
package org.evolvica.core.population;

import java.util.Comparator;
import java.util.TreeSet;

import org.evolvica.core.IIndividual;
import org.evolvica.core.IIndividualComparator;

/** Population based on a tree set that is always sorted.
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.9 $ $Date: 2004/05/26 11:58:12 $
 */
public class SortedPopulation extends AbstractPopulation {
	
	/**
	* Individual comparator.
	*/
	protected IIndividualComparator comparator;
	
	/** Constructor.
	*
	*/
	public SortedPopulation() {
		this.population = new TreeSet( new TreeMapComparator( null ) ); // FIX: add a default comparator
	}
	
	/**
	 * @return IIndividualComparator
	 */
	public IIndividualComparator getComparator() {
		return comparator;
	}

	/**
	 * Sets the comparator.
	 * @param comparator The comparator to set
	 */
	public void setComparator( IIndividualComparator comparator ) {
		this.comparator = comparator;
		if ( population.size() == 0 ) {
			population = new TreeSet( new TreeMapComparator( comparator ) );
		} else {
			TreeSet newPopulation = new TreeSet( new TreeMapComparator( comparator ) );
			newPopulation.addAll( population );
			this.population = newPopulation;
		}
	}

	private class TreeMapComparator implements Comparator {
		
		private IIndividualComparator iComparator;
		
		public TreeMapComparator( IIndividualComparator iComp ) {
			this.iComparator = iComp;
		}
		
		public int compare( Object o1, Object o2 ) {
			if ( o1 == o2 ) {
				return 0;
			} else {
				int result = iComparator.compare( (IIndividual) o1, (IIndividual) o2 );
				if ( result == 0 ) {
					int o1hash = o1.hashCode();
					int o2hash = o2.hashCode();
					if ( o1hash < o2hash ) {
						return -1;
					} else if ( o1hash > o2hash ) {
						return 1;
					} else {
						return 0; // CHANGE: this case should never happen -> throw an exception?
					}
				} else {
					return result;
				}
			}
			
		}
		
	}
	
}
