
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:12 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: UniformInsertion.java,v $
 *
 */

package org.evolvica.core.population;

import org.evolvica.core.IIndividual;
import org.evolvica.core.IIndividualSet;
import org.evolvica.core.IndividualArray;
import org.evolvica.util.RandomUtils;

/** Uniform Reinsertion.
 * This operator will discard random individuals from the population
 * according to the value of generation gap and insert random individuals
 * from the individual collection according to the value of reinsertion rate.
 * With the respective values a growing or shrinking population can be set up.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:12 $
 */
public class UniformInsertion extends AbstractInsertionFilter {
	
	/**
	 * @see org.evolvica.core.population.IInsertionFilter#insert(org.evolvica.core.population.IPopulation, org.evolvica.core.IIndividualSet)
	 */
	public void insert( IPopulation population, IIndividualSet set ) {
		IIndividual[] popArray = population.toArray();
		int[] survivedFromGate = RandomUtils.randomIntegerSample( popArray.length - generationGapSize( population ), popArray.length, true );
		IIndividual[] setArray = set.toArray();
		int[] survivedFromCollection = RandomUtils.randomIntegerSample( this.insertionRateSize( set ), popArray.length, true );
		IIndividualSet insertSet = new IndividualArray();
		for ( int i = 0; i < survivedFromGate.length; i++ ) {
			insertSet.add( popArray[survivedFromGate[i]] );
		}
		for ( int i = 0; i < survivedFromCollection.length; i++ ) {
			insertSet.add( popArray[survivedFromCollection[i]] );
		}
		population.clear();
		population.add( insertSet.toArray() );
	}
	
}
