/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:13 $
 * $Revision: 1.11 $
 * $State: Exp $
 * $RCSfile: AbstractRecombinator.java,v $
 *
 */
package org.evolvica.core.recombine;

import org.evolvica.core.*;
import org.evolvica.core.common.AbstractGeneticOperator;
import org.evolvica.engine.IProcessor;

/** Abstract base class for a recombinator.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.11 $ $Date: 2004/05/26 11:58:13 $
 */
abstract public class AbstractRecombinator extends AbstractGeneticOperator implements IProcessor {
	
	/**
	 * Individual builder to be used.
	 */
	protected IIndividualBuilder builder = new DefaultIndividualBuilder();
	
	/**
	 * Constructor.
	 */
	public AbstractRecombinator() {
		// this.descriptor.setInterface( 1, 1 );
	}
	
	/**
	 * @see org.evolvica.engine.IProcessor#process(java.lang.Object)
	 */
	public Object process( Object input ) {
		int required = getRequiredParents();
		IIndividual[] parents = new IIndividual[required];
		IIndividualSet childrenSet = new IndividualArray();
		ISetIterator it = ((IIndividualSet) input).iterator();
		boolean breakCondition = false;
		while( it.hasNext() ) {
			for ( int i = 0; i < required; i++ ) {
				if ( it.hasNext() ) {
					parents[i] = it.next();
				} else {
					breakCondition = true;
					break;
				}
			}
			if ( breakCondition == false ) {
				childrenSet.add( recombine( parents ) );
			} else {
				break;
			}
		}
		return childrenSet;
	}
	
	/**
	 * Performes the recombination.
	 * 
	 * @param parents the parent individuals
	 * @return array of child individuals
	 */
	abstract public IIndividual[] recombine( IIndividual[] parents );
	
	/**
	 * Returns the number of required parents for a valid recombination.
	 * 
	 * @return number of parents
	 */
	abstract public int getRequiredParents();
	
	/**
	 * Return the number of produced children.
	 * 
	 * @return number of children
	 */
	abstract public int getProducedChildren();
	
	/**
	 * Returns the builder.
	 * 
	 * @return individual builder
	 */
	public IIndividualBuilder getBuilder() {
		return builder;
	}

	/**
	 * Sets the individual builder.
	 * 
	 * @param builder individual builder
	 */
	public void setBuilder( IIndividualBuilder builder ) {
		this.builder = builder;
	}

}
