/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:13 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: DoubleRecombinator.java,v $
 *
 */

package org.evolvica.core.recombine.Double;

import org.evolvica.core.DoubleGene;
import org.evolvica.core.IIndividual;
import org.evolvica.core.recombine.AbstractRecombinator;

/**
 * Abstract base class for a double recombinator.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:13 $
 */
abstract public class DoubleRecombinator extends AbstractRecombinator {

	/** Constructor.
	 * 
	 */
	public DoubleRecombinator() {
		// nothing to do here
	}
	
	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#recombine(org.evolvica.core.IIndividual[])
	 */
	public IIndividual[] recombine( IIndividual[] parents ) {
		double[] parentGenes = new double[parents.length];
		for ( int i = 0; i < parents.length; i++ ) {
			parentGenes[i] = ((DoubleGene) parents[i].getGenotype()).getValue();	
		}
		double[] childGenes = recombine( parentGenes );
		IIndividual[] children = new IIndividual[childGenes.length];
		for ( int i = 0; i < children.length; i++ ) {
			children[i] = builder.newIndividual();
			children[i].setGenotype( new DoubleGene( childGenes[i] ) );
		}
		return children;
	}
	
	/** Recombines double numbers.
	 * @param parents array of double values from parents
	 * @return array of double values for the children
	 */
	abstract double[] recombine( double[] parents );
	
}
