/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:14 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: DiscreteRecombinator.java,v $
 *
 */
package org.evolvica.core.recombine.Int;

import org.evolvica.util.RandomUtils;

/** Discrete recombination between two integers.
 * This operator creates a single chromosome by choosing randomly one of the parent chromosomes.
 * 
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:58:14 $
 */
public class DiscreteRecombinator extends IntegerRecombinator {
	
	/**
	 * @see org.evolvica.core.recombine.Int.IntegerRecombinator#recombine(int[])
	 */
	public int[] recombine( int[] parents ) {
		int a = RandomUtils.randomInt( 2 );
		int child = parents[0] * a + parents[1] * ( 1 - a );
		// this is the variant for a size N array
		// return cra[rand.nextInt( cra.length )]
		return new int[] { child };
	}

	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#getProducedChildren()
	 */
	public int getProducedChildren() {
		return 1;
	}

	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#getRequiredParents()
	 */
	public int getRequiredParents() {
		return 2;
	}

}
