
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:14 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: ExtendedLineRecombinator.java,v $
 *
 */

package org.evolvica.core.recombine.Int;

import org.evolvica.util.RandomUtils;

/** Extended line recombination for integer numbers.
 * This operator produces one child out of two parents.
 * 
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:14 $
 */
public class ExtendedLineRecombinator extends LineRecombinator {
	
	
	/**
	 * Precision.
	 */
	protected float precision = 5.0f; // good default value
	
	/**
	 * Step size.
	 */
	protected float stepsize = 0.1f; // good default value
	
	/**
	 * Direction.
	 */
	protected float directionProp = 0.5f;
	
	/**
	 * Domain.
	 */
	protected float domain = 10.0f;
	
	/**
	 * Constructor.
	 */
	public ExtendedLineRecombinator() {
		this( 5.0f, 0.1f, 0.5f, 10.0f );
	}
	
	/**
	 * Constructor.
	 * 
	 * @param precision precision
	 * @param stepsize stepsize
	 * @param directionProp direction
	 * @param domain domain
	 */
	public ExtendedLineRecombinator( float precision, float stepsize, float directionProp, float domain ) {
		this.precision = precision;
		this.stepsize = stepsize;
		this.directionProp = directionProp;
		this.domain = domain;
	}
	
	/**
	 * @see org.evolvica.core.recombine.Int.IntegerRecombinator#recombine(int[])
	 */
	public int[] recombine( int[] parents ) {
		float parent0 = (float) parents[0];
		float parent1 = (float) parents[1];
		// setup calculation
		float a = (float) Math.pow( 2, ( -precision * RandomUtils.randomFloat( 1.0f ) ) ); // rand = u = [0,1] gleichverteilt
		float ri = domain * stepsize; // this must be done
		float s;
		if ( ( RandomUtils.randomFloat( 2.0f ) * ( directionProp * 2.0f ) ) < 0.0f ) {
			s = -1.0f;
		} else {
			s = 1.0f;
		}
		// final calculation
		int child = Math.round( parent0 + s * ri * a * ( (parent1 - parent0) / ( (float) Math.abs( parent0 - parent1) ) ) );
		return new int[] { child };
	}
	
}
