/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:14 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: IntegerRecombinator.java,v $
 *
 */

package org.evolvica.core.recombine.Int;

import org.evolvica.core.IIndividual;
import org.evolvica.core.IntegerGene;
import org.evolvica.core.recombine.AbstractRecombinator;

/**
 * Abstract base class for integer recombinators.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:14 $
 */
abstract public class IntegerRecombinator extends AbstractRecombinator {
	
	/** Constructor.
	 * 
	 */
	public IntegerRecombinator() {
		// nothing to do here
	}
	
	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#recombine(org.evolvica.core.IIndividual[])
	 */
	public IIndividual[] recombine( IIndividual[] parents ) {
		int[] parentGenes = new int[parents.length];
		for ( int i = 0; i < parents.length; i++ ) {
			parentGenes[i] = ((IntegerGene) parents[i].getGenotype()).getValue();	
		}
		int[] childGenes = recombine( parentGenes );
		IIndividual[] children = new IIndividual[childGenes.length];
		for ( int i = 0; i < children.length; i++ ) {
			children[i] = builder.newIndividual();
			children[i].setGenotype( new IntegerGene( childGenes[i] ) );
		}
		return children;
	}
	
	/** Recombines integer numbers.
	 * @param parents array of integer values from parents
	 * @return array of integer values for the children
	 */
	abstract public int[] recombine( int[] parents );
	
}
