/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:14 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: LineRecombinator.java,v $
 *
 */

package org.evolvica.core.recombine.Int;

import org.evolvica.util.RandomUtils;

/** Line recombination for integer numbers.
 * This operator produces one child out of two parents.
 * 
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:58:14 $
 */
public class LineRecombinator extends IntegerRecombinator {
	
	/** Scaling factor.
	* The default value is 0.25.
	*/
	protected float scaling;
	
	/** Constructor.
	*/
	public LineRecombinator() {
		this( 0.25f );
	}
	
	/** Constructor with scaling factor.
	* 
	* @param scaling scaling factor
	*/
	public LineRecombinator( float scaling ) {
		this.scaling = scaling;
	}
	
	/**
	 * @see org.evolvica.core.recombine.Int.IntegerRecombinator#recombine(int[])
	 */
	public int[] recombine( int[] parents ) {
		float d = RandomUtils.randomFloat( 2.0f * scaling + 1 ) - scaling;
		int child = Math.round( ( (float) parents[0] * d ) + ( (float) parents[1] * ( 1.0f - d ) ) );
		return new int[] { child };
	}

	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#getProducedChildren()
	 */
	public int getProducedChildren() {
		return 1;
	}

	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#getRequiredParents()
	 */
	public int getRequiredParents() {
		return 2;
	}

}
