/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:14 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: BinaryRecombinator.java,v $
 * 
 */
package org.evolvica.core.recombine.binary;

import org.evolvica.core.BinaryGene;
import org.evolvica.core.IIndividual;
import org.evolvica.core.recombine.AbstractRecombinator;
import org.evolvica.util.BitVector;

/** Abstract base class for a binary recombinator.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:58:14 $
 */
abstract public class BinaryRecombinator extends AbstractRecombinator {

	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#recombine(org.evolvica.core.IIndividual[])
	 */
	public IIndividual[] recombine( IIndividual[] parents ) {
		BitVector[] parentGenes = new BitVector[parents.length];
		for ( int i = 0; i < parents.length; i++ ) {
			parentGenes[i] = ((BinaryGene) parents[i].getGenotype()).getValue();	
		}
		BitVector[] childGenes = recombine( parentGenes );
		IIndividual[] children = new IIndividual[childGenes.length];
		for ( int i = 0; i < children.length; i++ ) {
			children[i] = builder.newIndividual();
			children[i].setGenotype( new BinaryGene( childGenes[i] ) );
		}
		return children;
	}
	
	/**
	 * Recombines two bitsets.
	 * 
	 * @param parents parent bitsets
	 * @return child bitsets
	 */
	abstract protected BitVector[] recombine( BitVector[] parents );
	
}
