/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:14 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: MultipointCrossover.java,v $
 * 
 */
package org.evolvica.core.recombine.binary;

import java.util.Random;

import org.evolvica.util.BitVector;
import org.evolvica.util.RandomUtils;

/** Multipoint Crossover for binary strings.
 * This operator produces two children from two parents.
 * For the number of intersections na obtains:
 * 0 &lt; na &lt; width( BinaryString ).
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:14 $
 */
public class MultipointCrossover extends BinaryRecombinator {

	/** Random generator.
	*
	*/
	protected Random random = new Random();

	/** Number of intersections of the binary string.
	*
	*/
	protected int intersections;
	
	/**
	 * Constructor.
	 */
	public MultipointCrossover() {
		this( 1 );
	}
	
	/** Constructor.
	*
	* @param intersections number of intersections
	*/
	public MultipointCrossover( int intersections ) {
		this.intersections = intersections;
	}

	/**
	 * @see org.evolvica.core.recombine.binary.BinaryRecombinator#recombine(org.evolvica.util.BitVector[])
	 */
	protected BitVector[] recombine( BitVector[] parents ) {
		int size;
		if ( parents[0].length() > parents[1].length() ) {
			size = parents[0].length();
		} else {
			size = parents[1].length();
		}
		BitVector child0 = new BitVector( size ); // the offspring BitSets
		BitVector child1 = new BitVector( size );
		int[] intersectionPoints = RandomUtils.randomIntegerSample( intersections, size );
		boolean toggleBit = true; // true means put bits from parent0 -> child0, false from parent0 -> child1
		int nextCut = 0;
		for( int i = 0; i < size; i++ ) {
			if ( toggleBit == true ) {
				child0.set( i, parents[0].get( i ) );
				child1.set( i, parents[1].get( i ) );
			} else {
				child0.set( i, parents[1].get( i ) );
				child1.set( i, parents[0].get( i ) );
			}
			if ( ( nextCut < intersections ) && ( i == intersectionPoints[nextCut] ) ) {
				if ( toggleBit == true ) {
					toggleBit = false;
				} else {
					toggleBit = true;
				}
				nextCut++;
			}
		}
		return new BitVector[] { child0, child1 };
	}
	
	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#getProducedChildren()
	 */
	public int getProducedChildren() {
		return 2;
	}

	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#getRequiredParents()
	 */
	public int getRequiredParents() {
		return 1;
	}

}
