/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:15 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: FloatArrayRecombinator.java,v $
 * 
 */
package org.evolvica.core.recombine.farray;

import org.evolvica.core.FloatArrayGene;
import org.evolvica.core.IIndividual;
import org.evolvica.core.recombine.AbstractRecombinator;

/** Abstract base class for a float array recombinator.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:15 $
 */
abstract public class FloatArrayRecombinator extends AbstractRecombinator {

	/** Constructor.
	 * 
	 */
	public FloatArrayRecombinator() {
		// nothing to do here
	}
	
	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#recombine(org.evolvica.core.IIndividual[])
	 */
	public IIndividual[] recombine( IIndividual[] parents ) {
		float[][] parentGenes = new float[parents.length][];
		for ( int i = 0; i < parents.length; i++ ) {
			parentGenes[i] = ((FloatArrayGene) parents[i].getGenotype()).getValue();	
		}
		float[][] childGenes = recombine( parentGenes );
		IIndividual[] children = new IIndividual[childGenes.length];
		for ( int i = 0; i < children.length; i++ ) {
			children[i] = builder.newIndividual();
			children[i].setGenotype( new FloatArrayGene( childGenes[i] ) );
		}
		return children;
	}
	
	/** Recombines float arrays.
	 * @param parents array of float arrays from parents
	 * @return array of float arrays for the children
	 */
	protected abstract float[][] recombine( float[][] parents );
	
}
