
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:16 $
 * $Revision: 1.9 $
 * $State: Exp $
 * $RCSfile: MultipointCrossover.java,v $
 *
 */

package org.evolvica.core.recombine.iarray;

import java.util.Random;

import org.evolvica.util.RandomUtils;

/** Multipoint Crossover for integer arrays.
 * This operator produces two children from two parents. For the number of intersection na obtains:
 * 0 < na < width( list ).
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.9 $ $Date: 2004/05/26 11:58:16 $
 */
public class MultipointCrossover extends IntegerArrayRecombinator {

	/** Random generator.
	*/
	protected Random random;

	/** Number of intersections.
	*/
	protected int intersections = 1;
	
	/** Intersection points.
	*/
	protected int[] intersectionPoints = null;
	
	/** Constructor.
	 * The number of intersections is set to 1.
	 *
	 */
	public MultipointCrossover() {
		this( 1 );
	}
	
	/** Constructor.
	*
	* @param intersections number of intersections of the lists
	*/
	public MultipointCrossover( int intersections ) {
		this.intersections = intersections;
	}

	/** Constructor with intersection points.
	* This constructor should only be used for debugging purposes.
	*
	* @param intersectionPoints array with intersection points
	*/
	public MultipointCrossover( int[] intersectionPoints ) {
		this.intersectionPoints = intersectionPoints;
	}

	/**
	 * @see org.evolvica.core.recombine.iarray.IntegerArrayRecombinator#recombine(int[][])
	 */
	protected int[][] recombine( int[][] parents ) {
		int size;
		if ( parents[0].length > parents[1].length ) {
			size = parents[0].length;
		} else {
			size = parents[1].length;
		}
		int[] child0 = new int[size];
		int[] child1 = new int[size];
		int[] isPoints; // intersection points
		if ( intersectionPoints == null ) {
			isPoints = RandomUtils.randomIntegerSample( intersections, size );
		} else {
			isPoints = intersectionPoints;
		}
		boolean toggleBit = true; // true means put elements from parent0 -> child0, false from parent0 -> child1
		int nextCut = 0;
		for( int i = 0; i < size; i++ ) {
			if ( toggleBit == true ) {
				child0[i] = parents[0][i];
				child1[i] = parents[1][i];
			} else {
				child0[i] = parents[1][i];
				child1[i] = parents[0][i];
			}
			if ( ( nextCut < intersections ) && (i == isPoints[nextCut]) ) {
				if ( toggleBit == true ) {
					toggleBit = false;
				} else {
					toggleBit = true;
				}
				nextCut++;
			}
		}
		return new int[][] { child0, child1 };
	}

	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#getProducedChildren()
	 */
	public int getProducedChildren() {
		return 2;
	}

	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#getRequiredParents()
	 */
	public int getRequiredParents() {
		return 2;
	}

}
