
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:16 $
 * $Revision: 1.10 $
 * $State: Exp $
 * $RCSfile: UniformOrderBasedCrossover.java,v $
 *
 */

package org.evolvica.core.recombine.iarray;

import java.util.HashSet;
import java.util.LinkedList;

import org.evolvica.util.BitVector;

/** Uniform Order Based Crossover (UOBX) for arrays.
 * This operator requires two parents and produces two descendants.
 * 
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.10 $ $Date: 2004/05/26 11:58:16 $
 */
public class UniformOrderBasedCrossover extends IntegerArrayRecombinator {
	
	/** Exchange bit vector.
	* 
	*/
	protected BitVector bitvector = null;
	
	/** Constructor.
	* 
	*/
	public UniformOrderBasedCrossover() {
		this( null );
	}
	
	/** Constructor with bit vector for exchange.
	* This constructor should only be used for debugging purposes.
	*
	* @param bitvector bit vector for exchange
	*/
	public UniformOrderBasedCrossover( BitVector bitvector ) {
		this.bitvector = bitvector;
	}

	/**
	 * @see org.evolvica.core.recombine.iarray.IntegerArrayRecombinator#recombine(int[][])
	 */
	protected int[][] recombine( int[][] parents ) {
		// look for size of chromosomes and create bitstring
		int size = parents[0].length;
		BitVector bv;
		if ( bitvector == null ) {
			bv = new BitVector( size );
			bv.randomize();
		} else {
			bv = bitvector;
		}
		// create children
		int[] child0 = new int[size];
		int[] child1 = new int[size];
		// start the algorithm
		// create lookup tables with already contained objects
		HashSet set0 = new HashSet();
		HashSet set1 = new HashSet();
		// lists contained indexes with free slots
		LinkedList indexList0 = new LinkedList();
		LinkedList indexList1 = new LinkedList();
		// assign objects according to bits in bit string
		// (1) parent 0 -> child 0; (0) parent 1 -> child 1
		for ( int i = 0; i < size; i++ ) {
			if ( bv.get( i ) == true ) { // case (1)
				int o = parents[0][i];
				child0[i] = o;
				set0.add( new Integer( o ) );
				indexList1.add( new Integer( i ) );
			} else { // case (0)
				int o = parents[1][i];
				child1[i] = o;
				set1.add( new Integer( o ) );
				indexList0.add( new Integer( i ) );
			}
		}
		// fill gaps in both children
		for ( int i = 0, j = 0; i < size; i++ ) { // handle child 0
			int o = parents[1][i];
			if ( ! set0.contains( new Integer( o ) ) ) {
				int index = ((Integer) indexList0.get( j++ )).intValue();
				child0[index] = o;
			}
		}
		for ( int i = 0, j = 0; i < size; i++ ) { // handle child 1
			int o = parents[0][i];
			if ( ! set1.contains( new Integer( o ) ) ) {
				int index = ((Integer) indexList1.get( j++ )).intValue();
				child1[index] = o;
			}
		}
		return new int[][] { child0, child1 };
	}
	
	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#getProducedChildren()
	 */
	public int getProducedChildren() {
		return 2;
	}

	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#getRequiredParents()
	 */
	public int getRequiredParents() {
		return 2;
	}

}
