/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:16 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: MultipointCrossover.java,v $
 *
 */
package org.evolvica.core.recombine.string;

import java.util.Random;

import org.evolvica.util.RandomUtils;

/** Multipoint Crossover for strings.
 * This operator generates two children out of two parents.
 * For the number of intersections na obtains:
 * 0 < na < width( string ).
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:58:16 $
 */
public class MultipointCrossover extends StringRecombinator {

	/** Random generator.
	*/
	protected Random random = new Random();

	/** Number of intersections of the strings.
	*/
	protected int intersections;

	/**
	 * Constructor.
	 */
	public MultipointCrossover() {
		this( 1 );
	}
	
	/** Constructor.
	*
	* @param intersections number of intersections
	*/
	public MultipointCrossover( int intersections ) {
		this.intersections = intersections;
	}

	/**
	 * @see org.evolvica.core.recombine.string.StringRecombinator#recombine(java.lang.String[])
	 */
	public String[] recombine( String[] parents ) {
		int size;
		if ( parents[0].length() > parents[1].length() ) {
			size = parents[1].length();
		} else {
			size = parents[0].length();
		}
		StringBuffer child0 = new StringBuffer( size ); // the offspring Strings
		StringBuffer child1 = new StringBuffer( size );
		int[] intersectionPoints = RandomUtils.randomIntegerSample( intersections, size );
		boolean toggleBit = true; // true means put characters from parent0 -> child0, false from parent0 -> child1
		int nextCut = 0;
		for( int i = 0; i < size; i++ ) {
			if ( toggleBit == true ) {
				child0.append( parents[0].charAt( i ) );
				child1.append( parents[1].charAt( i ) );
			} else {
				child0.append( parents[1].charAt( i ) );
				child1.append( parents[0].charAt( i ) );
			}
			if ( (nextCut < intersections) && (i == intersectionPoints[nextCut]) ) {
				if ( toggleBit == true ) {
					toggleBit = false;
				} else {
					toggleBit = true;
				}
				nextCut++;
			}
		}
		return new String[] { child0.toString(), child1.toString() };
	}

	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#getProducedChildren()
	 */
	public int getProducedChildren() {
		return 2;
	}

	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#getRequiredParents()
	 */
	public int getRequiredParents() {
		return 2;
	}

}
