/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:17 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: StringRecombinator.java,v $
 *
 */

package org.evolvica.core.recombine.string;

import org.evolvica.core.IIndividual;
import org.evolvica.core.StringGene;
import org.evolvica.core.recombine.AbstractRecombinator;

/**
 * Abstract base class for a string recombinator.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:17 $
 */
abstract public class StringRecombinator extends AbstractRecombinator {

	/** Constructor.
	 * 
	 */
	public StringRecombinator() {
		// nothing to do here
	}
	
	/**
	 * @see org.evolvica.core.recombine.AbstractRecombinator#recombine(org.evolvica.core.IIndividual[])
	 */
	public IIndividual[] recombine( IIndividual[] parents ) {
		String[] parentGenes = new String[parents.length];
		for ( int i = 0; i < parents.length; i++ ) {
			parentGenes[i] = ((StringGene) parents[i].getGenotype()).getValue();	
		}
		String[] childGenes = recombine( parentGenes );
		IIndividual[] children = new IIndividual[childGenes.length];
		for ( int i = 0; i < children.length; i++ ) {
			children[i] = builder.newIndividual();
			children[i].setGenotype( new StringGene( childGenes[i] ) );
		}
		return children;
	}
	
	/** Recombines strings.
	 * @param parents array of strings from parents
	 * @return array of strings for the children
	 */
	abstract public String[] recombine( String[] parents );
	
}
