/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:17 $
 * $Revision: 1.12 $
 * $State: Exp $
 * $RCSfile: AbstractSelector.java,v $
 *
 */
package org.evolvica.core.select;

import org.evolvica.core.IIndividual;
import org.evolvica.core.IIndividualSet;
import org.evolvica.core.IndividualArray;
import org.evolvica.core.common.AbstractGeneticOperator;
import org.evolvica.engine.IProcessor;

/** Abstract base class for a selector.
 * A selector selects a number of individuals out of a selection pool.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.12 $ $Date: 2004/05/26 11:58:17 $
 */
abstract public class AbstractSelector extends AbstractGeneticOperator implements IProcessor {
	
	/**
	 * Flag to allow multiple selections.
	 * This is unused at the moment.
	 */
	protected boolean allowMultipleSelection = true;
	
	/**
	 * Number of individuals selected.
	 */
	protected int numberPerSelection = -1;
	
	/**
	 * Constructor.
	 */
	public AbstractSelector() {
		this( -1 );
	}
	
	/**
	 * Constructor.
	 * 
	 * @param numberPerSelection number of individuals selected
	 */
	public AbstractSelector( int numberPerSelection ) {
		this.numberPerSelection = numberPerSelection;
		// this.descriptor.setInterface( 1, 1 );
	}
	
	/**
	 * @see org.evolvica.engine.IProcessor#process(java.lang.Object)
	 */
	public Object process( Object input ) {
		IIndividualSet set = new IndividualArray();
		set.add( select( ((IIndividualSet) input).toArray() ) );
		return set;
	}
	
	/**
	 * Selects individuals out of a given selection pool.
	 * 
	 * @param pool selection pool
	 * @return selected individuals
	 */
	abstract public IIndividual[] select( IIndividual[] pool );
	
	/**
	 * Returns the number per selection.
	 * 
	 * @return number per selection
	 */
	public int getNumberPerSelection() {
		return numberPerSelection;
	}

	/**
	 * Sets the number per selection.
	 * 
	 * @param i number per selection
	 */
	public void setNumberPerSelection( int i ) {
		numberPerSelection = i;
	}

}
