
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:17 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: FitnessTruncationSelector.java,v $
 *
 */

package org.evolvica.core.select;

import org.evolvica.core.IFitness;
import org.evolvica.core.IFitnessComparator;
import org.evolvica.core.IIndividual;

/** Truncation selection by fitness value.
 * This operator selects all individuals that have a better fitness than the given threshold.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:17 $
 */
public class FitnessTruncationSelector extends TruncationSelector {
	
	/** Threshold fitness value.
	* 
	*/
	protected IFitness threshold = null;
	
	/**
	* Fitness comparator.
	*/
	protected IFitnessComparator comparator = null;
	
	/**
	 * Comparator.
	 */
	public FitnessTruncationSelector() {
		this( -1, null, null );
	}
	
	/** Constructor with number per selection and threshold.
	* 
	* @param numberPerSelection number of individuals per selection
	* @param threshold threshold fitness value
	* @param comparator fitness comparator
	*/
	public FitnessTruncationSelector( int numberPerSelection, IFitness threshold, IFitnessComparator comparator ) {
		super( numberPerSelection );
		this.threshold = threshold;
		this.comparator = comparator;
	}
	
	/**
	 * @see org.evolvica.core.select.TruncationSelector#threshold(org.evolvica.core.IIndividual[])
	 */
	protected int threshold( IIndividual[] pool ) {
		int index = 0;
		for ( int i = 0; i < pool.length; i++ ) {
			if ( comparator.compare( pool[i].getFitness(), threshold ) < 0 ) {
				++index;
			}
		}
		return index;
	}

	/** Sets the threshold.
	* 
	* @param threshold new threshold
	*/
	public void setThreshold( IFitness threshold ) {
		this.threshold = threshold;
	}
	
	/** Gets the threshold.
	* 
	* @return current threshold
	*/
	public IFitness getThreshold() {
		return threshold;
	}
	
	/**
	 * Gets the fitness comparator.
	 * 
	 * @return fitness comparator.
	 */
	public IFitnessComparator getComparator() {
		return comparator;
	}

	/**
	 * Sets the fitness comparator.
	 * 
	 * @param comparator fitness comparator
	 */
	public void setComparator( IFitnessComparator comparator ) {
		this.comparator = comparator;
	}

}
