
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:17 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: PoolSizeTruncationSelector.java,v $
 *
 */

package org.evolvica.core.select;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;

import org.evolvica.core.IIndividual;


/** Truncation selection by fixed percentage.
 * This operator selects all individuals that are ranked in their
 * population between 0% and <threshold> or in other
 * words it selects the best <threshold>% of all individuals.
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:17 $
 */
public class PoolSizeTruncationSelector extends TruncationSelector {

	/** Threshold value.
	*
	*/
	protected float threshold = 0.5f;
	
	/** Constructor.
	* The threshold is set to 0.5.
	*
	*/
	public PoolSizeTruncationSelector() {
		this( -1, 0.5f );
	}

	/** Constructor number per selection and threshold.
	*
	* @param numberPerSelection number how many individuals are selected
	* @param threshold threshold percentage
	*/
	public PoolSizeTruncationSelector( int numberPerSelection, float threshold ) {
		super( numberPerSelection );
		this.threshold = threshold;
	}

	/** Sets the threshold.
	*
	* @param threshold new threshold
	* @exception PropertyVetoException thrown in case the property cannot be set
	*/
	public void setThreshold( float threshold ) throws PropertyVetoException {
		if ( threshold < 0.0 || threshold > 1.0 ) {
			throw new PropertyVetoException(
				"threshold value must be 0.0 < t < 1.0",
				new PropertyChangeEvent( this, "threshold", new Float( threshold ), new Float( this.threshold ) )
			);
		} else {
			this.threshold = threshold;
		}
	}

	/** Gets the threshold.
	*
	* @return current threshold
	*/
	public float getThreshold() {
		return threshold;
	}
	
	/**
	 * @see org.evolvica.core.select.TruncationSelector#threshold(org.evolvica.core.IIndividual[])
	 */
	protected int threshold( IIndividual[] pool ) {
		return (int) ( (float) pool.length * threshold );
	}
	
}
