
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:17 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: RandomSelector.java,v $
 *
 */

package org.evolvica.core.select;

import org.evolvica.core.IIndividual;
import org.evolvica.util.RandomUtils;

/** Random Selection.
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:17 $
 */
public class RandomSelector extends AbstractSelector {
	
	/**
	 * Constructor.
	 */
	public RandomSelector() {
		this( -1 );
	}
	
	/** Constructor with number per selection.
	*
	* @param numberPerSelection number how many individuals are selected
	*/
	public RandomSelector( int numberPerSelection ) {
		super( numberPerSelection );
	}

	/**
	 * @see org.evolvica.core.select.AbstractSelector#select(org.evolvica.core.IIndividual[])
	 */
	public IIndividual[] select( IIndividual[] pool ) {
		IIndividual[] rval = null;
		if ( numberPerSelection < 0 ) {
			rval = new IIndividual[pool.length];
		} else {
			rval = new IIndividual[numberPerSelection];
		}
		for ( int i = 0; i < rval.length; i++ ) {
			rval[i] = pool[RandomUtils.randomInt( pool.length )];
		}
		return rval;
	}

}
