
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:17 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: RouletteWheelSelector.java,v $
 *
 */

package org.evolvica.core.select;

import org.evolvica.core.IIndividual;
import org.evolvica.util.RandomUtils;


/** Roulette Wheel Selection.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:17 $
 */
public class RouletteWheelSelector extends AbstractSelector {
	
	/** Array with line segments.
	* 
	*/
	protected float[] bounds= null;
	
	/**
	 * Constructor.
	 */
	public RouletteWheelSelector() {
		this( -1 );
	}
	
	/** Constructor with number per selection.
	* 
	* @param numberPerSelection number of individuals per selection
	*/
	public RouletteWheelSelector( int numberPerSelection ) {
		super( numberPerSelection );
	}
	
	/**
	 * @see org.evolvica.core.select.AbstractSelector#select(org.evolvica.core.IIndividual[])
	 */
	public IIndividual[] select(IIndividual[] pool) {
		setupLineArray( pool );
		IIndividual[] rval = null;
		if ( numberPerSelection < 0 ) {
			rval = new IIndividual[pool.length];
		} else {
			rval = new IIndividual[numberPerSelection];
		}
		for ( int i = 0; i < rval.length; i++ ) {
			rval[i] = pool[spinWheel( pool )];
		}
		return rval;
	}

	/**
	 * Sets up the array of selection propabilities.
	 * 
	 * @param pool selection pool
	 */
	protected void setupLineArray( IIndividual[] pool ) {
		bounds = new float[pool.length];
		float total = totalFitness( pool );
		float bound = (float) 0.0;
		for ( int i = 0; i < pool.length; i++ ) {
			float fitness = ((Float) pool[i].getFitness().getValue()).floatValue() / total; // calculate line segment
			bound = bound + fitness;
			bounds[i] = bound;
		}
	}
	
	/** Spins the roulette wheel and returns the diced number.
	* 
	* @param pool selection pool
	* @return number generated by the wheel
	*/
	protected int spinWheel( IIndividual[] pool ) {
		int position = pool.length;
		float vec;
		float ins = RandomUtils.randomFloat( 1.0f );
		for ( int i = 0; i < pool.length; i++ ) {
			vec = bounds[i];
			if ( vec > ins ) {
				position = i;
				break;
			}
		}
		return position;
		
	}

	/** Helper for adding all fitness values of the collection.
	* 
	* @param pool selection pool
	* @return sum of all fitness values
	*/
	protected float totalFitness( IIndividual[] pool ) {
		float fitness = 0.0f;
		for ( int i = 0; i < pool.length; i++ ) {
			fitness = fitness + ((Float) pool[i].getFitness().getValue()).floatValue();
		}
		return fitness;
	}

}
