
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:17 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: ScoreTruncationSelector.java,v $
 *
 */

package org.evolvica.core.select;

import org.evolvica.core.IIndividual;
import org.evolvica.core.IScore;
import org.evolvica.core.IScoreComparator;

/** Truncation selection by score.
 * This operator selects all individuals that have a better score
 * than the given threshold.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:17 $
 */
public class ScoreTruncationSelector extends TruncationSelector {
	
	/** Score threshold.
	* 
	*/
	protected IScore threshold = null;
	
	/**
	* Score comparator.
	*/
	protected IScoreComparator comparator = null;
	
	/** Constructor.
	* 
	*/
	public ScoreTruncationSelector() {
		this( -1, null, null );
	}
	
	/** Constructor with number per selection and threshold.
	* 
	* @param numberPerSelection number of individuals per selection
	* @param threshold threshold score
	* @param comparator score comparator
	*/
	public ScoreTruncationSelector( int numberPerSelection, IScore threshold, IScoreComparator comparator ) {
		super( numberPerSelection );
		this.threshold = threshold;
		this.comparator = comparator;
	}
	
	/**
	 * @see org.evolvica.core.select.TruncationSelector#threshold(org.evolvica.core.IIndividual[])
	 */
	protected int threshold( IIndividual[] pool ) {
		int index = 0;
		for ( int i = 0; i < pool.length; i++ ) {
			if ( comparator.compare( pool[i].getScore(), threshold ) < 0 ) {
				++index;
			}
		}
		return index;
	}
	
	/** Sets the threshold.
	* 
	* @param threshold new threshold
	*/
	public void setThreshold( IScore threshold ) {
		this.threshold = threshold;
	}
	
	/** Gets the threshold.
	* 
	* @return current threshold
	*/
	public IScore getThreshold() {
		return threshold;
	}
	
	/**
	 * Gets the comparator.
	 * 
	 * @return current comparator
	 */
	public IScoreComparator getComparator() {
		return comparator;
	}

	/**
	 * Sets the comparator.
	 * 
	 * @param comparator new comparator
	 */
	public void setComparator( IScoreComparator comparator ) {
		this.comparator = comparator;
	}

}
