
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:18 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: TruncationSelector.java,v $
 *
 */

package org.evolvica.core.select;

import org.evolvica.core.IIndividual;
import org.evolvica.util.RandomUtils;

/** Abstract base class for a truncation selection.
 * A truncation selection only selects individuals, which have got an
 * attribute above a particular threshold. The threshold can be percentage
 * value, the score or a fitness value. Individuals below the threshold have
 * got the selection probability of 0. Individuals above are selected randomly.
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:58:18 $
 */
abstract public class TruncationSelector extends AbstractSelector {

	/** Constructor with number per selection.
	*
	* @param numberPerSelection number how many individuals are selected
	*/
	public TruncationSelector( int numberPerSelection ) {
		super( numberPerSelection );
	}

	/**
	 * @see org.evolvica.core.select.AbstractSelector#select(org.evolvica.core.IIndividual[])
	 */
	public IIndividual[] select( IIndividual[] pool ) {
		IIndividual[] rval = null;
		if ( numberPerSelection < 0 ) {
			rval = new IIndividual[pool.length];
		} else {
			rval = new IIndividual[numberPerSelection];
		}
		int threshold = threshold( pool );
		for ( int i = 0; i < rval.length; i++ ) {
			rval[i] = pool[RandomUtils.randomInt( threshold )];
		}
		return rval;
	}

	/**
	 * Calculates the truncation threshold.
	 * 
	 * @param pool selection pool.
	 * @return truncation threshold
	 */
	abstract protected int threshold( IIndividual[] pool );

}
