/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:18 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: AbsoluteScoreCondition.java,v $
 *
 */

package org.evolvica.core.terminate;

import org.evolvica.core.*;

/**
 * Absolute score condition.
 * This condition is fulfilled if a score from an individual set is
 * better than the target score.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:18 $
 */
public class AbsoluteScoreCondition extends ScoreComparingCondition {
	
	/**
	 * Target score.
	 */
	protected IScore target;
	
	/**
	 * Constructor.
	 */
	public AbsoluteScoreCondition() {
		this( null, null );
	}
	
	/**
	 * Constructor.
	 * 
	 * @param target target score
	 * @param comparator score comparator
	 */
	public AbsoluteScoreCondition( IScore target, IScoreComparator comparator ) {
		super( comparator );
		this.target = target;
	}
	
	/**
	 * @see org.evolvica.core.terminate.ICondition#isTrue(org.evolvica.core.IIndividualSet)
	 */
	public boolean isTrue( IIndividualSet iset ) {
		IIndividual best = IndividualSetUtils.findBest( iset, comparator );
		int compare = comparator.compare( target, best.getScore() );
		if ( compare >= 0 ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Returns the target score.
	 * 
	 * @return target score
	 */
	public IScore getTarget() {
		return target;
	}

	/**
	 * Sets the target score.
	 * 
	 * @param score target score
	 */
	public void setTarget(IScore score) {
		target = score;
	}

}
