/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/06/24 16:28:13 $
 * $Revision: 1.1 $
 * $State: Exp $
 * $RCSfile: AndCondition.java,v $
 * 
 */
package org.evolvica.core.terminate;

import org.evolvica.core.IIndividualSet;


/** A condition which contains several other conditions and concatenates them with a logical AND.
 * The condition returns false if no conditions are set.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.1 $ $Date: 2004/06/24 16:28:13 $
 */
public class AndCondition implements IMultiCondition {
	
	private ICondition[] conditions = new ICondition[0];
	
	/**
	 * @see org.evolvica.core.terminate.IMultiCondition#getConditions()
	 */
	public ICondition[] getConditions() {
		return conditions;
	}

	/**
	 * @see org.evolvica.core.terminate.IMultiCondition#setConditions(org.evolvica.core.terminate.ICondition[])
	 */
	public void setConditions( ICondition[] conditions ) {
		this.conditions = conditions;
	}

	/**
	 * @see org.evolvica.core.terminate.ICondition#isTrue(org.evolvica.core.IIndividualSet)
	 */
	public boolean isTrue( IIndividualSet iset ) {
		boolean rval = false;
		if ( conditions != null ) {
			boolean b = true;
			for ( int i = 0; i < conditions.length; i++ ) {
				if ( conditions[i] != null ) {
					b &= conditions[i].isTrue( iset );
				} else {
					b = false;
				}
			}
			rval = b;
		}
		return rval;
	}
	
}
