/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:18 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: BestWorstScoreCondition.java,v $
 *
 */

package org.evolvica.core.terminate;

import org.evolvica.core.*;

/**
 *  Best-Worst condition.
 * This condition is fulfilled if the difference between the best and the worst
 * score in an induividual falls below a certain value.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:58:18 $
 */
public class BestWorstScoreCondition extends ScoreComparingCondition {
	
	/**
	 * Threshold.
	 */
	protected double threshold;
	
	/**
	 * Constructor.
	 */
	public BestWorstScoreCondition() {
		this( 0.0, null );
	}
	
	/**
	 * Constructor.
	 * 
	 * @param threshold threshold
	 * @param comparator score comparator
	 */
	public BestWorstScoreCondition( double threshold, IScoreComparator comparator ) {
		super( comparator );
		this.threshold = threshold;
	}
	
	/**
	 * @see org.evolvica.core.terminate.ICondition#isTrue(org.evolvica.core.IIndividualSet)
	 */
	public boolean isTrue( IIndividualSet iset ) {
		IIndividual best = IndividualSetUtils.findBest( iset, comparator );
		IIndividual worst = IndividualSetUtils.findWorst( iset, comparator );
		double bestScore = ((Number) best.getScore().getValueAsObject()).doubleValue();
		double worstScore = ((Number) worst.getScore().getValueAsObject()).doubleValue();
		if ( bestScore - worstScore < threshold ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Returns the threshold.
	 * 
	 * @return threshold
	 */
	public double getThreshold() {
		return threshold;
	}

	/**
	 * Sets the threshold.
	 * 
	 * @param d threshold
	 */
	public void setThreshold(double d) {
		threshold = d;
	}

}
