/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:18 $
 * $Revision: 1.10 $
 * $State: Exp $
 * $RCSfile: ConditionalRouter.java,v $
 *
 */
package org.evolvica.core.terminate;

import org.evolvica.core.IIndividualSet;
import org.evolvica.core.common.AbstractRouter;

/** Conditional router.
 * A conditional router sends the incoming individual set to its output 0
 * if an underlying condition is false and to its output 1 if the condition
 * is true.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.10 $ $Date: 2004/05/26 11:58:18 $
 */
public class ConditionalRouter extends AbstractRouter {
	
	/**
	 * Underlying condition.
	 */
	protected ICondition condition = null;
	
	/**
	 * Constructor.
	 */
	public ConditionalRouter() {
		this( null );
	}
	
	/**
	 * Constructor with condition.
	 * 
	 * @param condition condition for this router
	 */
	public ConditionalRouter( ICondition condition ) {
		this.condition = condition;
		this.output = new Object[2];
	}
	
	/**
	 * @see org.evolvica.core.common.AbstractRouter#route(org.evolvica.core.IIndividualSet)
	 */
	public int route( IIndividualSet input ) {
		if ( condition.isTrue( input ) ) {
			return 1;
		} else {
			return 0;
		}
	}

	/**
	 * Returns the condition.
	 * @return ICondition
	 */
	public ICondition getCondition() {
		return condition;
	}

	/**
	 * Sets the condition.
	 * @param condition The condition to set
	 */
	public void setCondition( ICondition condition ) {
		this.condition = condition;
	}

}
