/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:19 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: CounterCondition.java,v $
 *
 */
package org.evolvica.core.terminate;

import org.evolvica.core.IIndividualSet;

/**
 * Termination after reaching a specified value of a counter.
 * The break condition is fulfilled in case of (current value of counter >= maximal value of counter).
 * Each time this condition is checked the internal counter is incremented by 1.
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:58:19 $
 */
public class CounterCondition implements ICondition {

	/** Maximal value of the counter.
	 * The default value is 1.
	 */
	protected int maxValue = 1;

	/** 
	 * Counter to use for break condition testing.
	 */
	protected int counter = 0;

	/** Constructor.
	 *
	 */
	public CounterCondition() {
		this( 1 );
	}
	
	/**
	 * Constructor.
	 * 
	 * @param maxValue maximal counter value
	 */
	public CounterCondition( int maxValue ) {
		this.maxValue = maxValue;
	}
	
	/** Test of the condition.
	 *
	 * @param set individual set
	 * @return result of the test of the condition
	 */
	public boolean isTrue( IIndividualSet set ) {
		if ( counter >= maxValue ) {
			return true;
		} else {
			++counter;
			return false;
		}
	}

	/**
	 * @return int
	 */
	public int getMaxValue() {
		return maxValue;
	}

	/**
	 * Sets the maxValue.
	 * @param maxValue The maxValue to set
	 */
	public void setMaxValue( int maxValue ) {
		this.maxValue = maxValue;
	}

}
