
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:19 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: ModuloCondition.java,v $
 *
 */

package org.evolvica.core.terminate;

import org.evolvica.core.IIndividualSet;

/** Modulo Condition.
 * This condition is fullfilled if a given counter % a given value equals 0.
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:58:19 $
 */
public class ModuloCondition implements ICondition {

	/** Modulo value.
	*/
	protected int modulo;

	/** Counter used for condition testing.
	*/
	protected int counter = 0;
	
	/**
	 * Constructor.
	 */
	public ModuloCondition() {
		this( 10 );
	}
	
	/** Constructor.
	*
	* @param value modulo value
	*/
	public ModuloCondition( int value ) {
		this.modulo = value;
	}
	
	/** Test of condition.
	*
	* @param set individual set
	* @return result of the test of break condition
	*/
	public boolean isTrue( IIndividualSet set ) {
		++counter;
		if ( counter % modulo == 0 ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Returns the modulo value.
	 * 
	 * @return modulo value
	 */
	public int getModulo() {
		return modulo;
	}

	/**
	 * Stes the modulo value.
	 * 
	 * @param value modulo value
	 */
	public void setModulo( int value ) {
		modulo = value;
	}

}
