/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:19 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: PhiCondition.java,v $
 *
 */

package org.evolvica.core.terminate;

import org.evolvica.core.*;

/**
 * Termination dependent on Phi.
 * The condition is fulfilled, if phi falls below a particular threshold.
 * Phi is defined as Abs( 1 - Abs( best score / average score ) ). .
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:58:19 $
 */
public class PhiCondition extends ScoreComparingCondition {
	
	/**
	 * Threshold.
	 */
	protected double threshold;
	
	/**
	 * Constructor.
	 */
	public PhiCondition() {
		this( 0.0, null );
	}
	
	/**
	 * Constructor.
	 * 
	 * @param threshold threshold
	 * @param comparator score comparator
	 */
	public PhiCondition( double threshold, IScoreComparator comparator ) {
		super( comparator );
		this.threshold = threshold;
	}
	
	/**
	 * @see org.evolvica.core.terminate.ICondition#isTrue(org.evolvica.core.IIndividualSet)
	 */
	public boolean isTrue(IIndividualSet iset) {
		IIndividual best = IndividualSetUtils.findBest( iset, comparator );
		double bestScore = ((Number) best.getScore().getValueAsObject()).doubleValue();
		double averageScore = 0.0;
		for( ISetIterator it = iset.iterator(); it.hasNext(); ) {
			double score = ((Number) it.next().getScore().getValueAsObject()).doubleValue();
			averageScore += score;
		}
		averageScore /= (double) iset.size();
		double phi = Math.abs( 1.0 - Math.abs( bestScore / averageScore ) );
		if ( phi <= threshold ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Returns the threshold.
	 * 
	 * @return threshold
	 */
	public double getThreshold() {
		return threshold;
	}

	/**
	 * Sets the threshold.
	 * 
	 * @param d threshold
	 */
	public void setThreshold( double d ) {
		threshold = d;
	}

}
