/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:20 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: RuntimeCondition.java,v $
 *
 */
package org.evolvica.core.terminate;

import org.evolvica.core.IIndividualSet;
import org.evolvica.util.EVLog;

/** Runtime condition.
 * The condition is fulfilled in case of (currentTime - startTime > interval). The interval
 * is specified with the constructor.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:20 $
 */
public class RuntimeCondition implements ICondition {

	/** Time in milliseconds, that must be exceeded for termination.
	*
	*/
	protected long breakValue = 0;
	
	/**
	 * Constructor.
	 */
	public RuntimeCondition () {
		this( 60 );
	}
	
	/** Constructor with runtime in seconds.
	*
	* @param seconds runtime in seconds
	*/
	public RuntimeCondition( int seconds ) {
		this( 1000 * (long) seconds );
	}

	/** Constructor with runtime in milliseconds.
	*
	* @param milliSeconds runtime in milliseconds
	*/
	public RuntimeCondition( long milliSeconds ) {
		breakValue = System.currentTimeMillis() + milliSeconds;
	}

	/** Test of condition.
	*
	* @param set individual set
	* @return result of test of break condition
	*/
	public boolean isTrue( IIndividualSet set ) {
		long currentTime = System.currentTimeMillis();
		if ( currentTime >= breakValue ) {
			EVLog.info( "runtime condition met at " + currentTime + " milliseconds." );
			return true;
		} else {
			return false;
		}
	}

}
