/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:20 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: ScoreComparingCondition.java,v $
 *
 */

package org.evolvica.core.terminate;

import org.evolvica.core.IScoreComparator;

/**
 * Abstract base class for a score comparing condition.
 * A score comparing condition relies on a user defined score.
 * This score is compared to the best score in an individual
 * collection. The result of the comparison is used to decide
 * if the condition is fulfilled or not.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:20 $
 */
abstract public class ScoreComparingCondition implements ICondition {
	
	/**
	 * Score comparator.
	 * 
	 */
	protected IScoreComparator comparator = null;
	
	/**
	 * Constructor.
	 * 
	 * @param comparator comparator to be used
	 */
	public ScoreComparingCondition( IScoreComparator comparator ) {
		this.comparator = comparator;
	}

	/**
	 * Returns the comparator.
	 * 
	 * @return score comparator
	 */
	public IScoreComparator getComparator() {
		return comparator;
	}

	/**
	 * Sets the comparator.
	 * 
	 * @param comparator new score comparator
	 */
	public void setComparator( IScoreComparator comparator ) {
		this.comparator = comparator;
	}

}
