/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:21 $
 * $Revision: 1.4 $
 * $State: Exp $
 * $RCSfile: AbstractDescriptor.java,v $
 * 
 */
package org.evolvica.engine;

import java.io.Serializable;

import org.apache.commons.lang.StringUtils;

/** Abstract base class for a descriptor.
 *
 * @author Andreas Rummler
 * @since 0.6.0
 * @version $Revision: 1.4 $ $Date: 2004/05/26 11:58:21 $
 */
abstract public class AbstractDescriptor implements Cloneable, Comparable, Serializable {

	private String displayName = null;
	private boolean expert = false;
	private boolean hidden = false;
	private String name = null;
	private boolean preferred = false;
	private String shortDescription = "no description available";
	
	/**
	 * Constructor.
	 * 
	 * @param name name of the descriptor
	 */
	public AbstractDescriptor( String name ) {
		this.name = name;
	}
	
	/**
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	abstract public int compareTo( Object obj );

	/**
	 * Returns the display name.
	 * 
	 * @return display name
	 */
	public String getDisplayName() {
		if ( StringUtils.isEmpty( displayName ) ) {
			return name;
		} else {
			return displayName;
		}
	}
	
	/**
	 * Returns the name of the operator class.
	 * 
	 * @return name
	 */
	public String getName() {
		return name;
	}
	/**
		 * Returns the description.
		 * 
		 * @return description
		 */
	public String getShortDescription() {
		return shortDescription;
	}

	/**
	 * Returns whether this property is an expert one.
	 * 
	 * @return expert flag
	 */
	public boolean isExpert() {
		return expert;
	}

	/**
	 * Returns whether this property is hidden.
	 * 
	 * @return hidden flag
	 */
	public boolean isHidden() {
		return hidden;
	}

	/**
	 * Returns whether this property is preferred.
	 * 
	 * @return preferred flag
	 */
	public boolean isPreferred() {
		return preferred;
	}

	/**
	 * Sets the display name.
	 * 
	 * @param string display name
	 */
	public void setDisplayName( String string ) {
		displayName = string;
	}

	/**
	 * Sets the expert flag.
	 * 
	 * @param b expert flag
	 */
	public void setExpert( boolean b ) {
		expert = b;
	}

	/**
	 * Sets the hidden flag.
	 * 
	 * @param b hidden flag
	 */
	public void setHidden( boolean b ) {
		hidden = b;
	}

	/**
	 * Sets the name.
	 * 
	 * @param string name
	 */
	public void setName( String string ) {
		name = string;
	}

	/**
	 * Sets the preferred flag.
	 * 
	 * @param b preferred flag
	 */
	public void setPreferred( boolean b ) {
		preferred = b;
	}

	/**
		 * Sets the description
		 * 
		 * @param string description
		 */
	public void setShortDescription(String string) {
		shortDescription = string;
	}
	
}
