/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:21 $
 * $Revision: 1.13 $
 * $State: Exp $
 * $RCSfile: AbstractOperator.java,v $
 *
 */
package org.evolvica.engine;

import org.apache.commons.lang.RandomStringUtils;
import org.evolvica.util.EVLog;
import org.evolvica.util.SerializationException;

/** Abstract base class for algorithm operators.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.13 $ $Date: 2004/05/26 11:58:21 $
 */
public abstract class AbstractOperator implements IOperator {
	
	/**
	 * Descriptor.
	 * 
	 */
	protected OperatorDescriptor descriptor = null;
	
	/**
	 * Associated algorithm element.
	 * 
	 */
	protected IElement element = null;
	
	/**
	 * Name of this operator.
	 * 
	 */
	protected String name = null;
	
	/**
	 * ID of this operator.
	 * 
	 */
	protected String id = RandomStringUtils.randomAlphanumeric( 12 ).toLowerCase();
	
	/**
	 * Constructor.
	 */
	public AbstractOperator() {
		try {
			descriptor = DescriptorBuilder.build( DescriptorResolver.locateDescriptor( getClass() ) );
		} catch ( SerializationException sx ) {
			descriptor = new OperatorDescriptor( getClass() );
			EVLog.error( "Error loading descriptor for operator " + getClass().getName(), sx );
		}
		this.name = descriptor.getName();
	}
	
	/**
	 * @see org.evolvica.engine.IOperator#attachDescriptor(org.evolvica.engine.OperatorDescriptor)
	 */
	public void attachDescriptor( OperatorDescriptor descriptor ) {
		if ( descriptor != null ) {
			this.descriptor = descriptor;
		} 
	}
	
	/**
	 * @see org.evolvica.engine.IOperator#attachElement(org.evolvica.engine.IElement)
	 */
	public void attachElement( IElement element ) {
		this.element = element;
	}
	
	/**
	 * @see org.evolvica.engine.IOperator#descriptor()
	 */
	public OperatorDescriptor descriptor() {
		return descriptor;
	}
	
	/**
	 * @see org.evolvica.engine.IOperator#element()
	 */
	public IElement element() {
		return element;
	}
	
	/**
	 * @see org.evolvica.engine.IOperator#getId()
	 */
	public String getId() {
		return id;
	}
	
	/**
	 * @see org.evolvica.engine.IOperator#setId(java.lang.String)
	 */
	public void setId( String id ) {
		this.id = id; 
	}
	
	/**
	 * @see org.evolvica.engine.IOperator#getName()
	 */
	public String getName() {
		return name;
	}

	/**
	 * @see org.evolvica.engine.IOperator#setName(java.lang.String)
	 */
	public void setName( String name ) {
		this.name = name;
	}

}
