/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:22 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: ConduitElement.java,v $
 *
 */
 
package org.evolvica.engine;

import java.beans.PropertyChangeEvent;

import org.apache.commons.beanutils.MethodUtils;
import org.evolvica.util.EVLog;

/** Conduit element.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:22 $
 */
public class ConduitElement extends AbstractElement {
	
	/**
	 * Underlying conduit operator.
	 * 
	 */
	protected IConduit conduit;
	
	/**
	 * Input data.
	 * 
	 */
	protected Object[] data;
	
	/**
	 * Constructor.
	 * 
	 * @param conduit underlying conduit operator
	 */
	public ConduitElement( IConduit conduit ) {
		super( conduit.descriptor().getInputSize(), conduit.descriptor().getOutputSize() );
		this.conduit = conduit;
		this.data = new Object[incoming.length];
	}
	
	/**
	 * @see org.evolvica.engine.IElement#getOperator()
	 */
	public IOperator getOperator() {
		return conduit;
	}

	/**
	 * @see org.evolvica.engine.IElement#activate()
	 */
	public void activate() {
		for ( int i = 0; i < incoming.length; i++ ) {
			data[i] = incoming[i].popData();
		}
		Object[] outData = conduit.process( data );
		for (int i = 0; i < outgoing.length; i++) {
			outgoing[i].pushData( outData[i] );
		}
	}

	/**
	 * @see org.evolvica.engine.IElement#isActivatable()
	 */
	public boolean isActivatable() {
		for ( int i = 0; i < incoming.length; i++ ) {
			data[i] = incoming[i].getData();
		}
		return conduit.isActivatable( data );
	}

	/**
	 * @see org.evolvica.engine.IElement#setIncomingSize(int)
	 */
	public void setIncomingSize( int size ) {
		if ( size != incoming.length && size > 1 ) {
			try {
				MethodUtils.invokeExactMethod( conduit, "setInputSize", new Object[] { new Integer( size ) }, new Class[] { Integer.TYPE } );
			} catch ( Exception ex ) {
				EVLog.error( "error setting input size on operator " + conduit.getName() );
				return;
			}
			IConnection[] newIncoming = new IConnection[size]; // create new array with incoming connections
			if ( size < incoming.length ) {
				System.arraycopy( incoming, 0, newIncoming, 0, size ); // copy old connections
				for ( int i = size; i < incoming.length; i++ ) { // remove connections that do not have a slot any longer
					getAlgorithm().removeConnection( incoming[i] );
				}
			} else {
				System.arraycopy( incoming, 0, newIncoming, 0, incoming.length ); // copy old connections
			}
			incoming = newIncoming;
			input = new Object[size];
			firePropertyChangeEvent( new PropertyChangeEvent( this, Events.INCOMING_SIZE_CHANGED, null, new Integer( size ) ) );
		}
	}
	
	/**
	 * @see org.evolvica.engine.IElement#setOutgoingSize(int)
	 */
	public void setOutgoingSize( int size ) {
		if ( size != outgoing.length && size > 1 ) {
			try {
				MethodUtils.invokeExactMethod( conduit, "setOutputSize", new Object[] { new Integer( size ) }, new Class[] { Integer.TYPE } );
			} catch ( Exception ex ) {
				EVLog.error( "error setting output size on operator " + conduit.getName() );
				return;
			}
			IConnection[] newOutgoing = new IConnection[size]; // create new array with outgoing connections
			if ( size < outgoing.length ) {
				System.arraycopy( outgoing, 0, newOutgoing, 0, size ); // copy old connections
				for ( int i = size; i < outgoing.length; i++ ) { // remove connections that do not have a slot any longer
					getAlgorithm().removeConnection( outgoing[i] );
				}
			} else {
				System.arraycopy( outgoing, 0, newOutgoing, 0, outgoing.length ); // copy old connections
			}
			outgoing = newOutgoing;
			output = new Object[size];
			firePropertyChangeEvent( new PropertyChangeEvent( this, Events.OUTGOING_SIZE_CHANGED, null, new Integer( size ) ) );
		}
	}
	
}
