/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:22 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: Connection.java,v $
 *
 */
package org.evolvica.engine;

import gnu.trove.THashSet;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;

/** Default implementation for a connection.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:22 $
 */
public class Connection implements IConnection {
	
	private THashSet propertyChangeListeners = new THashSet();
	
	/**
	 * The source element.
	 */
	protected IElement source;
	
	/**
	 * The target element.
	 */
	protected IElement target;
	
	/**
	 * The stored data object.
	 */
	protected Object data;
	
	/**
	 * Constructor.
	 * 
	 * @param source source element
	 * @param target target element
	 */
	public Connection( IElement source, IElement target ) {
		this.source = source;
		this.target = target;
	}
	
	/**
	 * @see org.evolvica.engine.IConnection#getSource()
	 */
	public IElement getSource() {
		return source;
	}

	/**
	 * @see org.evolvica.engine.IConnection#setSource(org.evolvica.engine.IElement)
	 */
	public void setSource( IElement source ) {
		this.source = source;
	}

	/**
	 * @see org.evolvica.engine.IConnection#getTarget()
	 */
	public IElement getTarget() {
		return target;
	}
	
	/**
	 * @see org.evolvica.engine.IConnection#setTarget(org.evolvica.engine.IElement)
	 */
	public void setTarget( IElement target ) {
		this.target = target;
	}

	/**
	 * @see org.evolvica.engine.IConnection#pushData(java.lang.Object)
	 */
	public void pushData( Object data ) {
		this.data = data;
	}
	
	/**
	 * @see org.evolvica.engine.IConnection#hasData()
	 */
	public boolean hasData() {
		if ( data != null ) {
			return true;
		} else {
			return false;
		}
	}
	
	/**
	 * @see org.evolvica.engine.IConnection#popData()
	 */
	public Object popData() {
		Object tmp = data;
		data = null;
		return tmp;
	}
	
	/**
	 * @see org.evolvica.engine.IConnection#getData()
	 */
	public Object getData() {
		return data;
	}
	
	public void addPropertyChangeListener( PropertyChangeListener listener ) {
		if ( listener != null ) {
			propertyChangeListeners.add( listener );
		}
	}
	
	public void removePropertyChangeListener( PropertyChangeListener listener ) {
		if ( listener != null ) {
			propertyChangeListeners.remove( listener );
		}
	}
	
	public void firePropertyChangeEvent( PropertyChangeEvent event ) {
		for ( Iterator it = propertyChangeListeners.iterator(); it.hasNext(); ) {
			((PropertyChangeListener) it.next()).propertyChange( event );
		}
	}
	
}
