/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:22 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: ConnectorElement.java,v $
 *
 */
package org.evolvica.engine;

/** Connector (processor) element.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:58:22 $
 */
public class ConnectorElement extends AbstractElement {
	
	/**
	 * Underlying processing operator.
	 * 
	 */
	protected IProcessor processor;
	
	/**
	 * Constructor.
	 * 
	 * @param processor underlying processing operator
	 */
	public ConnectorElement( IProcessor processor ) {
		super( 1, 1 );
		this.processor = processor;
	}
	
	/**
	 * @see org.evolvica.engine.IElement#getOperator()
	 */
	public IOperator getOperator() {
		return processor;
	}
	
	/**
	 * @see org.evolvica.engine.IElement#activate()
	 */
	public void activate() {
		outgoing[0].pushData( processor.process( incoming[0].popData() ) );
	}

	/**
	 * @see org.evolvica.engine.IElement#isActivatable()
	 */
	public boolean isActivatable() {
		if ( incoming[0].hasData() ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * @see org.evolvica.engine.IElement#setIncomingSize(int)
	 */
	public void setIncomingSize( int size ) {
		// do nothing, this element is immutable
	}
	
	/**
	 * @see org.evolvica.engine.IElement#setOutgoingSize(int)
	 */
	public void setOutgoingSize( int size ) {
		// do nothing, this element is immutable
	}
	
}
