/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:23 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: ElementFactory.java,v $
 *
 */
 
package org.evolvica.engine;

/** Factory for algorithm elements.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:58:23 $
 */
public class ElementFactory {
	
	private ElementFactory() {
		// no need for instantiation
	}
	
	/**
	 * Creates a new algorithm element for a given operator.
	 * 
	 * @param operator operator
	 * @return newly created algorithm element for the given operator or null if
	 * no element could be created
	 */
	public static final IElement newElement( IOperator operator ) {
		IElement element = null;
		if ( operator instanceof ISource ) {
			element = new SourceElement( (ISource) operator );
		} else if ( operator instanceof IProcessor ) {
			element = new ConnectorElement( (IProcessor) operator );
		} else if ( operator instanceof ISink ) {
			element = new SinkElement( (ISink) operator );
		} else if ( operator instanceof IFork ) {
			element = new ForkElement( (IFork) operator );
		} else if ( operator instanceof IJoin ) {
			element = new JoinElement( (IJoin) operator );
		} else if ( operator instanceof IConduit ) {
			element = new ConduitElement( (IConduit) operator );
		}
		operator.attachElement( element );
		return element;
	}
	
}
