/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:23 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: ForkElement.java,v $
 *
 */
 
package org.evolvica.engine;

import java.beans.PropertyChangeEvent;

import org.apache.commons.beanutils.MethodUtils;
import org.evolvica.util.EVLog;

/** Fork element.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:23 $
 */
public class ForkElement extends AbstractElement {
	
	/**
	 * Underlying fork operator.
	 * 
	 */
	protected IFork fork;
	
	/**
	 * Constructor.
	 * 
	 * @param fork underlying fork operator
	 */
	public ForkElement( IFork fork ) {
		super( 1, fork.descriptor().getOutputSize() );
		this.fork = fork;
	}
	
	/**
	 * @see org.evolvica.engine.IElement#getOperator()
	 */
	public IOperator getOperator() {
		return fork;
	}

	/**
	 * @see org.evolvica.engine.IElement#activate()
	 */
	public void activate() {
		Object[] output = fork.fork( incoming[0].getData() );
		for ( int i = 0; i < output.length; i++ ) {
			if ( output[i] != null ) {
				outgoing[i].pushData( output[i] );
			}
		}
	}

	/**
	 * @see org.evolvica.engine.IElement#isActivatable()
	 */
	public boolean isActivatable() {
		if ( incoming[0].hasData() ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * @see org.evolvica.engine.IElement#setIncomingSize(int)
	 */
	public void setIncomingSize( int size ) {
		// do nothing, the incoming size of this element is immutable
	}
	
	/**
	 * @see org.evolvica.engine.IElement#setOutgoingSize(int)
	 */
	public void setOutgoingSize( int size ) {
		if ( size != outgoing.length && size > 1 ) {
			try {
				MethodUtils.invokeExactMethod( fork, "setOutputSize", new Object[] { new Integer( size ) }, new Class[] { Integer.TYPE } );
			} catch ( Exception ex ) {
				EVLog.error( "error setting output size on operator " + fork.getName() );
				return;
			}
			IConnection[] newOutgoing = new IConnection[size]; // create new array with outgoing connections
			if ( size < outgoing.length ) {
				System.arraycopy( outgoing, 0, newOutgoing, 0, size ); // copy old connections
				for ( int i = size; i < outgoing.length; i++ ) { // remove connections that do not have a slot any longer
					getAlgorithm().removeConnection( outgoing[i] );
				}
			} else {
				System.arraycopy( outgoing, 0, newOutgoing, 0, outgoing.length ); // copy old connections
			}
			outgoing = newOutgoing;
			output = new Object[size];
			firePropertyChangeEvent( new PropertyChangeEvent( this, Events.OUTGOING_SIZE_CHANGED, null, new Integer( size ) ) );
		}
	}
	
}
