/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:24 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: IConduit.java,v $
 *
 */
package org.evolvica.engine;

/** Interface for a conduit.
 * A conduit can have an arbitrary number of inputs and outputs
 * and is used for special operators that do not fit in other
 * classes.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:24 $
 */
public interface IConduit extends IOperator {
	
	/**
	 * Processes an array of data objects.
	 * 
	 * @param input array of input objects
	 * @return array of output objects
	 */
	public Object[] process( Object[] input );
	
	/**
	 * Checks if this conduit can be activated.
	 * 
	 * @param input input objects.
	 * @return true if this conduit can be activated with the given input data, false otherwise
	 */
	public boolean isActivatable( Object[] input );
	
}
