/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:24 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: IConnection.java,v $
 *
 */
package org.evolvica.engine;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

/** Interface for connections between algorithmic elements.
 * Connections are similiar to directed edges in a data flow graph.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:24 $
 */
public interface IConnection {
	
	public void addPropertyChangeListener( PropertyChangeListener listener );
	
	public void removePropertyChangeListener( PropertyChangeListener listener ); 
	
	public void firePropertyChangeEvent( PropertyChangeEvent event );
	
	/**
	 * Returns the source element of this connection.
	 * 
	 * @return source element
	 */
	public IElement getSource();
	
	/**
	 * Sets a new source element.
	 * 
	 * @param source new source element
	 */
	public void setSource( IElement source );
	
	/**
	 * Returns the target element of this connection.
	 * 
	 * @return target element
	 */
	public IElement getTarget();
	
	/**
	 * Sets a new target element.
	 * 
	 * @param target new target element
	 */
	public void setTarget( IElement target );
	
	/**
	 * Pushes a data object into this connection.
	 * 
	 * @param data data object
	 */
	public void pushData( Object data );
	
	/**
	 * Checks if there is a data object associated to this connection.
	 * 
	 * @return true if this connection contains some data object, false instead
	 */
	public boolean hasData();
	
	/**
	 * Retrieves a data object from this connection.
	 * The object is returned and removed from this connection.
	 * 
	 * @return data object stored in this connection
	 */
	public Object popData();
	
	/**
	 * Retrieves a data object from this connection without removing it.
	 * 
	 * @return data object stored in this connection
	 */
	public Object getData();
	
}
