/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:24 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: IJoin.java,v $
 *
 */
package org.evolvica.engine;

/** Interface for a join operator.
 * A join operator has several input slots and excatly one output slot.
 * It combines data from an input array to one single data object in some
 * particular way. 
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:24 $
 */
public interface IJoin extends IOperator {

	/**
	 * Performs the join operation and returns a single data object.
	 * Note that the input array may contain NULL values.
	 * 
	 * @param input array of input data objects
	 * @return combined data object
	 */
	public Object join( Object[] input );
	
	/**
	 * Checks if this join can be activated with given input data.
	 * 
	 * @param input input data
	 * @return true if this join can be activated, false otherwise
	 */
	public boolean isActivatable( Object[] input );
	
}
