/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:24 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: IOperator.java,v $
 *
 */
package org.evolvica.engine;

import java.io.Serializable;

/** Toplevel interface for an operator used in an algorithm.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:24 $
 */
public interface IOperator extends Serializable {
	
	/**
	 * Attaches a descriptor to this operator.
	 * 
	 * @param descriptor descriptor to be attached
	 */
	public void attachDescriptor( OperatorDescriptor descriptor );
	
	/**
	 * Sets the algorithm element for this operator.
	 * 
	 * @param element associated algorithm element for this operator
	 */
	public void attachElement( IElement element );
	
	/**
	 * Returns a descriptor for this operator.
	 * @return descriptor
	 */
	public OperatorDescriptor descriptor();
	
	/**
	 * Returns the algorithm element that is attached to this operator.
	 * 
	 * @return attached algorithm element
	 */
	public IElement element();
	
	/**
	 * Returns the name of this operator.
	 * @return name of the operator
	 */
	public String getName();
	
	/**
	 * Sets the name of this operator.
	 * @param name new name for this operator
	 */
	public void setName( String name );
		
	/**
	 * Return the internal identifier of this operator.
	 * @return identifier of the operator
	 */
	public String getId();
	
	/**
	 * Sets the identifier of this operator.
	 * @param id new identifier for this operator
	 */
	public void setId( String id );
	
}
