/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:25 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: IValidator.java,v $
 *
 */
package org.evolvica.engine;

/** Interface for an algorithm validator.
 * A validator performs checks to determine if an algorithm can be run safely.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:58:25 $
 */
public interface IValidator {

	/**
	 * Validates an algorithm.
	 * 
	 * @param algorithm algorithm to check for errors and inconsistencies.
	 * @return boolean true if the algorithm is valid, false instead
	 */
	public boolean validate( IAlgorithm algorithm );
	
	/**
	 * Returns an array of strings containing messages that were generated during validation of an algorithm.
	 * 
	 * @return array of messages
	 */
	public String[] getValidationResult();
	
}
