/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:26 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: JoinElement.java,v $
 *
 */
 
package org.evolvica.engine;

import java.beans.PropertyChangeEvent;

import org.apache.commons.beanutils.MethodUtils;
import org.evolvica.util.EVLog;

/** An algorithm element for a join operator.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:58:26 $
 */
public class JoinElement extends AbstractElement {
	
	/**
	 * The underlying join operator.
	 * 
	 */
	protected IJoin join;
	
	/**
	 * Input data of this join element.
	 * 
	 */
	protected Object[] data;
	
	/**
	 * Constructor.
	 * 
	 * @param join the underlying join operator
	 */
	public JoinElement( IJoin join ) {
		super( join.descriptor().getInputSize(), 1 );
		this.join = join;
		this.data = new Object[incoming.length];
	}
	
	/**
	 * @see org.evolvica.engine.IElement#getOperator()
	 */
	public IOperator getOperator() {
		return join;
	}

	/**
	 * @see org.evolvica.engine.IElement#activate()
	 */
	public void activate() {
		for ( int i = 0; i < incoming.length; i++ ) {
			data[i] = incoming[i].popData();
		}
		outgoing[0].pushData( join.join( data ) );
	}

	/**
	 * @see org.evolvica.engine.IElement#isActivatable()
	 */
	public boolean isActivatable() {
		for ( int i = 0; i < incoming.length; i++ ) {
			data[i] = incoming[i].getData();
		}
		return join.isActivatable( data );
	}

	/**
	 * @see org.evolvica.engine.IElement#setIncomingSize(int)
	 */
	public void setIncomingSize( int size ) {
		if ( size != incoming.length && size > 1 ) {
			try {
				MethodUtils.invokeExactMethod( join, "setInputSize", new Object[] { new Integer( size ) }, new Class[] { Integer.TYPE } );
			} catch ( Exception ex ) {
				EVLog.error( "error setting input size on operator " + join.getName() );
				return;
			}
			IConnection[] newIncoming = new IConnection[size]; // create new array with incoming connections
			if ( size < incoming.length ) {
				System.arraycopy( incoming, 0, newIncoming, 0, size ); // copy old connections
				for ( int i = size; i < incoming.length; i++ ) { // remove connections that do not have a slot any longer
					getAlgorithm().removeConnection( incoming[i] );
				}
			} else {
				System.arraycopy( incoming, 0, newIncoming, 0, incoming.length ); // copy old connections
			}
			incoming = newIncoming;
			input = new Object[size];
			firePropertyChangeEvent( new PropertyChangeEvent( this, Events.INCOMING_SIZE_CHANGED, null, new Integer( size ) ) );
		}
	}
	
	/**
	 * @see org.evolvica.engine.IElement#setOutgoingSize(int)
	 */
	public void setOutgoingSize( int size ) {
		// do nothing, the outgoing size of this element is immutable
	}
	
}
