/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:26 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: Run.java,v $
 *
 */
package org.evolvica.engine;

import java.io.FileInputStream;
import java.io.IOException;

import org.apache.commons.cli.*;
import org.evolvica.util.SerializationException;
import org.xml.sax.SAXException;

/**
 * Runner.
 * This class only contains a main method that can be used to run algorithms
 * from the command line. If the main method is called without parameters the usage
 * is printed.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:58:26 $
 * @since 0.4.0
 */
public class Run {
	
	/**
	 * Main method.
	 * 
	 * @param args main arguments
	 */
	public static void main( String[] args ) {
		Parser parser = new GnuParser();
		Option fileOption = new Option( "f", "file", true, "file containing algorithm structure" );
		Options options = new Options();
		options.addOption( fileOption );
		try {
			CommandLine cl = parser.parse( options, args );
			if ( cl.hasOption( "f" ) ) {
				String file = cl.getOptionValue( "f" );
				try {
					IAlgorithm algorithm = new AlgorithmBuilder().build( new FileInputStream( file ) );
					algorithm.execute();
				} catch ( SAXException sax ) {
					System.out.println( "error: file contains no valid algorithm structure or structure has errors" );
				} catch ( IOException iox ) {
					System.out.println( "error: file not accessible/readable" );
				} catch ( SerializationException sx ) {
					System.out.println( "error: unable to (re)built algorithm structure contained in file" );
				}
				return;
			} else {
				// quiet
			}
		} catch ( ParseException px ) {
			// quiet
		}
		new HelpFormatter().printHelp( "Run", options );
	}

}
