/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:26 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: SinkElement.java,v $
 *
 */
package org.evolvica.engine;

/** Sink element.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:58:26 $
 */
public class SinkElement extends AbstractElement {
	
	/**
	 * Underlying sink operator.
	 * 
	 */
	protected ISink sink;
	
	/**
	 * Constructor.
	 * 
	 * @param sink underlying sink operator
	 */
	public SinkElement( ISink sink ) {
		super( 1, 0 );
		this.sink = sink;
	}
	
	/**
	 * @see org.evolvica.engine.IElement#getOperator()
	 */
	public IOperator getOperator() {
		return sink;
	}
	
	/**
	 * @see org.evolvica.engine.IElement#activate()
	 */
	public void activate() {
		sink.store( incoming[0].popData() );
	}

	/**
	 * @see org.evolvica.engine.IElement#isActivatable()
	 */
	public boolean isActivatable() {
		if ( incoming[0].hasData() ) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * @see org.evolvica.engine.IElement#setIncomingSize(int)
	 */
	public void setIncomingSize( int size ) {
		// do nothing, this element is immutable
	}
	
	/**
	 * @see org.evolvica.engine.IElement#setOutgoingSize(int)
	 */
	public void setOutgoingSize( int size ) {
		// do nothing, this element is immutable
	}
	
}
