/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:26 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: SourceElement.java,v $
 *
 */
package org.evolvica.engine;

/** Source element.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:58:26 $
 */
public class SourceElement extends AbstractElement {
	
	/**
	 * Underlying source operator.
	 * 
	 */
	protected ISource source;
	
	/**
	 * Constructor.
	 * 
	 * @param source underlying source operator
	 */
	public SourceElement( ISource source ) {
		super( 0, 1 );
		this.source = source;
	}
	
	/**
	 * @see org.evolvica.engine.IElement#getOperator()
	 */
	public IOperator getOperator() {
		return source;
	}
	
	/**
	 * @see org.evolvica.engine.IElement#activate()
	 */
	public void activate() {
		outgoing[0].pushData( source.create() );
	}

	/**
	 * @see org.evolvica.engine.IElement#isActivatable()
	 */
	public boolean isActivatable() {
		return true;
	}

	/**
	 * @see org.evolvica.engine.IElement#setIncomingSize(int)
	 */
	public void setIncomingSize( int size ) {
		// do nothing, this element is immutable
	}
	
	/**
	 * @see org.evolvica.engine.IElement#setOutgoingSize(int)
	 */
	public void setOutgoingSize( int size ) {
		// do nothing, this element is immutable
	}
	
}
