
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:35:27 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: SalesmanAlgorithm.java,v $
 *
 */

package org.evolvica.examples.salesman;

import java.io.File;

import org.evolvica.core.common.Collector;
import org.evolvica.core.common.DefaultSink;
import org.evolvica.core.eval.ReverseIntegerScoreComparator;
import org.evolvica.core.init.IntegerSequenceInitializer;
import org.evolvica.core.mutate.iarray.ScrambleMutator;
import org.evolvica.core.population.EliteInsertion;
import org.evolvica.core.population.SortedPopulation;
import org.evolvica.core.recombine.iarray.OrderCrossover;
import org.evolvica.core.select.PoolSizeTruncationSelector;
import org.evolvica.core.terminate.ConditionalRouter;
import org.evolvica.core.terminate.CounterCondition;
import org.evolvica.engine.AbstractAlgorithm;

public class SalesmanAlgorithm extends AbstractAlgorithm {
	
	// overload setup
	public void setup() throws Exception {
		
		// create initialization (sources)
		IntegerSequenceInitializer init = new IntegerSequenceInitializer( 29 );
		init.setSetSize( 100 );
		// create comparator: we use a reverse comparator because a shorter length is better
		ReverseIntegerScoreComparator comp = new ReverseIntegerScoreComparator();
		// create population
		SortedPopulation population = new SortedPopulation();
		population.setComparator( comp ); // sort the population according to this comparator
		population.setFilter( new EliteInsertion() );
		((EliteInsertion) population.getFilter()).setComparator( comp ); // attach comparator to insertion filter
		// create sink
		DefaultSink sink = new DefaultSink();
		sink.setComparator( comp );
		
		// create processing elements
		// evaluation
		SalesmanEvaluator eval = new SalesmanEvaluator( new File( "enter path to bavaria29.mat here" ) );
		// selection
		PoolSizeTruncationSelector select = new PoolSizeTruncationSelector( 100, 0.3f );
		// recombination
		OrderCrossover recombine = new OrderCrossover();
		// mutation
		ScrambleMutator mutate = new ScrambleMutator();
		
		// create the condition and add them to the conditional thread
		CounterCondition condition = new CounterCondition( 2000 ); // stop after 2000 generations
		
		// create conditional router
		ConditionalRouter router = new ConditionalRouter( condition );
		
		Collector collector = new Collector();
		
		// connect everything together
		connect( init, collector, 0 );
		connect( collector, eval );
		connect( eval, population );
		connect( eval, population );
		connect( population, router );
		connect( router, 0, select );
		connect( router, 1, sink );
		connect( select, recombine );
		connect( recombine, mutate );
		connect( mutate, collector, 1 );
		
	}
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#teardown()
	 */
	public void teardown() throws Exception {

	}

	public static void main( String[] args ) throws Exception {
		new SalesmanAlgorithm().execute();
	}
	
}
