
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:35:28 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: SalesmanEvaluator.java,v $
 *
 */

package org.evolvica.examples.salesman;

import java.io.File;

import org.evolvica.core.IIndividual;
import org.evolvica.core.IntegerArrayGene;
import org.evolvica.core.eval.AbstractEvaluator;
import org.evolvica.core.eval.IntegerScore;
import org.evolvica.util.EVLog;
import org.evolvica.util.EVRuntimeException;

/**
 * Evaluator for the TSP example.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:35:28 $
 * @since 0.4.0
 */
public class SalesmanEvaluator extends AbstractEvaluator {
	
	/**
	 * TSP matrix.
	 */
	protected SalesmanMatrix matrix = null;
	
	/**
	 * File containing TSP distance data.
	 */
	protected File matrixFile = null;
	
	/**
	 * Constructor.
	 */
	public SalesmanEvaluator() {
		this( null );
	}
	
	/**
	 * Constructor.
	 * 
	 * @param matrixFile file containing the distance data for the TSP instance
	 */
	public SalesmanEvaluator( File matrixFile ) {
		this.matrixFile = matrixFile;
		this.matrix = new SalesmanMatrix( matrixFile );
	}
	
	/**
	 * @see org.evolvica.core.eval.AbstractEvaluator#evaluate(org.evolvica.core.IIndividual)
	 */
	public void evaluate( IIndividual ind ) {
		int[] array = ((IntegerArrayGene) ind.getGenotype()).getValue();
		if ( array.length != matrix.getSize() ) {
			EVLog.error( "length of integer array genotype (" + array.length + ") does not match TSP matrix size (" + matrix.getSize() + ")" );
			throw new EVRuntimeException();
		} else {
			ind.setScore( new IntegerScore( matrix.getDistance( array ) ) );
		}
	}

	/**
	 * Returns the TSP data file.
	 * @return TSP data file
	 */
	public File getMatrixFile() {
		return matrixFile;
	}

	/**
	 * Sets the TSP distance data file.
	 * @param file distance data file
	 */
	public void setMatrixFile( File matrixFile ) {
		this.matrixFile = matrixFile;
		this.matrix = new SalesmanMatrix( matrixFile );
	}

}
