/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits and Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:35:28 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: SalesmanMatrix.java,v $
 *
 */

package org.evolvica.examples.salesman;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.LineNumberReader;
import java.util.LinkedList;
import java.util.List;
import java.util.StringTokenizer;

import org.evolvica.util.EVLog;

public class SalesmanMatrix {
	
	private int[][] distances = new int[0][0];
	private int[] bestRoute = new int[0];
	private int bestValue = 0;
	private String comment = "no comment";
	
	public SalesmanMatrix( File file ) {
		process( read( file ) );
	}
	
	protected List read( File file ) {
		if ( file != null ) {
			String line = null;
			List buffer = new LinkedList();
			LineNumberReader reader = null;
			try {
				reader = new LineNumberReader( new FileReader( file ) );
			} catch ( IOException iox ) {
				return null;
			}
			do {
				try {
					line = reader.readLine();
				}
				catch ( IOException io ) {
					EVLog.error( "read failure" );
				}
				if ( line != null ) {
					buffer.add( line );
				}
			} while ( line != null );
			return buffer;
		} else {
			return null;
		}
	}
	
	protected void process( List buffer ) {
		if ( buffer != null ) {
			StringTokenizer st = new StringTokenizer( (String) buffer.get( 0 ) );
			// number of cities
			int tokens = st.countTokens();
			// get the best route
			bestRoute = new int[tokens];
			try {
				for ( int i = 0; i < tokens; i++ ) {
					bestRoute[i] = Integer.parseInt( (String) st.nextToken() ) - 1;
				}
			} catch ( Exception ex ) {
				bestRoute = new int[] { -1 };
			}
			// get best known value
			bestValue = Integer.parseInt( (String) buffer.get( 1 ) );
			// get the comment
			comment = (String) buffer.get( 2 );
			distances = new int[tokens][tokens];
			for ( int i = 0; i < tokens; i++ ) {
				st = new StringTokenizer( (String) buffer.get( i + 3 ) );
				int tokenNumber = 0;
				while ( st.hasMoreTokens() ) {
					int token = Integer.parseInt( st.nextToken() );
					distances[i][tokenNumber] = token;
					distances[tokenNumber][i] = token;
					tokenNumber++;
				}
			}
		}
	}
	
	public String getComment() {
		return comment;
	}
	
	public int[] bestRoute() {
		return bestRoute;
	}
	
	public int bestValue() {
		return bestValue;
	}
	
	public int getSize() {
		return bestRoute.length;
	}
	
	public int getDistance( int x, int y ) {
		return distances[x][y];
	}
	
	public int getDistance( int[] sequence ) {
		int distance = 0;
		for ( int i = 0; i < sequence.length - 1; i++ ) {
			distance += getDistance( sequence[i], sequence[i+1] );
		}
		distance += getDistance( sequence[sequence.length-1], sequence[0] ); // because of symmetry
		return distance;
	}
	
	public static void main(String[] args) throws Exception {
		SalesmanMatrix sm = new SalesmanMatrix( new File( "d:\\brazil58.mat" ) );
		System.out.println( sm.getSize() );
	}
	
}
