/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:35:29 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: SphereAlgorithm.java,v $
 *
 */
package org.evolvica.examples.sphere;

import org.evolvica.core.common.Collector;
import org.evolvica.core.common.DefaultSink;
import org.evolvica.core.common.Duplicator;
import org.evolvica.core.eval.ReverseFloatScoreComparator;
import org.evolvica.core.eval.SphereEvaluator;
import org.evolvica.core.init.FloatArrayInitializer;
import org.evolvica.core.mutate.Float.RelativeFloatMutator;
import org.evolvica.core.mutate.farray.FloatMutationWrapper;
import org.evolvica.core.population.ElitistInsertion;
import org.evolvica.core.population.SortedPopulation;
import org.evolvica.core.terminate.ConditionalRouter;
import org.evolvica.core.terminate.CounterCondition;
import org.evolvica.engine.AbstractAlgorithm;

/**
 * Sphere Algorithm.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:35:29 $
 * @since 0.4.0
 */
public class SphereAlgorithm extends AbstractAlgorithm {
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#setup()
	 */
	public void setup() throws Exception {
		
		// create initializer (source) & set parameters
		FloatArrayInitializer initializer = new FloatArrayInitializer();
		initializer.setArraySize( 3 ); // set the sphere function dimension
		initializer.setLowerBound( -100.0f ); // set the lower bound of generated random float numbers
		initializer.setUpperBound( 100.0f ); // set the upper bound of generated random float numbers
		initializer.setSetSize( 20 ); // set the number of individuals to be generated
		// create evaluator of the sphere function
		SphereEvaluator evaluator = new SphereEvaluator();
		// create a population
		SortedPopulation population = new SortedPopulation();
		population.setComparator( new ReverseFloatScoreComparator() );
		// create the insertion filter
		ElitistInsertion insertion = new ElitistInsertion( (float) 0.5, (float) 0.5, new ReverseFloatScoreComparator() );
		population.setFilter( insertion );
		// create mutation operator
		RelativeFloatMutator floatMutator = new RelativeFloatMutator( 0.2f );
		FloatMutationWrapper mutator = new FloatMutationWrapper( floatMutator );
		mutator.setPropability( 1.0f );
		// duplication mutation operator
		Duplicator duplicator = new Duplicator();
		// create sink
		DefaultSink sink = new DefaultSink();
		sink.setComparator( new ReverseFloatScoreComparator() );
		
		// define termination conditions
		ConditionalRouter router = new ConditionalRouter( new CounterCondition( 200 ) );
		
		Collector collector = new Collector();
		collector.setInputSize( 2 );
		
		connect( initializer, collector, 0 );
		connect( collector, evaluator );
		connect( evaluator, population );
		connect( population, router );
		connect( router, 0, duplicator );
		connect( router, 1, sink );
		connect( duplicator, mutator );
		connect( mutator, collector, 1 );
		
	}
	
	/**
	 * @see org.evolvica.engine.IAlgorithm#teardown()
	 */
	public void teardown() {
		// no need to do anything here
	}
	
	/**
	 * Main method
	 * 
	 * @param args arguments
	 */
	public static void main( String[] args ) {
		new SphereAlgorithm().execute();
	}
	
}
