/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:35:29 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: TrivialAlgorithm.java,v $
 *
 */
package org.evolvica.examples.trivial;

import org.evolvica.core.common.Collector;
import org.evolvica.core.common.DefaultSink;
import org.evolvica.core.common.Duplicator;
import org.evolvica.core.eval.ReverseIntegerScoreComparator;
import org.evolvica.core.population.ElitistInsertion;
import org.evolvica.core.population.SortedPopulation;
import org.evolvica.core.terminate.ConditionalRouter;
import org.evolvica.core.terminate.CounterCondition;
import org.evolvica.engine.AbstractAlgorithm;

/** Trivial Algorithm.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:35:29 $
 */
public class TrivialAlgorithm extends AbstractAlgorithm {
	
	// a comparator which compares individuals by their (integer) scores
	// and assumes that a higher score is a better score: we need this comparator later!
	//	protected DefaultIntegerScoreComparator comparator = new DefaultIntegerScoreComparator();
	// try out what happens, if you change the comparator to this one:
	protected ReverseIntegerScoreComparator comparator = new ReverseIntegerScoreComparator();
	
	protected String sinkID;
	
	// in the setup method everything is done to create an algorithm
	public void setup() throws Exception {
		// first step: create instances of operators that are to be used
		TrivialInitializer init = new TrivialInitializer(); // a problem dependent initializer
		SortedPopulation population = new SortedPopulation(); // a population, that sorts its contents
		Duplicator duplicator = new Duplicator();
		ConditionalRouter router = new ConditionalRouter();
		Collector collector = new Collector();
		TrivialEvaluator eval = new TrivialEvaluator(); // our own evaluator
		TrivialMutator mutator = new TrivialMutator();
//		TrivialSink sink = new TrivialSink(); // and a sink that captures the results of the algorithm
		DefaultSink sink = new DefaultSink();
		sink.setComparator( comparator );
		// the initializer should create 10 individuals; the next line sets
		// the appropriate property
		init.setSetSize( 10 );
		// we need to tell the population how to sort the individuals
		// therefore we append our comparator
		population.setComparator( comparator );
		// we use elitist reinsertion for replacing individuals in our population
		population.setFilter( new ElitistInsertion( (float) 0.5, (float) 0.5, comparator ) );
		// we need a condition for our router: 1000 generations
		router.setCondition( new CounterCondition( 1000 ) );
		// now we need to connect all operators together
		// connect the output of the initializer to the first input of the collector
		connect( init, collector, 0 );
		// connect the collector to the evaluator
		connect( collector, eval );
		// evaluator to population
		connect( eval, population );
		// population to the router
		connect( population, router );
		// connect the 0-output of the evaluator to the input of the duplicator
		// the individuals will take this path if the condition in the router is false
		connect( router, 0, duplicator );
		// connect the 1-output of the evaluator to the input of the sink
		// the individuals will take this path if the condition in the router is true
		connect( router, 1, sink );
		// connect the duplicator to the mutator
		connect( duplicator, mutator );
		// and from the mutator back to the collector: this closes the loop!
		connect( mutator, collector, 1 );
		// THAT'S ALL !!!
		// remember the sink ID for later use
		this.sinkID = sink.getId();
		
	}
	
	// in the teardown method we can extract the results of our algorithm
	public void teardown() {
		/*
		// get our sink
		TrivialSink sink = (TrivialSink) getElementById( sinkID ).getOperator();
		// get the individuals from the sink
		IIndividual[] contents = sink.consumedIndividuals();
		// now search for the best individual
		IIndividual best = contents[0];
		for ( int i = 0; i < contents.length; i++ ) {
			if ( comparator.compare( best, contents[i] ) < 0 ) {
				best = contents[i];
			}
		}
		// print the result:
		EVLog.def.info( "Finished! The best individual had a score of " + best.getScore() );
		*/
	}
	
	// now create a main method to be able to run the algorithm
	public static void main( String[] args ) throws Exception {
		new TrivialAlgorithm().execute();
		//new AlgorithmWriter().write( algorithm );
		//IAlgorithm algorithm = new AlgorithmBuilder().build( "file:/c:/tmp/algorithm.xml" );
		//algorithm.execute();
		//System.out.println( algorithm.getClass().getName() );
		//SphereAlgorithm algorithm = new SphereAlgorithm();
		//new AlgorithmWriter().write( algorithm );
		//algorithm.execute();
	}
	
}
