/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:35:30 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: TrivialInitializer.java,v $
 *
 */
package org.evolvica.examples.trivial;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyVetoException;

import org.apache.commons.lang.RandomStringUtils;
import org.evolvica.core.IGenotype;
import org.evolvica.core.StringGene;
import org.evolvica.core.init.AbstractInitializer;

/** Trivial initializer example.
 *
 * @author Andreas Rummler
 * @since 0.4.0
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:35:30 $
 */
public class TrivialInitializer extends AbstractInitializer {
	
	// define a field string size with a default value
	protected int stringSize = 10;
	
	/**
	 * @see org.evolvica.core.init.AbstractInitializer#createGenotype()
	 */
	protected IGenotype createGenotype() {
		// method to create a single genotype, in this case a string
		// with random alphabetic characters of length <stringSize>
		String s = RandomStringUtils.randomAlphabetic( stringSize );
		// wrap this string in a genotype and return it
		return new StringGene( s );
	}
	
	// the getter method for the property stringSize
	// without this method it is impossible to access the property
	public int getStringSize() {
		return stringSize;
	}

	// the setter method for the property stringSize
	// this method is used by the GUI to set new values for this property
	// the name of such methods is always "set" + propertyName with the first
	// character uppercase: stringSize -> setStringSize( ... )
	// the same scheme is used in all get-methods
	// this set-method has a special feature in addition: a PropertVetoException
	// is thrown to indicate that someone tried to set an invalid value to this
	// property: the stringSize must always be > 0!
	public void setStringSize( int i ) throws PropertyVetoException {
		if ( i > 0 ) {
			stringSize = i;
		} else {
			throw new PropertyVetoException( "stringSize must be positive!", // an error message that is displayed in the GUI
				new PropertyChangeEvent(
					this, // the sender of this event (always "this")
					"stringSize", // the property that was attempted to be changed
					new Integer( stringSize ), // the old value of the property
					new Integer( i ) // the new value of the property
				)
			);
		}
		
	}

}
