/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:09 $
 * $Revision: 1.11 $
 * $State: Exp $
 * $RCSfile: AlgorithmEditorPalette.java,v $
 *
 */
package org.evolvica.ui;

import java.util.*;
import java.util.Iterator;
import java.util.List;
import java.util.TreeSet;

import org.apache.commons.lang.StringUtils;
import org.eclipse.gef.palette.*;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.resource.ImageDescriptor;
import org.evolvica.engine.OperatorDescriptor;
import org.evolvica.ui.prefs.PreferenceDefaults;

/** Documentation missing.
 *
 * @author rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.11 $ $Date: 2004/05/26 11:53:09 $
 */
public class AlgorithmEditorPalette extends PaletteRoot {
	
	private static final IPreferenceStore PREF_STORE = AlgorithmEditorPlugin.plugin().getPreferenceStore();
	
	private OperatorDescriptor[] visibleDescriptors = null;
	
	/**
	 * Constructor for EditorPalette.
	 */
	public AlgorithmEditorPalette( IJavaProject project ) {
		createToolsPalette();
		boolean showOperators = PREF_STORE.getBoolean( PreferenceDefaults.SHOW_OPERATORS );
		if ( showOperators == true ) {
			createOperatorPalette( project );
			// open the first operator drawer
			PaletteDrawer secondDrawer = (PaletteDrawer) children.get( 1 );
			if ( secondDrawer != null ) {
				secondDrawer.setInitialState( PaletteDrawer.INITIAL_STATE_OPEN );
			}
		}
	}

	protected void createToolsPalette() {
		PaletteDrawer toolsDrawer = new PaletteDrawer( "Tools" );
		toolsDrawer.add( new SelectionToolEntry( "Select", "Selects an operator" ) );
		toolsDrawer.add( new ConnectionCreationToolEntry(
			"Connect",
			"Connects operators",
			null,
			Images.getImageDescriptor( "icons/connect16.png" ),
			Images.getImageDescriptor( "icons/connect32.png" )
		) );
		// activate selection tool
		//((PaletteToolEntry) toolsPalette.getChildren().get( 0 )).setDefault( true );
		// open the drawer by default
		toolsDrawer.setInitialState( PaletteDrawer.INITIAL_STATE_OPEN );
		add( toolsDrawer );
	}
	
	protected void createOperatorPalette( IJavaProject project ) {
		visibleDescriptors = CacheManager.getDescriptors( project );
		String[] categories = getCategories();
		for ( int i = 0; i < categories.length; i++ ) {
			PaletteDrawer drawer = new PaletteDrawer( categories[i] );
			OperatorDescriptor[] desc = getDescriptorsInCategory( categories[i] );
			for ( int j = 0; j < desc.length; j++ ) {
				String displayName = desc[j].getDisplayName();
				String description = desc[j].getShortDescription();
				OperatorCreationFactory factory = new OperatorCreationFactory( desc[j], project );
				insertToolEntry( drawer, new CreationToolEntry( displayName, description, factory, getImage( desc[j] ), null ) );
			}
			drawer.setInitialState( PaletteDrawer.INITIAL_STATE_CLOSED );
			insertDrawer( drawer );
		}
	}
	
	public void refresh( IJavaProject project ) {
		// refresh cache
		visibleDescriptors = CacheManager.getDescriptors( project );
		// create new category drawers that may be needed
		String[] categories = CacheManager.getCategories( project );
		for ( int i = 0; i < categories.length; i++ ) {
			PaletteDrawer drawer = findDrawer( categories[i] );
			if ( drawer == null ) {
				drawer = new PaletteDrawer( categories[i] );
				insertDrawer( drawer );
			}
		}
		// create entries for new operators
		for ( int i = 0; i < visibleDescriptors.length; i++ ) {
			String category = visibleDescriptors[i].getCategory();
			String displayName = visibleDescriptors[i].getDisplayName();
			PaletteDrawer drawer = findDrawer( category );
			CreationToolEntry entry = findEntry( drawer, displayName );
			if ( entry == null ) {
				String description = visibleDescriptors[i].getShortDescription();
				OperatorCreationFactory factory = new OperatorCreationFactory( visibleDescriptors[i], project );
				insertToolEntry( drawer, new CreationToolEntry( displayName, description, factory, getImage( visibleDescriptors[i] ), null ) );
			}
		}
		// remove obsolete entries
		for ( int i = 1; i < getChildren().size(); i++ ) { // don't touch "Tools" Drawer
			PaletteDrawer drawer = (PaletteDrawer) getChildren().get( i );
			for ( int j = 0; j < drawer.getChildren().size(); j++ ) {
				Object next = drawer.getChildren().get( j );
				if ( next instanceof CreationToolEntry ) {
					CreationToolEntry entry = (CreationToolEntry) next;
					OperatorDescriptor desc = findDescriptor( drawer.getLabel(), entry.getLabel() );
					if ( desc == null ) {
						drawer.remove( entry );
					}
				}
			}
		}
		// remove emtpy category drawers
		for ( int i = 0; i < getChildren().size(); i++ ) {
			PaletteDrawer drawer = (PaletteDrawer) getChildren().get( i );
			if ( drawer.getChildren().size() == 0 ) {
				remove( drawer );
			}
		}
	}
	
	protected void insertDrawer( PaletteDrawer drawer ) {
		String label = drawer.getLabel();
		if ( children.size() == 1 ) {
			add( drawer );
			return;
		} else {
			for ( int i = 1; i < children.size(); i++ ) {
				String current = ((PaletteDrawer) children.get( i )).getLabel();
				if ( label.compareTo( current ) < 0 ) {
					add( i, drawer );
					return;
				}
			}
			add( drawer );
		}
	}
	
	protected void insertToolEntry( PaletteDrawer drawer, CreationToolEntry entry ) {
		String label = entry.getLabel();
		if ( drawer.getChildren().size() == 0 ) {
			drawer.add( entry );
			return;
		} else {
			List drawerChildren = drawer.getChildren();
			for ( int i = 0; i < drawerChildren.size(); i++ ) {
				String current = ((CreationToolEntry) drawerChildren.get( i )).getLabel();
				if ( label.compareTo( current ) < 0 ) {
					drawer.add( i, entry );
					return;
				}
			}
			drawer.add( entry );
		}
	}
	
	protected PaletteDrawer findToolsDrawer() {
		return findDrawer( "Tools" );
	}
	
	protected PaletteDrawer findDrawer( String category ) {
		for (Iterator it = children.iterator(); it.hasNext(); ) {
			PaletteDrawer drawer = (PaletteDrawer) it.next();
			if ( drawer.getLabel().equals( category ) ) {
				return drawer;
			}
		}
		return null;
	}
	
	protected CreationToolEntry findEntry( PaletteDrawer category, String displayname ) {
		if ( category == null || displayname == null ) {
			return null;
		} else {
			for (Iterator it = category.getChildren().iterator(); it.hasNext(); ) {
				CreationToolEntry entry = (CreationToolEntry) it.next();
				if ( entry.getLabel().equals( displayname ) ) {
					return entry;
				}
			}
		}
		return null;
	}
	
	protected OperatorDescriptor findDescriptor( String category, String displayname ) {
		for ( int i = 0; i < visibleDescriptors.length; i++ ) {
			if ( visibleDescriptors[i].getCategory().equals( category ) && visibleDescriptors[i].getDisplayName().equals( displayname ) ) {
				return visibleDescriptors[i];
			}
		}
		return null;
	}
	
	protected String[] getCategories() {
		TreeSet categories = new TreeSet();
		for ( int i = 0; i < visibleDescriptors.length; i++ ) {
			String category = visibleDescriptors[i].getCategory();
			if ( StringUtils.isNotEmpty( category ) ) {
				categories.add( category );
			} else {
				categories.add( "misc" );
			}
		}
		return (String[]) categories.toArray( new String[categories.size()] );
	}
	
	protected OperatorDescriptor[] getDescriptorsInCategory( String category ) {
		TreeSet set = new TreeSet( new DescriptorComparator() );
		for ( int i = 0; i < visibleDescriptors.length; i++ ) {
			if ( category.equals( visibleDescriptors[i].getCategory() ) ) {
				set.add( visibleDescriptors[i] );
			}
		}
		return (OperatorDescriptor[]) set.toArray( new OperatorDescriptor[set.size()] );
	}
	
	protected ImageDescriptor getImage( OperatorDescriptor descriptor ) {
		int inputs = descriptor.getInputSize();
		int outputs = descriptor.getOutputSize();
		if ( inputs == 0 && outputs == 1 ) {
			return Images.getImageDescriptor( "icons/source16.png" );
		} else if ( inputs == 1 && outputs == 0 ) {
			return Images.getImageDescriptor( "icons/sink16.png" );
		} else if ( inputs == 1 && outputs == 1 ) {
			return Images.getImageDescriptor( "icons/processor16.png" );
		} else if ( inputs == 1 && outputs > 1 ) {
			return Images.getImageDescriptor( "icons/fork16.png" );
		} else if ( inputs > 1 && outputs == 1 ) {
			return Images.getImageDescriptor( "icons/join16.png" );
		} else if ( inputs > 1 && outputs > 1 ) {
			return Images.getImageDescriptor( "icons/conduit16.png" );
		} else {
			return null;
		}
	}
	
	private static class DescriptorComparator implements Comparator {
		
		public int compare( Object o1, Object o2 ) {
			return ((OperatorDescriptor) o1).getDisplayName().compareTo( ((OperatorDescriptor) o2).getDisplayName() );
		}

	}
	
}
