package org.evolvica.ui;

import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPluginDescriptor;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.evolvica.ui.prefs.PreferenceDefaults;

/**
 * The main plugin class to be used in the desktop.
 */
public class AlgorithmEditorPlugin extends AbstractUIPlugin {
	
	public final static Clipboard clipboard = new Clipboard( null );
	
	private static AlgorithmEditorPlugin plugin = null;
	
	private ResourceBundle resourceBundle;
	
	/**
	 * The constructor.
	 */
	public AlgorithmEditorPlugin( IPluginDescriptor descriptor ) {
		super( descriptor );
		plugin = this;
		try {
			resourceBundle= ResourceBundle.getBundle( "org.evolvica.edt.AlgorithmEditorPluginResources" );
		} catch ( MissingResourceException x ) {
			resourceBundle = null;
		}
	}
	
	public void startup() {
		// init preference defaults
		PreferenceDefaults.init();
		// load cached operators
		DiskCache.load();
	}
	
	public void shutdown() {
		// save cached operators
		DiskCache.save();
	}
	
	/**
	 * Returns the shared instance.
	 */
	public static AlgorithmEditorPlugin getDefault() {
		return plugin;
	}
	
	public static AlgorithmEditorPlugin plugin() {
		return plugin;
	}
	
	/**
	 * Returns the workspace instance.
	 */
	public static IWorkspace getWorkspace() {
		return ResourcesPlugin.getWorkspace();
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle= AlgorithmEditorPlugin.getDefault().getResourceBundle();
		try {
			return bundle.getString(key);
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle,
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}
	
	public static final ImageDescriptor getImage( String name ) {
		return ImageDescriptor.createFromFile( AlgorithmEditorPlugin.class, name );
	}
	
}
