/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:10 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: CacheManager.java,v $
 * 
 */
package org.evolvica.ui;

import java.net.URI;
import java.util.*;

import org.eclipse.jdt.core.*;
import org.evolvica.engine.OperatorDescriptor;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:53:10 $
 */
public class CacheManager {
	
	public static final void clearCache() {
		DiskCache.clear();
	}
	
	public static final void clearCache( URI location ) {
		DiskCache.clear( location );
	}
	
	private static final OperatorDescriptor[] doGetDescriptors( IJavaProject project, boolean forceRefresh ) {
		HashSet set = new HashSet();
		URI[] classpath = PathManager.getClassPath( project );
		URI[] srcpath = PathManager.getSourcePath( project );
		// look in classpath of project
		for ( int i = 0; i < classpath.length; i++ ) {
			// only look at binary archives on classpath
			OperatorDescriptor[] desc = DiskCache.findDescriptors( classpath[i], srcpath, forceRefresh );
			for ( int j = 0; j < desc.length; j++ ) {
				set.add( desc[j] );
			}
		}
		return (OperatorDescriptor[]) set.toArray( new OperatorDescriptor[set.size()] );
	}
	
	private static final OperatorDescriptor[] doGetLocalDescriptors( IJavaProject project, boolean forceRefresh ) {
		return DiskCache.findDescriptors( PathManager.getOutputLocation( project ), PathManager.getSourcePath( project ), forceRefresh );
	}
	
	public static final String[] getCategories( IJavaProject project ) {
		return getCategories( project, false );
	}
	
	public static final String[] getCategories( IJavaProject project, boolean forceRefresh ) {
		TreeSet set = new TreeSet();
		OperatorDescriptor[] descriptors = getDescriptors( project, forceRefresh );
		for (int i = 0; i < descriptors.length; i++) {
			set.add( descriptors[i].getCategory() );
		}
		return (String[]) set.toArray( new String[set.size()] );
	}
	
	public static final OperatorDescriptor[] getDescriptors( IJavaProject project ) {
		return getDescriptors( project, false );
	}
	
	public static final OperatorDescriptor[] getDescriptors( IJavaProject project, boolean forceRefresh ) {
		OperatorDescriptor[] descriptors = doGetDescriptors( project, forceRefresh );
		Arrays.sort( descriptors );
		return descriptors;
	}
	
	public static final OperatorDescriptor[] getDescriptors( IJavaProject project, String category ) {
		return getDescriptors( project, category, false );
	}
	
	public static final OperatorDescriptor[] getDescriptors( IJavaProject project, String category, boolean forceRefresh ) {
		TreeSet set = new TreeSet();
		OperatorDescriptor[] all = getDescriptors( project, forceRefresh );
		for ( int i = 0; i < all.length; i++ ) {
			if ( all[i].getCategory().equals( category ) ) {
				set.add( all[i] );
			}
		}
		return (OperatorDescriptor[]) set.toArray( new OperatorDescriptor[set.size()] );
	}
	
	public static final OperatorDescriptor[] getLocalDescriptors( IJavaProject project ) {
		return getLocalDescriptors( project, false );
	}
	
	public static final OperatorDescriptor[] getLocalDescriptors( IJavaProject project, boolean forceRefresh ) {
		OperatorDescriptor[] descriptors = doGetLocalDescriptors( project, forceRefresh );
		Arrays.sort( descriptors );
		return descriptors;
	}
	
}
