/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:10 $
 * $Revision: 1.11 $
 * $State: Exp $
 * $RCSfile: ClassCellEditor.java,v $
 * 
 */
package org.evolvica.ui;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IType;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.viewers.DialogCellEditor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ElementListSelectionDialog;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.11 $ $Date: 2004/05/26 11:53:10 $
 */
public class ClassCellEditor extends DialogCellEditor {

	protected Composite parent = null;
	
	protected Class propertyType = null;
	
	public ClassCellEditor( Composite parent ) {
		this( parent, null );
	}
	
	public ClassCellEditor( Composite parent, Class propertyType ) {
		super( parent );
		this.parent = parent;
		this.propertyType = propertyType;
	}
	
	public void setPropertyType( Class type ) {
		this.propertyType = type;
	}
	
	/**
	 * @see org.eclipse.jface.viewers.DialogCellEditor#openDialogBox(org.eclipse.swt.widgets.Control)
	 */
	protected Object openDialogBox( Control cellEditorWindow ) {
		ProgressMonitorDialog monitor = new ProgressMonitorDialog( cellEditorWindow.getShell() );
		monitor.setCancelable( false );
		monitor.open();
		IType[] types = findImplementors( monitor.getProgressMonitor() );
		monitor.close();
		ElementListSelectionDialog dialog = new ElementListSelectionDialog( cellEditorWindow.getShell(), new ClassLabelProvider() );
		dialog.setElements( types );
		dialog.setTitle( "Select Class" );
		dialog.open();
		IType itype = (IType) dialog.getFirstResult();
		if ( itype != null ) {
			try {
				return Class.forName( itype.getFullyQualifiedName() ).newInstance(); // IMPROVE: use own classloader here?
			} catch ( Exception ex ) {
				UILog.error( "unable to instantiate class " + itype.getElementName(), ex );
			}
		}
		return null;
	}

	protected void updateContents( Object value ) {
		if ( value == null ) {
			super.updateContents( "null" );
		} else {
			super.updateContents( value );
		}
	}

	protected IType[] findImplementors( IProgressMonitor monitor ) {
		AlgorithmEditor editor = (AlgorithmEditor) PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		try {
			IJavaProject project = editor.project;
			IType type = project.findType( propertyType.getName() );
			IType[] implementors = ClassManager.findImplementors( project, type, monitor );
			return implementors;
		} catch ( Exception ex ) {
			return new IType[0];
		}
	}

}

