/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:10 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: ClassManager.java,v $
 * 
 */
package org.evolvica.ui;

import java.util.Comparator;
import java.util.TreeSet;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.*;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.ITypeHierarchy;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:53:10 $
 */
public class ClassManager {
	
	protected static final TypeComparator TYPE_COMPARATOR = new TypeComparator();
	
	public static final IType[] findImplementors( IJavaProject project, IType type ) {
		return findImplementors( project, type, null );
	}
	
	public static final IType[] findImplementors( IJavaProject project, IType type, IProgressMonitor monitor ) {
		TreeSet set = new TreeSet( TYPE_COMPARATOR );
		// build type hierarchy
		ITypeHierarchy hierarchy = null;
		try {
			hierarchy = type.newTypeHierarchy( project, monitor );
		} catch ( JavaModelException jmx ) {
			UILog.error( "error creating type hierarchy for type " + type + " in project " + project, jmx );
			return new IType[0];
		}
		// try to add the given type
		try {
			int flags = type.getFlags();
			if ( ! Flags.isAbstract( flags ) && ! Flags.isInterface( flags ) ) {
				IMethod constructor = type.getMethod( type.getElementName(), new String[] { } );
				if ( Flags.isPublic( constructor.getFlags() ) ) {
					set.add( type );
				} else {
					UILog.debug( "can't add type " + type + " to hierarchy: type is abstract, interface or has no public constructor" );
				}
			}
		} catch ( JavaModelException jmx ) {
			UILog.debug( "can't add type " + type + " to hierarchy: error accessing type", jmx );
		}
		// try to add all the subtypes
		IType[] implementors = hierarchy.getAllSubtypes( type );
		for ( int i = 0; i < implementors.length; i++ ) {
			try {
				int flags = implementors[i].getFlags();
				if ( ! Flags.isAbstract( flags ) && ! Flags.isInterface( flags ) ) {
					IMethod constructor = implementors[i].getMethod( implementors[i].getElementName(), new String[] { } );
					if ( Flags.isPublic( constructor.getFlags() ) ) {
						set.add( implementors[i] );
					} else {
						UILog.warn( "type " + implementors[i].getElementName() + " unused: no public and/or parameterless constructor found" );
					}
				} else {
					UILog.debug( "type " + implementors[i].getElementName() + " unused: operator is abstract or interface" );
				}
			} catch ( JavaModelException jmx ) {
				UILog.debug( "can't access type " + implementors[i], jmx );
			}
		}
		return (IType[]) set.toArray( new IType[set.size()] );
	}
	
	public static final boolean extendsClass( IType type, String baseclass ) {
		try {
			ITypeHierarchy hierarchy = type.newSupertypeHierarchy( null );
			IType[] baseclasses = hierarchy.getAllSuperclasses( type );
			for ( int i = 0; i < baseclasses.length; i++ ) {
				if ( baseclasses[i].getFullyQualifiedName().equals( baseclass ) ) {
					return true;
				}
			}
		} catch ( JavaModelException jmx ) {
			// do nothing
		}
		return false;
	}
	
	public static final boolean implementsInterface( IType type, String interfaze ) {
		try {
			ITypeHierarchy hierarchy = type.newSupertypeHierarchy( null );
			IType[] interfaces = hierarchy.getAllInterfaces();
			for ( int i = 0; i < interfaces.length; i++ ) {
				if ( interfaces[i].getFullyQualifiedName().equals( interfaze ) ) {
					return true;
				}
			}
		} catch ( JavaModelException jmx ) {
			// do nothing
		}
		return false;
	}
	
	public static final boolean isInstantiatable( IType type ) {
		try {
			int flags = type.getFlags();
			if ( ! Flags.isInterface( flags ) && ! Flags.isAbstract( flags ) ) {
				IMethod constructor = type.getMethod( type.getElementName(), new String[] { } );
				if ( Flags.isPublic( constructor.getFlags() ) ) {
					return true;
				}
			}
		} catch ( JavaModelException jmx ) {
			// do nothing
		}
		return false;
	}
	
	// comparator for ITypes
	
	private static class TypeComparator implements Comparator {
		
		public int compare( Object o1, Object o2 ) {
			IType t1 = (IType) o1;
			IType t2 = (IType) o2;
			int compare = t1.getElementName().compareTo( t2.getElementName() );
			if ( compare == 0 ) {
				return t1.getFullyQualifiedName().compareTo( t2.getFullyQualifiedName() );
			} else {
				return compare;
			}
		}

	}
	
}
