/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:13 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: OperatorCreationFactory.java,v $
 *
 */
package org.evolvica.ui;

import java.net.URI;
import java.net.URL;
import java.net.URLClassLoader;

import org.eclipse.gef.requests.CreationFactory;
import org.eclipse.jdt.core.IJavaProject;
import org.evolvica.engine.DescriptorResolver;
import org.evolvica.engine.IOperator;
import org.evolvica.engine.OperatorDescriptor;

/** Documentation missing.
 *
 * @author rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:53:13 $
 */
public class OperatorCreationFactory implements CreationFactory {
	
	protected IJavaProject project = null;
	
	protected OperatorDescriptor descriptor = null;
	
	protected URI[] classpath;
	
	protected URLClassLoader loader = null;
	
	public OperatorCreationFactory( OperatorDescriptor descriptor, IJavaProject project ) {
		this.descriptor = descriptor;
		this.project = project;
		try {
			classpath = PathManager.getClassPath( project );
			// this.descriptor.resolve( classpath );
			URL[] urlClasspath = new URL[classpath.length];
			for ( int i = 0; i < urlClasspath.length; i++ ) {
				urlClasspath[i] = classpath[i].toURL();
			}
			loader = URLClassLoader.newInstance( urlClasspath, AlgorithmEditorPlugin.getDefault().getDescriptor().getPluginClassLoader() );
		} catch ( Exception ex ) {
			UILog.error( "error resolving descriptor " + descriptor.getBeanClassName() + " in project " + project.getElementName(), ex );
		}
	}
	
	/**
	 * @see org.eclipse.gef.requests.CreateRequest.Factory#getNewObject()
	 */
	public Object getNewObject() {
		if ( ! descriptor.isResolved() ) {
			DescriptorResolver.resolve( descriptor, loader );
		}
		try {
			OperatorDescriptor copy = DescriptorResolver.copy( descriptor );
			IOperator operator = (IOperator) copy.getBeanClass().newInstance();
			operator.attachDescriptor( copy );
			return operator;
		} catch ( Exception ex ) {
			UILog.error( "error instantiating operator " + descriptor.getBeanClassName() + " in project " + project.getElementName(), ex );
			return null;
		}
		/* old version
		try {
			OperatorDescriptor copy = descriptor.getResolved( classpath );
			IOperator operator = (IOperator) loader.loadClass( descriptor.getBeanClassName() ).newInstance();
			operator.attachDescriptor( copy );
			return new OperatorModel( operator );
		} catch ( Exception ex ) {
			UILog.error( "error resolving operator " + descriptor.getBeanClassName() + " in project " + project.getElementName(), ex );
			return null;
		}
		*/
	}

	/**
	 * @see org.eclipse.gef.requests.CreateRequest.Factory#getObjectType()
	 */
	public Object getObjectType() {
		return descriptor.getBeanClassName();
	}
	
	public static IOperator createOperator( OperatorDescriptor descriptor, IJavaProject project ) throws Exception {
		// examine classpath of project + create appropriate classloader
		URI[] classpath = PathManager.getClassPath( project );
		URL[] urlClasspath = new URL[classpath.length];
		for ( int i = 0; i < urlClasspath.length; i++ ) {
			urlClasspath[i] = classpath[i].toURL();
		}
		URLClassLoader loader = URLClassLoader.newInstance( urlClasspath, AlgorithmEditorPlugin.getDefault().getDescriptor().getPluginClassLoader() );
		// make a copy of the descriptor and try to resolve it
		OperatorDescriptor copy = DescriptorResolver.copy( descriptor );
		if ( ! copy.isResolved() ) {
			DescriptorResolver.resolve( copy, loader );
		}
		// finally try to instatiate the new operator
		IOperator operator = (IOperator) copy.getBeanClass().newInstance();
		operator.attachDescriptor( copy );
		return operator;
	}
	
}
