/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/06/14 10:59:03 $
 * $Revision: 1.10 $
 * $State: Exp $
 * $RCSfile: PathManager.java,v $
 *
 */
package org.evolvica.ui;

import java.net.URI;
import java.util.LinkedList;
import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jdt.core.*;

/**
 * Documentation missing.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.10 $ $Date: 2004/06/14 10:59:03 $
 * @since RELEASE_NUMBER_NOT_DETERMINED
 */
public class PathManager {
	
	public static final URI[] getClassPath( IJavaProject project ) {
		List classpath = new LinkedList();
		IPackageFragmentRoot[] fragmentRoots = null;
		try {
			fragmentRoots = project.getAllPackageFragmentRoots();	
		} catch ( JavaModelException jmx ) {
			UILog.error( "unable to access project classpath", jmx );
			return new URI[0];
		}
		// examine package fragments, add them if them are in binary form (skip source fragments)
		for ( int i = 0; i < fragmentRoots.length; i++ ) {
			try {
				if ( fragmentRoots[i].getKind() == IPackageFragmentRoot.K_BINARY ) {
					URI uri = getLocation( fragmentRoots[i] );
					if ( uri != null ) {
						classpath.add( uri );
					}
				}
			} catch ( JavaModelException jmx ) {
				UILog.error( "error accessing package fragment root kind", jmx );
			}
		}
		// add the output location to the classpath
		URI output = getOutputLocation( project );
		if ( output != null ) {
			classpath.add( output );
		}
		return (URI[]) classpath.toArray( new URI[classpath.size()] );
	}
	
	public static final String getFullyQualifiedName( ICompilationUnit unit ) {
		String name = null;
		try {
			if ( unit.getPackageDeclarations().length > 0 ) {
				name = unit.getPackageDeclarations()[0].getElementName() + "." + unit.getElementName();
			} else {
				name = unit.getElementName();
			}
			name = StringUtils.chompLast( name, ".java" );
		} catch ( JavaModelException jmx ) {
			UILog.error( "error determining fully qualified name", jmx );
		}
		return name;
	}
	
	public static final URI getLocation( ICompilationUnit unit ) {
		IPath unitPath = unit.getJavaProject().getProject().getLocation().removeLastSegments( 1 );
		try {
			String packageName = "";
			if ( unit.getPackageDeclarations().length > 0 ) {
				packageName = unit.getPackageDeclarations()[0].getElementName();
			}
			int packageLength = StringUtils.split( packageName, "." ).length;
			return unitPath.append( unit.getPath() ).removeLastSegments( packageLength + 1 ).toFile().toURI();
		} catch ( JavaModelException jmx ) {
			UILog.error( "error determining path to compilation unit", jmx );
			return unitPath.toFile().toURI(); // IMPROVE: return null here?
		}
	}
	
	public static final URI getLocation( IFile file ) {
		return file.getLocation().toFile().toURI();
	}
	
	public static final URI getLocation( IPackageFragmentRoot root ) {
		URI uri = null;
		if ( root.isExternal() ) {
			uri = root.getPath().toFile().toURI();
		} else {
			IPath projectPath = root.getJavaProject().getProject().getLocation().removeLastSegments( 1 );
			uri = projectPath.append( root.getPath() ).toFile().toURI();
		}
		return uri;
	}
	
	public static final URI getOutputLocation( IJavaProject project ) {
		IPath projectPath = project.getProject().getLocation().removeLastSegments( 1 );
		try {
			return projectPath.append( project.getOutputLocation() ).toFile().toURI();
		} catch ( JavaModelException jmx ) {
			UILog.error( "error determining output location of Java project", jmx );
			return projectPath.toFile().toURI(); // IMPROVE: return null here?
		}
	}
	
	public static final URI[] getSourcePath( IJavaProject project ) {
		List classpath = new LinkedList();
		IPackageFragmentRoot[] fragmentRoots = null;
		try {
			fragmentRoots = project.getAllPackageFragmentRoots();	
		} catch ( JavaModelException jmx ) {
			UILog.error( "unable to access project classpath", jmx );
			return new URI[0];
		}
		// examine package fragments, add them if them are in source form (skip binary fragments)
		for ( int i = 0; i < fragmentRoots.length; i++ ) {
			try {
				if ( fragmentRoots[i].getKind() == IPackageFragmentRoot.K_SOURCE ) {
					URI uri = getLocation( fragmentRoots[i] );
					if ( uri != null ) {
						classpath.add( uri );
					}
				}
			} catch ( JavaModelException jmx ) {
				UILog.error( "error accesing package fragment root kind", jmx );
			}
		}
		return (URI[]) classpath.toArray( new URI[classpath.size()] );
	}
	
}
