/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:14 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: SourceManager.java,v $
 * 
 */
package org.evolvica.ui;

import java.util.Arrays;
import java.util.LinkedList;

import org.apache.commons.lang.StringUtils;
import org.eclipse.jdt.core.*;
import org.evolvica.ui.editors.operator.CompilationUnitPropertyDescriptor;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:53:14 $
 */
public class SourceManager {
	
	public static final boolean isOperator( ICompilationUnit unit ) {
		try {
			IType[] types = unit.getTypes();
			if ( types.length > 0 && ClassManager.implementsInterface( types[0], "org.evolvica.engine.IOperator" ) ) {
				return true;
			}
		} catch ( JavaModelException jmx ) {
			//
		}
		return false;
	}
	
	public static final String getName( ICompilationUnit unit ) {
		IType type = getType( unit );
		if ( type != null ) {
			return type.getElementName();
		} else {
			return "???";
		}
	}
	
	public static final String getQualifiedName( ICompilationUnit unit ) {
		IType type = getType( unit );
		if ( type != null ) {
			return type.getFullyQualifiedName();
		} else {
			return "???";
		}
	}
	
	public static final IType getType( ICompilationUnit unit ) {
		try {
			IType[] types = unit.getTypes();
			if ( types.length > 0 ) {
				return types[0];
			}
		} catch ( JavaModelException jmx ) {
			//
		}
		return null;
	}
	
	public static CompilationUnitPropertyDescriptor[] getProperties( ICompilationUnit unit ) {
		LinkedList list = new LinkedList();
		IType type = getType( unit );
		while( type != null ) {
			try {
				IMethod[] methods = type.getMethods();
				for ( int i = 0; i < methods.length; i++ ) {
					list.add( methods[i] );
				}
				String superclassname = type.getSuperclassName();
				if ( superclassname == null ) {
					superclassname = "Object";
				}
				String[][] array = type.resolveType( superclassname );
				if ( array!= null && array.length > 0 ) {
					String supertype = array[0][0] + "." + array[0][1];
					type = unit.getJavaProject().findType( supertype );
				} else {
					// try a second approach: try to find type directly
					if ( superclassname != null ) {
						type = unit.getJavaProject().findType( superclassname );
					} else {
						type = null;
					}
				}
			} catch ( JavaModelException jmx ) {
				type = null;
			}
		}
		return analyzeMethods( (IMethod[]) list.toArray( new IMethod[list.size()] ) );
	}
	
	private static final CompilationUnitPropertyDescriptor[] analyzeMethods( IMethod[] allMethods ) {
		IMethod[] getMethods = findGetMethods( allMethods );
		IMethod[] setMethods = findSetMethods( allMethods );
		CompilationUnitPropertyDescriptor[] descriptors = new CompilationUnitPropertyDescriptor[getMethods.length];
		for ( int i = 0; i < getMethods.length; i++ ) {
			IMethod get = getMethods[i];
			IMethod set = null;
			String property = null;
			if ( get.getElementName().startsWith( "get" ) ) {
				property = get.getElementName().substring( 3 );
			} else {
				property = get.getElementName().substring( 2 );
			}
			for ( int j = 0; j < setMethods.length; j++ ) {
				if ( setMethods[j].getElementName().equals( "set" + property ) ) {
					set = setMethods[j];
				}
			}
			descriptors[i] = new CompilationUnitPropertyDescriptor( get, set );
		}
		Arrays.sort( descriptors );
		return descriptors;
	}
	
	private static final IMethod[] findGetMethods( IMethod[] methods ) {
		LinkedList list = new LinkedList();
		for ( int i = 0; i < methods.length; i++ ) {
			String name = methods[i].getElementName();
			boolean isCandidate = false;
			if ( name.startsWith( "get" ) && name.length() > 3 && Character.isUpperCase( name.charAt( 3 ) ) ) {
				isCandidate = true;
			}
			if ( name.startsWith( "is" ) && name.length() > 2 && Character.isUpperCase( name.charAt( 2 ) ) ) {
				isCandidate = true;
			}
			try {
				if ( isCandidate == true && Flags.isPublic( methods[i].getFlags() ) && methods[i].getParameterNames().length == 0 ) {
					list.add( methods[i] );
				}
			} catch ( JavaModelException jmx ) {
				// be quiet
			}

		}
		return (IMethod[]) list.toArray( new IMethod[list.size()] );
	}
	
	private static final IMethod[] findSetMethods( IMethod[] methods ) {
		LinkedList list = new LinkedList();
		for ( int i = 0; i < methods.length; i++ ) {
			String name = methods[i].getElementName();
			boolean isCandidate = false;
			if ( name.startsWith( "set" ) && name.length() > 3 && Character.isUpperCase( name.charAt( 3 ) ) ) {
				try {
					if ( Flags.isPublic( methods[i].getFlags() ) && methods[i].getParameterNames().length == 1 ) {
						list.add( methods[i] );
					}
				} catch ( JavaModelException jmx ) {
					// be quiet
				}
			}

		}
		return (IMethod[]) list.toArray( new IMethod[list.size()] );
	}
	
	private static final String getPropertyName( IMethod method ) {
		String propertyName = null;
		if ( method.getElementName().startsWith( "get" ) ) {
			propertyName = method.getElementName().substring( 3 );
		} else if ( method.getElementName().startsWith( "set" ) ) {
			propertyName = method.getElementName().substring( 3 );
		} else if ( method.getElementName().startsWith( "is" ) ) {
			propertyName = method.getElementName().substring( 2 );
		} else {
			return "";
		}
		String upper = propertyName.substring( 0, 1 );
		String lower = upper.toLowerCase();
		return StringUtils.replaceOnce( propertyName, upper, lower );
	}
	
}
