/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:14 $
 * $Revision: 1.3 $
 * $State: Exp $
 * $RCSfile: UILog.java,v $
 * 
 */
package org.evolvica.ui;

import java.util.Enumeration;

import org.apache.log4j.Appender;
import org.apache.log4j.ConsoleAppender;
import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Platform;
import org.evolvica.util.EVLog;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.3 $ $Date: 2004/05/26 11:53:14 $
 */
public class UILog {
	
	public static Logger ui = Logger.getLogger( "evolvica.ui" );
	
	static {
		EVLog.debug( "appending logger for User Interface to Evolvica Log Facility" );
		boolean consolelog = false;
		boolean debug = false;
		String[] args = Platform.getCommandLineArgs();
		for ( int i = 0; i < args.length; i++ ) {
			if ( args[i].equals( "-consolelog" ) ) {
				consolelog = true;
			} else if ( args[i].equals( "-debug" ) ) {
				debug = true;
			}
		}
		if ( consolelog == false ) { // we're not logging to the console, so replace the console logger
			Logger root = Logger.getRootLogger(); // remove console logger to prevent printing messages twice
			for ( Enumeration enum = root.getAllAppenders(); enum.hasMoreElements(); ) {
				Object next = enum.nextElement();
				if ( next instanceof ConsoleAppender ) {
					 root.removeAppender( (Appender) next );
				}
			}
			// add platform log appender to see generated messages in the logfile
			root.addAppender( new PlatformLogAppender() );
		}
	}
	
	/** Logs a debug message.
	*
	* @param message message
	*/
	public static void debug( String message ) {
		ui.debug( message );
	}

	/** Logs a debug message.
	*
	* @param message message
	* @param throwable throwable causing this message
	*/
	public static void debug( String message, Throwable throwable ) {
		ui.debug( message, throwable );
	}

	/** Logs an error message.
	*
	* @param message message
	*/
	public static void error( String message ) {
		ui.error( message );
	}

	/** Logs an error message.
	*
	* @param message message
	* @param throwable throwable causing this message
	*/
	public static void error( String message, Throwable throwable ) {
		ui.error( message, throwable );
	}

	/** Logs a fatal condition.
	*
	* @param message message
	*/
	public static void fatal( String message ) {
		ui.fatal( message );
	}

	/** Logs a fatal condition.
	*
	* @param message message
	* @param throwable throwable causing this message
	*/
	public static void fatal( String message, Throwable throwable ) {
		ui.fatal( message, throwable );
	}

	/** Logs an info message.
	*
	* @param message message
	*/
	public static void info( String message ) {
		ui.info( message );
	}

	/** Logs an info message.
	*
	* @param message message
	* @param throwable throwable causing this message
	*/
	public static void info( String message, Throwable throwable ) {
		ui.info( message, throwable );
	}

	/** Logs a warning message.
	*
	* @param message message
	*/
	public static void warn( String message ) {
		ui.warn( message );
	}

	/** Logs a warning message.
	*
	* @param message message
	* @param throwable throwable causing this message
	*/
	public static void warn( String message, Throwable throwable ) {
		ui.warn( message, throwable );
	}
	
}
