/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:14 $
 * $Revision: 1.4 $
 * $State: Exp $
 * $RCSfile: ValidatingTextCellEditor.java,v $
 *
 */
package org.evolvica.ui;

import java.beans.PropertyVetoException;
import java.lang.reflect.InvocationTargetException;

import org.apache.commons.beanutils.ConversionException;
import org.apache.commons.beanutils.PropertyUtils;
import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.widgets.Composite;
import org.evolvica.engine.AttributeDescriptor;
import org.evolvica.util.Convert;

/**
 * Documentation missing.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.4 $ $Date: 2004/05/26 11:53:14 $
 * @since RELEASE_NUMBER_NOT_DETERMINED
 */
public class ValidatingTextCellEditor extends TextCellEditor {
	
	protected Object bean = null;

	protected AttributeDescriptor descriptor = null;

	public ValidatingTextCellEditor( Composite parent, Object bean, AttributeDescriptor descriptor ) {
		super( parent );
		this.bean = bean;
		this.descriptor = descriptor;
		this.setValidator( new Validator() );
	}
	
	protected class Validator implements ICellEditorValidator {
		
		public String isValid( Object value ) {
			try {
				String property = descriptor.getName();
				// save old value
				Object oldValue = PropertyUtils.getProperty( bean, property );
				// try to set the new value
				Class type = PropertyUtils.getPropertyType( bean, property );
				Object convertedValue = Convert.toObject( (String) value, type );
				PropertyUtils.setProperty( bean, property, convertedValue );
				// restore old value
				PropertyUtils.setProperty( bean, property, oldValue );
				return null;
			} catch ( ConversionException cx ) {
				return "entered string cannot be converted to the datatype of the property";
			} catch ( NoSuchMethodException nsmx ) {
				return "property not readable/writable";
			} catch ( IllegalAccessException iax ) {
				return "property not accessible";
			} catch ( InvocationTargetException itx ) {
				Throwable t = itx.getCause();
				if ( t instanceof PropertyVetoException ) {
					return ((PropertyVetoException) t).getMessage();
				} else {
					return "property not accessible";
				}
			}
		}

}
	
}
