/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:15 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: CreateDescriptorAction.java,v $
 *
 */
package org.evolvica.ui.action;

import java.io.ByteArrayInputStream;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.evolvica.engine.IDatatype;
import org.evolvica.ui.ClassManager;
import org.evolvica.util.Convert;

/**
 * Documentation missing.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:53:15 $
 * @since RELEASE_NUMBER_NOT_DETERMINED
 */
public class CreateDescriptorAction implements IObjectActionDelegate {
	
	private IWorkbenchPart targetPart = null;
	private ISelection selection = null;
	
	/**
	 * Constructor for Action.
	 */
	public CreateDescriptorAction() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart( IAction action, IWorkbenchPart targetPart ) {
		this.targetPart = targetPart;
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run( IAction action ) {
		String message = null;
		IFile descriptorFile = null;
		if ( selection != null && selection.isEmpty() == false && selection instanceof IStructuredSelection ) {
			IFile file = (IFile) ((IStructuredSelection) selection).getFirstElement();
			ICompilationUnit unit = JavaCore.createCompilationUnitFrom( file );
			try {
				IType[] types = unit.getTypes();
				if ( types.length > 0 ) {
					if ( ClassManager.implementsInterface( types[0], "org.evolvica.engine.IOperator" ) ) {
						IPath descriptorPath = file.getFullPath().removeFirstSegments( 1 ).removeFileExtension().addFileExtension( "descriptor" );
						descriptorFile = file.getProject().getFile( descriptorPath );
						if ( ! descriptorFile.exists() ) { // descriptor file does not exist -> create it!
							StringBuffer buffer = new StringBuffer();
							buffer.append( "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n" );
							buffer.append( "<operator" );
							buffer.append( " beanClass=\"" + unit.findPrimaryType().getFullyQualifiedName() + "\"" );
							buffer.append( " datatype=\"" + Convert.toClassname( IDatatype.UNIVERSAL.getReferencedClass() ) + "\"" );
							buffer.append( " category=\"misc\"" );
							buffer.append( " displayName=\"" + unit.findPrimaryType().getElementName() + "\"" );
							buffer.append( " expert=\"false\"" );
							buffer.append( " hidden=\"false\"" );
							buffer.append( " inputSize=\"0\"" );
							buffer.append( " outputSize=\"0\"" );
							buffer.append( " preferred=\"false\"" );
							buffer.append( " shortDescription=\"no description available\"" );
							buffer.append( " vendor=\"Evolvica.org\"" );
							buffer.append( ">\n" );
							buffer.append( "</operator>\n" );
							ByteArrayInputStream stream = new ByteArrayInputStream( buffer.toString().getBytes() );
							descriptorFile.create( stream, true, null );
							stream.close();
							descriptorFile.getProject().getFile( descriptorFile.getFullPath().removeLastSegments( 1 ) ).refreshLocal( IResource.DEPTH_ONE, null );
						}
					} else {
						message = "selected Java resource is not an evolutionary operator";
					}
				}
			} catch ( Exception ex ) {
				ex.printStackTrace();
				message = "error accessing Java resource";
			}
		} else {
			message = "selected resource is not a Java resource";
		}
		if ( descriptorFile != null && message == null ) {
			try {
				targetPart.getSite().getPage().openEditor( descriptorFile, "org.evolvica.ui.DescriptorEditor", true );
			} catch ( PartInitException pmx ) {
				message = pmx.getMessage();
			}
		}
		if ( message != null ) {
			MessageDialog.openError( targetPart.getSite().getShell(), "Error", message );
		}
	}
	
	/*
	protected void writeDescriptor( IFile file, CreateDescriptorDialog dialog ) {
		try {
			dialog.getDescriptor().serialize();
			IProject project = file.getProject();
			IPath parentPath = file.getFullPath().removeFirstSegments( 1 );
			parentPath = parentPath.removeLastSegments( 1 );
			IFile parent = project.getFile( parentPath );
			parent.refreshLocal( IResource.DEPTH_ONE, null );
		} catch ( Exception ex ) {
			ex.printStackTrace();
			MessageDialog.openError( targetPart.getSite().getShell(), "Error", "I/O error while writing descriptor file" );
		}
	}
	*/
	
	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged( IAction action, ISelection selection ) {
		this.selection = selection;
	}

}
