/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:15 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: RunAlgorithmAction.java,v $
 *
 */
package org.evolvica.ui.action;

import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.debug.core.*;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.launching.IJavaLaunchConfigurationConstants;
import org.eclipse.jdt.launching.IRuntimeClasspathEntry;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.FileEditorInput;
import org.evolvica.engine.IValidator;
import org.evolvica.ui.AlgorithmEditor;

/**
 * Documentation missing.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:53:15 $
 * @since RELEASE_NUMBER_NOT_DETERMINED
 */
public class RunAlgorithmAction extends Action {
	
	public RunAlgorithmAction() {
		super( "runalgorithm" );
		setText( "Run Algorithm" );
	}
	
	public void run() {
		AlgorithmEditor editor = (AlgorithmEditor) PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		if ( editor.isDirty() ) {
			boolean save = MessageDialog.openQuestion( editor.getSite().getShell(), "Save Algorithm?", "Algorithm is modified, save it to disk?" );
			if ( save == true ) {
				editor.doSave( null );
			}
		}
		IValidator validator = editor.getAlgorithm().getValidator();
		if ( validator != null ) {
			boolean isValid = validator.validate( editor.getAlgorithm() );
			if ( isValid == false ) {
				boolean run = MessageDialog.openQuestion( editor.getSite().getShell(), "Run Algorithm?", "Algorithm is invalid, run it anyway?" );
				if ( run == false ) {
					return;
				}
			}
		}
		try {
			// get java project
			IProject project = ((FileEditorInput) editor.getEditorInput()).getFile().getProject();
			IJavaProject javaProject = JavaCore.create( project );
			// create launch configuration
			ILaunchManager manager = DebugPlugin.getDefault().getLaunchManager();
			ILaunchConfigurationType type = manager.getLaunchConfigurationType( IJavaLaunchConfigurationConstants.ID_JAVA_APPLICATION );
			String configName = editor.getAlgorithm().getName();
			ILaunchConfigurationWorkingCopy copy = type.newInstance( null, configName );
			// configure
			copy.setAttribute( IJavaLaunchConfigurationConstants.ATTR_PROJECT_NAME, javaProject.getElementName() );
			copy.setAttribute( IJavaLaunchConfigurationConstants.ATTR_MAIN_TYPE_NAME, "org.evolvica.engine.Run" );
			copy.setAttribute( IJavaLaunchConfigurationConstants.ATTR_DEFAULT_CLASSPATH, false );
			// build classpath
			List classpath = new LinkedList();
			resolveProjectClasspath( javaProject, classpath );
			copy.setAttribute( IJavaLaunchConfigurationConstants.ATTR_CLASSPATH, classpath );
			// build command line
			String file = editor.getEditorInput().getName();
			copy.setAttribute( IJavaLaunchConfigurationConstants.ATTR_PROGRAM_ARGUMENTS, "-f " + file );
			// save changes & do a launch
			ILaunchConfiguration config = copy.doSave();
			config.launch( ILaunchManager.RUN_MODE, null );
		} catch ( Exception ex ) {
			ex.printStackTrace();
		}
	}
	
	protected void resolveProjectClasspath( IJavaProject project, List classpath ) {
		try {
			IRuntimeClasspathEntry[] unresolved = JavaRuntime.computeUnresolvedRuntimeClasspath( project );
			for ( int i = 0; i < unresolved.length; i++ ) {
				classpath.add( unresolved[i].getMemento() );
			}
		} catch ( Exception ex ) {
			ex.printStackTrace();
		}
	}
	
}
