/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:16 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: SetInputAction.java,v $
 * 
 */
package org.evolvica.ui.action;

import java.lang.reflect.Method;

import org.apache.commons.beanutils.MethodUtils;
import org.apache.commons.lang.StringUtils;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.ui.actions.SelectionAction;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.ui.IWorkbenchPart;
import org.evolvica.engine.IOperator;
import org.evolvica.ui.parts.OperatorEditPart;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:53:16 $
 */
public class SetInputAction extends SelectionAction {

	public static final String ID = SetInputAction.class.getName();
	
	public SetInputAction( IWorkbenchPart part ) {
		super( part );
		setId( ID );
		setText( "Set Input Size" );
	}
	
	/**
	 * @see org.eclipse.gef.ui.actions.EditorPartAction#calculateEnabled()
	 */
	protected boolean calculateEnabled() {
		if ( getSelectedObjects().isEmpty() ) {
			return false;
		}
		if ( getSelectedObjects().get( 0 ) instanceof OperatorEditPart ) {
			IOperator operator = ((OperatorEditPart) getSelectedObjects().get( 0 )).getElement().getOperator();
			Method method = MethodUtils.getAccessibleMethod( operator.getClass(), "setInputSize", Integer.TYPE );
			if ( method == null ) {
				return false;
			} else {
				return true;
			}
		} else {
			return false;
		}
	}
	
	public void run() {
		OperatorEditPart part = (OperatorEditPart) getSelectedObjects().get( 0 );
		IOperator operator = part.getElement().getOperator();
		InputDialog dialog = new InputDialog(
			getEditorPart().getSite().getShell(), "Set Input Size", "Enter a new value for the number of inputs",
			String.valueOf( part.getElement().incomingSize() ), new InputSizeValidator()
		);
		int rval = dialog.open();
		if ( rval == InputDialog.OK ) {
			part.getElement().setIncomingSize( Integer.parseInt( dialog.getValue() ) );
			// refresh selection
			part.setSelected( EditPart.SELECTED_NONE );
			part.setSelected( EditPart.SELECTED_PRIMARY );
		}
	}
	
	protected class InputSizeValidator implements IInputValidator {

		public String isValid( String newText ) {
			if ( StringUtils.isEmpty( newText ) ) {
				return "input size may not be empty";
			}
			try {
				int size = Integer.parseInt( newText );
				if ( size > 9 ) {
					return "You're joking! I don't believe that you need an operator with " + size + " inputs.";
				}
				if ( size < 1 ) {
					return "input size may not be smaller than 1.";
				}
				return null;
			} catch ( Exception ex ) {
				return "given input size is not a number";
			}
		}
		
	}
	
}
