/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:23 $
 * $Revision: 1.3 $
 * $State: Exp $
 * $RCSfile: CompilationUnitPropertyDescriptor.java,v $
 *
 */
package org.evolvica.ui.editors.operator;

import org.apache.commons.lang.StringUtils;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.Signature;
import org.evolvica.ui.SourceManager;

/**
 * Documentation missing.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.3 $ $Date: 2004/05/26 11:53:23 $
 * @since RELEASE_NUMBER_NOT_DETERMINED
 */
public class CompilationUnitPropertyDescriptor implements Comparable {
	
	protected IMethod getMethod = null;
	
	protected IMethod setMethod = null;
	
	public CompilationUnitPropertyDescriptor( IMethod getMethod, IMethod setMethod ) {
		this.getMethod = getMethod;
		this.setMethod = setMethod;
	}
	
	public boolean isReadonly() {
		if ( setMethod == null ) {
			return true;
		} else {
			return false;
		}
	}
	
	public String getPropertyName() {
		String propertyName = null;
		if ( getMethod.getElementName().startsWith( "get" ) ) {
			propertyName = getMethod.getElementName().substring( 3 );
		} else if ( getMethod.getElementName().startsWith( "is" ) ) {
			propertyName = getMethod.getElementName().substring( 2 );
		} else {
			return "???";
		}
		String upper = propertyName.substring( 0, 1 );
		String lower = upper.toLowerCase();
		return StringUtils.replaceOnce( propertyName, upper, lower );
	}
	
	public String getReturnType() {
		try {
			String type = Signature.toString( getMethod.getReturnType() );
			String resolvedType = null;
			if ( Character.isUpperCase( type.charAt( 0 ) ) && type.indexOf( '.' ) < 0 ) {
				StringBuffer buffer = new StringBuffer();
				String tmp = type;
				while ( tmp.indexOf( "[]" ) > 0  ) {
					tmp = StringUtils.replaceOnce( tmp, "[]", "" );
					buffer.append( "[]" );
				}
				IType itype = SourceManager.getType( getMethod.getCompilationUnit() );
				String[][] array = itype.resolveType( tmp );
				buffer.insert( 0 , array[0][0] + "." + array[0][1] );
				resolvedType = buffer.toString();
			}
			if ( resolvedType != null ) {
				return resolvedType;
			} else {
				return type;
			}
		} catch ( Exception ex ) {
			return "???";
		}
	}

	public int compareTo( Object o ) {
		CompilationUnitPropertyDescriptor other = (CompilationUnitPropertyDescriptor) o;
		return this.getPropertyName().compareTo( other.getPropertyName() );
	}
	
}
