/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:23 $
 * $Revision: 1.6 $
 * $State: Exp $
 * $RCSfile: OperatorEditor.java,v $
 * 
 */
package org.evolvica.ui.editors.operator;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.MultiEditorInput;
import org.eclipse.ui.part.MultiPageEditorPart;
import org.evolvica.ui.UILog;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.6 $ $Date: 2004/05/26 11:53:23 $
 */
public class OperatorEditor extends MultiPageEditorPart {
	
	protected JavaSourceEditor sourceEditor = null;
	
	protected DescriptorEditor descriptorEditor = null;
	
	protected void createPages() {
		FileEditorInput sourceInput = null;
		FileEditorInput descriptorInput = null;
		IEditorInput editorInput = getEditorInput();
		if ( editorInput instanceof MultiEditorInput ) {
			sourceInput = (FileEditorInput) ((MultiEditorInput) editorInput).getInput()[0];
			descriptorInput = (FileEditorInput) ((MultiEditorInput) editorInput).getInput()[1];
		} else {
			FileEditorInput input = (FileEditorInput) editorInput;
			if ( input.getFile().getName().endsWith( ".java" ) ) {
				sourceInput = input;
				descriptorInput = findAssociatedDescriptorInput( input );
			} else {
				sourceInput = findAssociatedSourceInput( input );
				descriptorInput = input;
			}
		}
		createSourcePage( sourceInput );
		createDescriptorPage( sourceInput, descriptorInput );
		setTitle( sourceInput.getName() );
	}
	
	protected void createSourcePage( FileEditorInput sourceInput ) {
		try {
			sourceEditor = new JavaSourceEditor();
			int index = addPage( sourceEditor, sourceInput );
			setPageText( index, "Source" ); 
		} catch ( PartInitException pix ) {
			UILog.error( "Error creating Java Editor for input " + sourceInput.getFile(), pix );
		}
	}
	
	protected void createDescriptorPage( FileEditorInput sourceInput, FileEditorInput descriptorInput ) {
		try {
			descriptorEditor = new DescriptorEditor( sourceInput, descriptorInput );
			int index = addPage( descriptorEditor, sourceInput );
			setPageText( index, "Descriptor" );
		} catch ( PartInitException pix ) {
			UILog.error( "Error creating Descriptor Editor for input " + sourceInput.getFile() + ", " + descriptorInput.getFile(), pix );
		}
	}
	
	public void doSave( IProgressMonitor monitor ) {
		if ( sourceEditor.isDirty() ) {
			sourceEditor.doSave( monitor );
		}
		if ( descriptorEditor.isDirty() ) {
			descriptorEditor.doSave( monitor );
		}
	}

	public void doSaveAs() {
		sourceEditor.doSaveAs();
		// descriptorEditor.doSaveAs();
	}

	public void gotoMarker( IMarker marker ) {
		
	}

	public boolean isSaveAsAllowed() {
		return false;
	}
	
	public FileEditorInput findAssociatedSourceInput( FileEditorInput descInput ) {
		IFile desc = descInput.getFile();
		IPath sourcePath = desc.getFullPath().removeFirstSegments( 1 ).removeFileExtension().addFileExtension( "java" );
		IFile source = desc.getProject().getFile( sourcePath );
		return new FileEditorInput( source );
	}
	
	public FileEditorInput findAssociatedDescriptorInput( FileEditorInput sourceInput ) {
		IFile source = sourceInput.getFile();
		IPath descriptorPath = source.getFullPath().removeFirstSegments( 1 ).removeFileExtension().addFileExtension( "descriptor" );
		IFile descriptor = source.getProject().getFile( descriptorPath );
		return new FileEditorInput( descriptor );
	}
	
	protected void pageChange( int newPageIndex ) {
		super.pageChange( newPageIndex );
		if ( newPageIndex == 1 ) {
			descriptorEditor.updatePartControl();
		}
	}

}
