/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:24 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: PropertyTableContentProvider.java,v $
 *
 */
package org.evolvica.ui.editors.operator;

import java.util.*;
import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;

import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.evolvica.engine.AttributeDescriptor;
import org.evolvica.engine.OperatorDescriptor;
import org.evolvica.ui.SourceManager;

/**
 * Documentation missing.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:53:24 $
 * @since RELEASE_NUMBER_NOT_DETERMINED
 */
public class PropertyTableContentProvider implements IStructuredContentProvider {
	
	protected ICompilationUnit source = null;
	
	protected OperatorDescriptor descriptor = null;
	
	protected TreeMap map = new TreeMap();
	
	public Object[] getElements( Object inputElement ) {
		refreshMap();
		PropertyTableContentItem[] items = new PropertyTableContentItem[map.size()];
		int index = 0;
		for ( Iterator it = map.entrySet().iterator(); it.hasNext(); ) {
			Map.Entry entry = (Map.Entry) it.next();
			items[index++] = (PropertyTableContentItem) entry.getValue();
		}
		return items;
	}
	
	public void inputChanged( Viewer viewer, Object oldInput, Object newInput ) {
		if ( newInput != null ) {
			source = (ICompilationUnit) ((Object[]) newInput)[0];
			descriptor = (OperatorDescriptor) ((Object[]) newInput)[1];
			map.clear();
			refreshMap();
		}
	}
	
	protected void refreshMap() {
		CompilationUnitPropertyDescriptor[] properties = SourceManager.getProperties( source );
		// update items/add new items
		for ( int i = 0; i < properties.length; i++ ) {
			PropertyTableContentItem item = (PropertyTableContentItem) map.get( properties[i].getPropertyName() );
			if ( item == null ) {
				item = new PropertyTableContentItem();
				map.put( properties[i].getPropertyName(), item );
				updateItem( item, properties[i] );
			}
		}
		// remove outdated items
		LinkedList outdated = new LinkedList();
		for ( Iterator it = map.entrySet().iterator(); it.hasNext(); ) {
			Map.Entry entry = (Map.Entry) it.next();
			String key = (String) entry.getKey();
			boolean isOutdated = true;
			for ( int i = 0; i < properties.length; i++ ) {
				if ( properties[i].getPropertyName().equals( key ) ) {
					isOutdated = false;
					break;
				}
			}
			if ( isOutdated == true ) {
				outdated.add( key );
			}
		}
		for ( Iterator it = outdated.iterator(); it.hasNext(); ) {
			map.remove( it.next() );
		}
	}
	
	protected void updateItem( PropertyTableContentItem contentItem, CompilationUnitPropertyDescriptor cuDesc ) {
		String propertyName = cuDesc.getPropertyName();
		// add information from compilation unit
		contentItem.setName( propertyName );
		contentItem.setDisplayName( propertyName );
		contentItem.setDescription( "no description available" );
		contentItem.setType( cuDesc.getReturnType() );
		// handle special cases
		if ( propertyName.equals( "class" ) ) {
			contentItem.setDescription( "class of the operator" );
		} else if ( propertyName.equals( "name" ) ) {
			contentItem.setDescription( "name of the operator" );
		} else if ( propertyName.equals( "id" ) ) {
			contentItem.setDescription( "unique ID of the operator" );
			contentItem.setReadonly( true );
			contentItem.setExpert( true );
		}
		if ( cuDesc.isReadonly() ) {
			contentItem.setReadonly( true );
		}
		// add information from descriptor
		AttributeDescriptor ad = descriptor.getAttribute( propertyName );
		if ( ad != null ) {
			contentItem.setDisplayName( ad.getDisplayName() );
			contentItem.setDescription( ad.getShortDescription() );
			contentItem.setExpert( ad.isExpert() );
		}
	}
	
	public void dispose() {
		// TODO Auto-generated method stub
	}
	
}
