/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:26 $
 * $Revision: 1.9 $
 * $State: Exp $
 * $RCSfile: AlgorithmEditPart.java,v $
 *
 */
package org.evolvica.ui.parts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.draw2d.ConnectionLayer;
import org.eclipse.draw2d.FreeformLayout;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.tools.MarqueeDragTracker;
import org.eclipse.swt.graphics.Color;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;
import org.evolvica.engine.Events;
import org.evolvica.engine.IAlgorithm;
import org.evolvica.engine.IElement;
import org.evolvica.ui.AlgorithmEditor;
import org.evolvica.ui.policy.OperatorLayoutEditPolicy;

/** Documentation missing.
 *
 * @author rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.9 $ $Date: 2004/05/26 11:53:26 $
 */
public class AlgorithmEditPart extends AbstractGraphicalEditPart implements PropertyChangeListener {
	
	protected DragTracker tracker = new MarqueeDragTracker();

	public void activate() {
		super.activate();
		getAlgorithm().addPropertyChangeListener( this );
	}
	
	public void deactivate() {
		getAlgorithm().removePropertyChangeListener( this );
		super.deactivate();
	}
	
	protected IFigure createFigure() {
		ConnectionLayer layer = new ConnectionLayer();
		layer.setLayoutManager( new FreeformLayout() );
		layer.setBackgroundColor( new Color( null, 242, 242, 242 ) );
		layer.setOpaque( true );
		layer.setConnectionRouter( new EVConnectionRouter() );
		return layer;
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 * You need to tell how children nodes will be layed out...
	**/
	protected void createEditPolicies() {
		installEditPolicy( EditPolicy.LAYOUT_ROLE, new OperatorLayoutEditPolicy() );
	}
	
	public IAlgorithm getAlgorithm() {
		return (IAlgorithm) getModel();
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#getModelChildren()
	 * You must implement this method if you want your root model to have children!
	 */
	protected List getModelChildren() {
		List children = new LinkedList();
		for ( Iterator it = getAlgorithm().iterateElements(); it.hasNext(); ) {
			children.add( it.next() );
		}
		return children;
	}
	
	/**
	 * This method is not mandatory to implement, but if you do not implement
	 * it, you will not have the ability to rectangle-selects several figures...
	**/
	public DragTracker getDragTracker( Request req ) {
		return tracker;
	}
	
	public void propertyChange( PropertyChangeEvent event ) {
		String property = event.getPropertyName();
		if ( property == Events.ELEMENT_ADDED ) {
			refreshChildren();
			// manually add the operator edit part to the algorithm's metadata store as a listener
			getAlgorithm().addMetadataChangeListener( findEditPart( (IElement) event.getNewValue() ) );
		} else if ( property == Events.ELEMENT_REMOVED ) {
			// manually remove the edit part from metadata store's listener set
			getAlgorithm().removeMetadataChangeListener( findEditPart( (IElement) event.getNewValue() ) );
			refreshChildren();
		}
		IEditorPart active = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		if ( active instanceof AlgorithmEditor ) {
			((AlgorithmEditor) active).setDirty( true );
		}
		/*
		if ( property == UIEvents.ALGORITHM_CHANGED ) {
			firePropertyChange( UIEvents.ALGORITHM_CHANGED, event.getOldValue(), event.getNewValue() );
		} else if ( property == UIEvents.PROPERTY_CHANGE_ERROR ) {
			firePropertyChange( UIEvents.PROPERTY_CHANGE_ERROR, event.getOldValue(), event.getNewValue() );
		} else if ( property == UIEvents.PROPERTY_CHANGE_VETO ) {
			firePropertyChange( UIEvents.PROPERTY_CHANGE_VETO, event.getOldValue(), event.getNewValue() );
		} else if ( property == UIEvents.OPERATOR_CREATE_ERROR ) {
			firePropertyChange( UIEvents.OPERATOR_CREATE_ERROR, null, null );
		}
		*/
	}
	
	private OperatorEditPart findEditPart( IElement element ) {
		List c = getChildren();
		for ( Iterator it = c.iterator(); it.hasNext(); ) {
			Object part = it.next();
			if ( part instanceof OperatorEditPart ) {
				if ( ((OperatorEditPart) part).getElement() == element ) {
					return (OperatorEditPart) part;
				}
			}
		}
		return null;
	}
	
}
