/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:26 $
 * $Revision: 1.7 $
 * $State: Exp $
 * $RCSfile: EVConnectionAnchor.java,v $
 * 
 */
package org.evolvica.ui.parts;

import org.eclipse.draw2d.AbstractConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Point;

/** Documentation missing.
 *
 * @author rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.7 $ $Date: 2004/05/26 11:53:26 $
 */
public class EVConnectionAnchor extends AbstractConnectionAnchor {
	
	public static final String SOURCE = "source";
	
	public static final String TARGET = "target";
	
	protected Point location = null; // position relative to the owner figures position
	protected String name = null;
	protected String type = null;
	protected boolean isConnected = false;
	protected int index = 0;
	protected int position = 0;
	
	public EVConnectionAnchor( IFigure owner, Point location, String type, int index ) {
		super( owner );
		this.location = location;
		this.type = type;
		this.index = index;
		this.name = type + index;
		if ( type == SOURCE ) {
			position = PositionConstants.EAST;
		} else {
			position = PositionConstants.WEST;
		}
	}
	
	/**
	 * @see org.eclipse.draw2d.ConnectionAnchor#getLocation(org.eclipse.draw2d.geometry.Point)
	 */
	public Point getLocation( Point point ) {
		Point ownerLocation = getOwner().getBounds().getLocation();
		Point loc = new Point( ownerLocation.x + location.x, ownerLocation.y + location.y );
		getOwner().translateToAbsolute( loc );
		return loc;
	}
	
	// returns the absolute location of this anchor, dependent on the owner's position
	public Point getAbsoluteLocation() {
		return getLocation( getOwner().getBounds().getLocation() );
	}
	
	// returns the location of this anchor relative to (0, 0)
	public Point getReferencePoint() {
		Point reference = null;
		if ( location != null ) {
			reference = location;
		} else {
			if ( type == TARGET ) {
				int separation = ((OperatorShape) getOwner()).inputSeparation();
//				int yposition = separation * index + OperatorShape.H_TERMINAL * index + separation;
				int yposition = separation * index + separation;
				reference = new Point( 0, yposition );
			} else {
				int separation = ((OperatorShape) getOwner()).outputSeparation();
//				int yposition = separation * index + OperatorShape.H_TERMINAL * index + separation;
				int yposition = separation * index + separation;
				reference = new Point( OperatorShape.W_FIGURE, yposition );
			}
		}
		return reference;
	}
	
	public String getName() {
		return this.name;
	}
	
	public String getType() {
		return this.type;
	}
	
	public int getIndex() {
		return index;
	}
	
	public boolean isSourceAnchor() {
		if ( type == SOURCE ) {
			return true;
		} else {
			return false;
		}
	}
	
	public void setConnected( boolean isConnected ) {
		this.isConnected = isConnected;
	}
	
	public boolean isConnected() {
		return isConnected;
	}
	
	public String toString() {
		return name + " @ " + getAbsoluteLocation();
	}
	
}
