/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:27 $
 * $Revision: 1.4 $
 * $State: Exp $
 * $RCSfile: EVConnectionRouter.java,v $
 *
 */
package org.evolvica.ui.parts;

import org.eclipse.draw2d.*;
import org.eclipse.draw2d.AbstractRouter;
import org.eclipse.draw2d.Connection;
import org.eclipse.draw2d.PositionConstants;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;

/**
 * Documentation missing.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.4 $ $Date: 2004/05/26 11:53:27 $
 * @since RELEASE_NUMBER_NOT_DETERMINED
 */
public class EVConnectionRouter extends AbstractRouter {
	
	/**
	 * @see org.eclipse.draw2d.ConnectionRouter#route(org.eclipse.draw2d.Connection)
	 */
	public void route( Connection connection ) {
		ConnectionAnchor src = connection.getSourceAnchor();
		ConnectionAnchor end = connection.getTargetAnchor();
		if ( src == null || end == null ) {
			return;
		}
		if ( ! (src instanceof EVConnectionAnchor) || ! (end instanceof EVConnectionAnchor) ) {
			PointList list = new PointList();
			list.addPoint( getStartPoint( connection ) );
			list.addPoint( getEndPoint( connection ) );
			return;
		}
		EVConnectionAnchor sourceAnchor = (EVConnectionAnchor) connection.getSourceAnchor();
		EVConnectionAnchor targetAnchor = (EVConnectionAnchor) connection.getTargetAnchor();
		//
		PointList list = new PointList();
		Point s = getStartPoint( connection );
		Point e = getEndPoint( connection );
		//
		Point s0 = null, s1 = null, e0 = null, e1 = null;
		if ( sourceAnchor.position == PositionConstants.EAST ) {
			s0 = new Point( s.x + 20, s.y );
			if ( targetAnchor.position == PositionConstants.WEST ) {
				e0 = new Point( e.x - 20, e.y );
				if ( s0.x < e0.x ) { // CASE 1
					int dx = (e0.x - s0.x) / 2;
					s1 = new Point( s0.x + dx, s0.y );
					e1 = new Point( s0.x + dx, e0.y );
				} else { // CASE 2
					int dy = (s0.y - e0.y) / 2;
					s1 = new Point( s0.x, s0.y - dy );
					e1 = new Point( e0.x, s0.y - dy );
				}
			} else {
				e0 = new Point( e.x + 20, e.y );
				if ( s0.x < e0.x ) { // CASE 3
					s1 = new Point( e0.x, s0.y );
				} else { // CASE 4
					s1 = new Point( s0.x, e0.y );
				}
			}
		} else {
			s0 = new Point( s.x - 20, s.y );
			if ( targetAnchor.position == PositionConstants.EAST ) {
				e0 = new Point( e.x + 20, e.y );
				if ( s0.x > e0.x ) { // CASE 5
					int dx = (s0.x - e0.x) / 2;
					s1 = new Point( s0.x - dx, s0.y );
					e1 = new Point( s0.x - dx, e0.y );
				} else { // CASE 6
					int dy = (s0.y - e0.y) / 2;
					s1 = new Point( s0.x, s0.y - dy );
					e1 = new Point( e0.x, s0.y - dy );
				}
			} else {
				e0 = new Point( e.x - 20, e.y );
				if ( s0.x > e0.x ) { // CASE 7
					s1 = new Point( e0.x, s0.y );
				} else { // CASE 8
					s1 = new Point( s0.x, e0.y );
				}
			}
		}
		//
		list.addPoint( s );
		list.addPoint( s0 );
		if ( s1 != null ) {
			list.addPoint( s1 );
		}
		if ( e1 != null ) {
			list.addPoint( e1 );
		}
		list.addPoint( e0 );
		list.addPoint( e );
		connection.setPoints( list );
	}

}
