/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:27 $
 * $Revision: 1.10 $
 * $State: Exp $
 * $RCSfile: OperatorEditPart.java,v $
 *
 */
package org.evolvica.ui.parts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.*;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.requests.DropRequest;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.swt.graphics.Color;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;
import org.evolvica.engine.Events;
import org.evolvica.engine.IConnection;
import org.evolvica.engine.IElement;
import org.evolvica.ui.*;
import org.evolvica.ui.AlgorithmEditor;
import org.evolvica.ui.UIEvents;
import org.evolvica.ui.UIHelper;
import org.evolvica.ui.policy.OperatorEditPolicy;
import org.evolvica.ui.policy.OperatorNodeEditPolicy;
import org.evolvica.ui.prefs.PreferenceDefaults;
import org.evolvica.util.Metadata;

/**
 * Documentation missing.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.10 $ $Date: 2004/05/26 11:53:27 $
 * @since RELEASE_NUMBER_NOT_DETERMINED
 */
public class OperatorEditPart extends AbstractGraphicalEditPart implements NodeEditPart, PropertyChangeListener {
	
	public void activate() {
		super.activate();
		getElement().addPropertyChangeListener( this );
		getElement().getAlgorithm().addMetadataChangeListener( this );
	}
	
	public void deactivate() {
		getElement().removePropertyChangeListener( this );
		// Wrong! the element does not have an associated algorithm any more
		// getElement().getAlgorithm().removeMetadataChangeListener( this );
		// this is the workaround:
		((AlgorithmEditPart) getParent()).getAlgorithm().removeMetadataChangeListener( this );
		super.deactivate();
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#createFigure()
	**/
	protected IFigure createFigure() {
		IElement model = (IElement) getModel();
		int x = 0;
		int y = 0;
		boolean mirrored = false;
		Color bgcolor = new Color( null, PreferenceConverter.getColor( AlgorithmEditorPlugin.plugin().getPreferenceStore(),PreferenceDefaults.OPERATOR_BG_COLOR ) );
		try {
			x = getElement().getMetadata( UIEvents.ELEMENT_X ).getValueAsInteger();
			y = getElement().getMetadata( UIEvents.ELEMENT_Y ).getValueAsInteger();
			mirrored = getElement().getMetadata( UIEvents.ELEMENT_MIRRORED ).getValueAsBoolean();
			bgcolor = UIHelper.decodeColor( getElement().getMetadata( UIEvents.ELEMENT_BGCOLOR ).getValue() );
		} catch ( NullPointerException npx ) {
			// quiet -> use the defaults
		}
		Point location = new Point( x, y );
		// create the operator figure
		OperatorShape figure = new OperatorShape( model.getOperator().getName(), model.incomingSize(), model.outgoingSize() );
		figure.setLocation( location );
		figure.setMirrored( mirrored );
		figure.setBackgroundColor( bgcolor );
		// return the figure
		return figure;
	}
	
	protected OperatorShape getOperatorShape() {
		return (OperatorShape) getFigure();
	}
	
	protected void createEditPolicies() {
		installEditPolicy( EditPolicy.COMPONENT_ROLE, new OperatorEditPolicy() );
		installEditPolicy( EditPolicy.GRAPHICAL_NODE_ROLE, new OperatorNodeEditPolicy() );
	}
	
	public IElement getElement() {
		return (IElement) getModel();
	}
	
	public void propertyChange( PropertyChangeEvent event ) {
		boolean setDirty = false;
		String property = event.getPropertyName();
		// check for metadata change
		if ( property == Metadata.METADATA_CHANGE ) {
			if ( event.getSource() == getModel() ) {
				Metadata metaData = (Metadata) event.getNewValue();
				String metaPath = metaData.getPath();
				if ( metaPath.equals( UIEvents.ELEMENT_BGCOLOR ) ) {
					getOperatorShape().setBackgroundColor( UIHelper.decodeColor( metaData.getValue() ) );
					setDirty = true;
				} else if ( metaPath.equals( UIEvents.ELEMENT_MIRRORED ) ) {
					getOperatorShape().setMirrored( Boolean.valueOf( event.getNewValue().toString() ).booleanValue() );
					refreshVisuals();
					setDirty = true;
				} else if ( metaPath.equals( UIEvents.ELEMENT_X ) || metaPath.equals( UIEvents.ELEMENT_Y ) ) {
					refreshVisuals();
					setDirty = true;
				}
			}
		} else if ( property == Events.OUTGOING_CONNECTED ) {
			IConnection connection = (IConnection) event.getNewValue();
			int index = connection.getSource().getOutgoingIndex( connection );
			getOperatorShape().getSourceAnchor( index ).setConnected( true );
			refreshSourceConnections();
			setDirty = true;
		} else if ( property == Events.OUTGOING_DISCONNECTED ) {
			getOperatorShape().getSourceAnchor( ((Integer) event.getOldValue()).intValue() ).setConnected( false );
			refreshSourceConnections();
			setDirty = true;
		} else if ( property == Events.INCOMING_CONNECTED ) {
			IConnection connection = (IConnection) event.getNewValue();
			int index = connection.getTarget().getIncomingIndex( connection );
			getOperatorShape().getTargetAnchor( index ).setConnected( true );
			refreshTargetConnections();
			setDirty = true;
		} else if ( property == Events.INCOMING_DISCONNECTED ) {
			getOperatorShape().getTargetAnchor( ((Integer) event.getOldValue()).intValue() ).setConnected( false );
			refreshTargetConnections();
			setDirty = true;
		} else if ( property == Events.INCOMING_SIZE_CHANGED ) {
			getOperatorShape().setInputs( ((Integer) event.getNewValue()).intValue() );
			refreshVisuals();
			setDirty = true;
		} else if ( property == Events.OUTGOING_SIZE_CHANGED ) {
			getOperatorShape().setOutputs( ((Integer) event.getNewValue()).intValue() );
			refreshVisuals();
			setDirty = true;
		} else if ( property == UIEvents.ELEMENT_RENAMED ) {
			getOperatorShape().setName( (String) event.getNewValue() );
			refreshVisuals();
			setDirty = true;
		}
		if ( setDirty == true ) {
			IEditorPart active = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
			if ( active instanceof AlgorithmEditor ) {
				((AlgorithmEditor) active).setDirty( true );
			}
		}
	}
	
	protected List getModelSourceConnections() {
		LinkedList list = new LinkedList();
		for ( int i = 0; i < getElement().outgoingSize(); i++ ) {
			IConnection connection = getElement().getOutgoing( i );
			if ( connection != null ) {
				list.add( connection );
			}
		}
		return list;
	}
	
	protected List getModelTargetConnections() {
		LinkedList list = new LinkedList();
		for ( int i = 0; i < getElement().incomingSize(); i++ ) {
			IConnection connection = getElement().getIncoming( i );
			if ( connection != null ) {
				list.add( connection );
			}
		}
		return list;
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 **/
	protected void refreshVisuals() {
		int x = 0;
		int y = 0;
		try {
			x = getElement().getMetadata( UIEvents.ELEMENT_X ).getValueAsInteger();
			y = getElement().getMetadata( UIEvents.ELEMENT_Y ).getValueAsInteger();
		} catch ( NullPointerException npx ) {
			// be quiet
		}
		Point location = new Point( x, y );
		Dimension size = getOperatorShape().getSize();
		Rectangle rect = new Rectangle( location, size );
		getOperatorShape().repaint();
		if ( sourceConnections != null ) {
			for ( Iterator it = sourceConnections.iterator(); it.hasNext(); ) {
				ConnectionEditPart part = (ConnectionEditPart) it.next();
				part.refresh();
			}
		}
		if ( targetConnections != null ) {
			for (Iterator it = targetConnections.iterator(); it.hasNext(); ) {
				ConnectionEditPart part = (ConnectionEditPart) it.next();
				part.refresh();
			}
		}
		((GraphicalEditPart) getParent()).setLayoutConstraint( this, getFigure(), rect );
	}
	
	/**
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	public ConnectionAnchor getSourceConnectionAnchor( ConnectionEditPart connection ) {
		IConnection model = (IConnection) connection.getModel();
		return getOperatorShape().getAnchor( EVConnectionAnchor.SOURCE, getElement().getOutgoingIndex( model ) );
	}

	/**
	 * @see org.eclipse.gef.NodeEditPart#getSourceConnectionAnchor(org.eclipse.gef.Request)
	 */
	public ConnectionAnchor getSourceConnectionAnchor( Request request ) {
		return getOperatorShape().getNearestFreeSourceAnchor( ((DropRequest) request).getLocation() );
	}

	/**
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.ConnectionEditPart)
	 */
	public ConnectionAnchor getTargetConnectionAnchor( ConnectionEditPart connection ) {
		IConnection model = (IConnection) connection.getModel();
		return getOperatorShape().getAnchor( EVConnectionAnchor.TARGET, getElement().getIncomingIndex( model ) );
	}

	/**
	 * @see org.eclipse.gef.NodeEditPart#getTargetConnectionAnchor(org.eclipse.gef.Request)
	 */
	public ConnectionAnchor getTargetConnectionAnchor( Request request ) {
		return getOperatorShape().getNearestFreeTargetAnchor( ((DropRequest) request).getLocation() );
	}

	public String toString() {
	  return "part " + getModel();
	}

}
