/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:27 $
 * $Revision: 1.8 $
 * $State: Exp $
 * $RCSfile: OperatorShape.java,v $
 * 
 */
package org.evolvica.ui.parts;

import java.util.HashSet;
import java.util.Iterator;

import org.eclipse.draw2d.*;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Ray;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.swt.graphics.Color;
import org.evolvica.ui.AlgorithmEditorPlugin;
import org.evolvica.ui.prefs.PreferenceDefaults;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.8 $ $Date: 2004/05/26 11:53:27 $
 */
public class OperatorShape extends Shape {
	
	public static final int W_FIGURE = 72;
	public static final int W_TERMINAL = 8;
	public static final int H_SEPARATOR = 2;
	public static final int H_LABEL = 12;
	public static final int SEPARATION = 16;
	
	private static final IPreferenceStore PREF_STORE = AlgorithmEditorPlugin.plugin().getPreferenceStore();
	
	protected String name = null;
	
	protected int inputs = 0;
	
	protected int outputs = 0;
	
	protected Color inputColor = null;
	
	protected Color outputColor = null;
	
	protected HashSet anchors = new HashSet();
	
	protected boolean isMirrored = false;
	
	public OperatorShape( String name, int inputs, int outputs ) {
		this.fgColor = new Color( null, PreferenceConverter.getColor( PREF_STORE, PreferenceDefaults.OPERATOR_FG_COLOR ) );
		this.bgColor = new Color( null, PreferenceConverter.getColor( PREF_STORE, PreferenceDefaults.OPERATOR_BG_COLOR ) );
		this.inputColor = new Color( null, PreferenceConverter.getColor( PREF_STORE, PreferenceDefaults.OPERATOR_IN_COLOR ) );
		this.outputColor = new Color( null, PreferenceConverter.getColor( PREF_STORE, PreferenceDefaults.OPERATOR_OUT_COLOR ) );
		this.name = name;
		this.inputs = inputs;
		this.outputs = outputs;
		updateShapeSize();
		updateShapeAnchors();
		/*
		for ( int i = 0; i < inputs; i++ ) {
			int y = inputSeparation() * i + inputSeparation();
			anchors.add(  new EVConnectionAnchor( this, new Point( 0, y ), EVConnectionAnchor.TARGET, i ) );
		}
		for ( int i = 0; i < outputs; i++ ) {
			int y = outputSeparation() * i + outputSeparation();
			anchors.add(  new EVConnectionAnchor( this, new Point( shapeWidth(), y ), EVConnectionAnchor.SOURCE, i ) );
		}
		*/
	}
	
	protected void updateShapeSize() {
		this.getBounds().width = shapeWidth();
		this.getBounds().height = shapeHeight();
	}
	
	protected void updateShapeAnchors() {
		int currentSize = getTargetAnchorSize();
		if ( inputs < currentSize ) {
			for ( int i = inputs; i < currentSize; i++ ) {
				anchors.remove( getAnchor( EVConnectionAnchor.TARGET, i ) );
			}
		} else if ( inputs > currentSize ) {
			for ( int i = currentSize; i < inputs; i++ ) {
				int y = inputSeparation() * i + inputSeparation();
				anchors.add(  new EVConnectionAnchor( this, new Point( 0, y ), EVConnectionAnchor.TARGET, i ) );
			}
		}
		currentSize = getSourceAnchorSize();
		if ( outputs < currentSize ) {
			for ( int i = outputs; i < currentSize; i++ ) {
				anchors.remove( getAnchor( EVConnectionAnchor.SOURCE, i ) );
			}
		} else if ( outputs > currentSize ) {
			for ( int i = currentSize; i < outputs; i++ ) {
				int y = outputSeparation() * i + outputSeparation();
				anchors.add(  new EVConnectionAnchor( this, new Point( shapeWidth(), y ), EVConnectionAnchor.SOURCE, i ) );
			}
		}
	}
	
	public void setMirrored( boolean mirrored ) {
		if ( mirrored == true ) {
			for ( Iterator it = anchors.iterator(); it.hasNext(); ) {
				EVConnectionAnchor anchor = (EVConnectionAnchor) it.next();
				if ( anchor.type == EVConnectionAnchor.SOURCE ) { // source anchors must be moved right -> left
					int y = outputSeparation() * anchor.index + outputSeparation();
					anchor.location = new Point( 0, y );
					anchor.position = PositionConstants.WEST;
				} else { // target anchors must be moved left -> right
					int y = inputSeparation() * anchor.index + inputSeparation();
					anchor.location = new Point( shapeWidth(), y );
					anchor.position = PositionConstants.EAST;
				}
			}
			isMirrored = true;
		} else {
			for ( Iterator it = anchors.iterator(); it.hasNext(); ) {
				EVConnectionAnchor anchor = (EVConnectionAnchor) it.next();
				if ( anchor.type == EVConnectionAnchor.SOURCE ) { // source anchors must be moved left -> right
					int y = outputSeparation() * anchor.index + outputSeparation();
					anchor.location = new Point( shapeWidth(), y );
					anchor.position = PositionConstants.EAST;
				} else { // target anchors must be moved right -> left
					int y = inputSeparation() * anchor.index + inputSeparation();
					anchor.location = new Point( 0, y );
					anchor.position = PositionConstants.WEST;
				}
			}
			isMirrored = false;
		}
	}
	
	public boolean isMirrored() {
		return isMirrored;
	}
	
	public void setName( String string ) {
		name = string;
	}

	public void setInputs( int i ) {
		inputs = i;
		updateShapeSize();
		updateShapeAnchors();
	}

	public void setOutputs(int i) {
		outputs = i;
		updateShapeSize();
		updateShapeAnchors();
	}

	/**
	 * @see org.eclipse.draw2d.Shape#fillShape(org.eclipse.draw2d.Graphics)
	 */
	protected void fillShape( Graphics graphics ) {
		graphics.setBackgroundColor( bgColor );
		Point l = getLocation();
		graphics.fillRectangle( l.x, l.y, shapeWidth(), shapeHeight() );
	}

	/**
	 * @see org.eclipse.draw2d.Shape#outlineShape(org.eclipse.draw2d.Graphics)
	 */
	protected void outlineShape( Graphics graphics ) {
		Point l = getLocation();
		int lw = lineWidth / 2;
		// draw outline
		graphics.drawRectangle( l.x + lw, l.y + lw, shapeWidth() - lw - 1, shapeHeight() - lw - 1 ); // FIX: rectangle does not have the right size
		// draw separator
		graphics.drawLine( l.x, l.y + separationLocation(), l.x + shapeWidth(), l.y + separationLocation() );
		// draw name
		if ( name.length() < 10 ) {
			graphics.drawText( name, l.x + 4, l.y + separationLocation() + 1 );
		} else {
			graphics.drawText( name.substring( 0, 8 ) + "...", l.x + 4, l.y + separationLocation() + 1 );
		}
		// draw terminals + add anchors
		if ( isMirrored == true ) {
			outlineMirroredTerminals( graphics );
		} else {
			outlineTerminals( graphics );
		}
	}
	
	protected void outlineTerminals( Graphics graphics ) {
		Point l = getLocation();
		graphics.setForegroundColor( inputColor );
		for ( int i = 0; i < inputs; i++ ) {
			int y = inputSeparation() * i + inputSeparation();
			graphics.drawLine( l.x, l.y + y - 1, l.x + W_TERMINAL, l.y + y - 1 );
			graphics.drawLine( l.x, l.y + y, l.x + W_TERMINAL, l.y + y );
			graphics.drawLine( l.x, l.y + y + 1, l.x + W_TERMINAL, l.y + y + 1 );
			graphics.drawText( String.valueOf( i ), l.x + W_TERMINAL + 4, l.y + y - H_LABEL / 2 );
		}
		graphics.setForegroundColor( outputColor );
		for ( int i = 0; i < outputs; i++ ) {
			int y = outputSeparation() * i + outputSeparation();
			graphics.drawLine( l.x + shapeWidth(), l.y + y - 1, l.x + shapeWidth() - W_TERMINAL, l.y + y - 1 );
			graphics.drawLine( l.x + shapeWidth(), l.y + y, l.x + shapeWidth() - W_TERMINAL, l.y + y );
			graphics.drawLine( l.x + shapeWidth(), l.y + y + 1, l.x + shapeWidth() - W_TERMINAL, l.y + y + 1 );
			graphics.drawText( String.valueOf( i ), l.x  + shapeWidth() - W_TERMINAL - 8, l.y + y - H_LABEL / 2 );
		}
	}
	
	protected void outlineMirroredTerminals( Graphics graphics ) {
		Point l = getLocation();
		graphics.setForegroundColor( inputColor );
		for ( int i = 0; i < inputs; i++ ) {
			int y = inputSeparation() * i + inputSeparation();
			graphics.drawLine( l.x + shapeWidth(), l.y + y - 1, l.x + shapeWidth() - W_TERMINAL, l.y + y - 1 );
			graphics.drawLine( l.x + shapeWidth(), l.y + y, l.x + shapeWidth() - W_TERMINAL, l.y + y );
			graphics.drawLine( l.x + shapeWidth(), l.y + y + 1, l.x + shapeWidth() - W_TERMINAL, l.y + y + 1);
			graphics.drawText( String.valueOf( i ), l.x  + shapeWidth() - W_TERMINAL - 8, l.y + y - H_LABEL / 2 );
		}
		graphics.setForegroundColor( outputColor );
		for ( int i = 0; i < outputs; i++ ) {
			int y = outputSeparation() * i + outputSeparation();
			graphics.drawLine( l.x, l.y + y - 1, l.x + W_TERMINAL, l.y + y - 1);
			graphics.drawLine( l.x, l.y + y, l.x + W_TERMINAL, l.y + y );
			graphics.drawLine( l.x, l.y + y + 1, l.x + W_TERMINAL, l.y + y + 1);
			graphics.drawText( String.valueOf( i ), l.x + W_TERMINAL + 4, l.y + y - H_LABEL / 2 );
		}
	}
	
	protected int inputSeparation() {
		if ( inputs >= outputs ) {
			return SEPARATION;
		} else {
			return SEPARATION * (outputs + 1) / (inputs + 1);
		}
	}
	
	protected int outputSeparation() {
		if ( outputs >= inputs ) {
			return SEPARATION;
		} else {
			return SEPARATION * (inputs + 1) / (outputs + 1);
		}
	}
	
	protected int separationLocation() {
		if ( inputs > outputs ) {
			return inputSeparation() * (inputs + 1);
		} else {
			return outputSeparation() * (outputs + 1);
		}
	}
	
	protected int shapeHeight() {
		if ( inputs > outputs ) {
			return inputSeparation() * (inputs + 1) + H_SEPARATOR + H_LABEL;
		} else {
			return outputs + outputSeparation() * (outputs + 1) + H_SEPARATOR + H_LABEL;
		}
	}
	
	protected int shapeWidth() {
		return W_FIGURE;
	}
	
	public EVConnectionAnchor getAnchor( String type, int index ) {
		for (Iterator it = anchors.iterator(); it.hasNext(); ) {
			EVConnectionAnchor anchor = (EVConnectionAnchor) it.next();
			if ( anchor.getType() == type && anchor.getIndex() == index ) {
				return anchor;
			}
		}
		return null;
	}
	
	protected int getSourceAnchorSize() {
		int size = 0;
		for ( Iterator it = anchors.iterator(); it.hasNext(); ) {
			EVConnectionAnchor anchor = (EVConnectionAnchor) it.next();
			if ( anchor.type == EVConnectionAnchor.SOURCE ) {
				++size;
			}
		}
		return size;
	}
	
	protected int getTargetAnchorSize() {
		int size = 0;
		for ( Iterator it = anchors.iterator(); it.hasNext(); ) {
			EVConnectionAnchor anchor = (EVConnectionAnchor) it.next();
			if ( anchor.type == EVConnectionAnchor.TARGET ) {
				++size;
			}
		}
		return size;
	}
	
	public EVConnectionAnchor getSourceAnchor( int index ) {
		return getAnchor( EVConnectionAnchor.SOURCE, index );
	}
	
	public EVConnectionAnchor getTargetAnchor( int index ) {
		return getAnchor( EVConnectionAnchor.TARGET, index );
	}
	
	public EVConnectionAnchor getNearestSourceAnchor( Point location ) {
		double smallestDistance = Double.MAX_VALUE;
		EVConnectionAnchor nearestAnchor = null;
		for ( Iterator it = anchors.iterator(); it.hasNext(); ) {
			EVConnectionAnchor anchor = (EVConnectionAnchor) it.next();
			if ( anchor.getType() == EVConnectionAnchor.SOURCE ) {
//				System.out.println( "mouse location: " + location );
//				System.out.println( "anchor " + anchor + " location: " + anchor.getAbsoluteLocation() );
				double distance = new Ray( location, anchor.getAbsoluteLocation() ).length();
				if ( distance < smallestDistance ) {
					smallestDistance = distance;
					nearestAnchor = anchor;
				}
			}
		}
//		System.out.println( "nearest source anchor: " + nearestAnchor );
		return nearestAnchor;
	}
	
	public EVConnectionAnchor getNearestFreeSourceAnchor( Point location ) {
		double smallestDistance = Double.MAX_VALUE;
		EVConnectionAnchor nearestAnchor = null;
		for ( Iterator it = anchors.iterator(); it.hasNext(); ) {
			EVConnectionAnchor anchor = (EVConnectionAnchor) it.next();
			if ( anchor.isSourceAnchor() && ! anchor.isConnected() ) {
				double distance = new Ray( location, anchor.getAbsoluteLocation() ).length();
				if ( distance < smallestDistance ) {
					smallestDistance = distance;
					nearestAnchor = anchor;
				}
			}
		}
		return nearestAnchor;
	}
	
	public EVConnectionAnchor getNearestTargetAnchor( Point location ) {
		double smallestDistance = Double.MAX_VALUE;
		EVConnectionAnchor nearestAnchor = null;
		for ( Iterator it = anchors.iterator(); it.hasNext(); ) {
			EVConnectionAnchor anchor = (EVConnectionAnchor) it.next();
			if ( anchor.getType() == EVConnectionAnchor.TARGET ) {
//				System.out.println( "mouse location: " + location );
//				System.out.println( "anchor " + anchor + " location: " + anchor.getAbsoluteLocation() );
				double distance = new Ray( location, anchor.getAbsoluteLocation() ).length();
				if ( distance < smallestDistance ) {
					smallestDistance = distance;
					nearestAnchor = anchor;
				}
			}
		}
//		System.out.println( "nearest target anchor: " + nearestAnchor );
		return nearestAnchor;
	}
	
	public EVConnectionAnchor getNearestFreeTargetAnchor( Point location ) {
		double smallestDistance = Double.MAX_VALUE;
		EVConnectionAnchor nearestAnchor = null;
		for ( Iterator it = anchors.iterator(); it.hasNext(); ) {
			EVConnectionAnchor anchor = (EVConnectionAnchor) it.next();
			if ( ! anchor.isSourceAnchor() && ! anchor.isConnected() ) {
				double distance = new Ray( location, anchor.getAbsoluteLocation() ).length();
				if ( distance < smallestDistance ) {
					smallestDistance = distance;
					nearestAnchor = anchor;
				}
			}
		}
		return nearestAnchor;
	}

}
