/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:30 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: ResourceChangeHandler.java,v $
 * 
 */
package org.evolvica.ui.views.navigator;

import java.net.URI;
import java.util.LinkedList;

import org.eclipse.core.resources.*;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.*;
import org.evolvica.ui.DiskCache;
import org.evolvica.ui.PathManager;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:53:30 $
 */
public class ResourceChangeHandler implements IResourceChangeListener {
	
	private static final int IRRELEVANT = 0;
	private static final int PROJECT_OPENED = 1;
	private static final int PROJECT_CLOSED = 2;
	private static final int PROJECT_ADDED = 3;
	private static final int PROJECT_REMOVED = 4;
	private static final int ALGORITHM_ADDED = 11;
	private static final int ALGORITHM_CHANGED = 12;
	private static final int ALGORITHM_REMOVED = 13;
	private static final int OPERATOR_ADDED = 21;
	private static final int OPERATOR_CHANGED = 22;
	private static final int OPERATOR_REMOVED = 23;
	
	private ProjectNavigator navigator = null;
	
	public ResourceChangeHandler( ProjectNavigator navigator ) {
		this.navigator = navigator;
	}
	
	public void resourceChanged( IResourceChangeEvent event ) {
		switch ( event.getType() ) {
			case IResourceChangeEvent.POST_AUTO_BUILD:
				System.out.println( "POST_AUTO_BUILD event occured" );
				break;
			case IResourceChangeEvent.POST_CHANGE:
				IResourceDelta[] deltas = findAllDeltas( event );
				for ( int i = 0; i < deltas.length; i++ ) {
					switch ( analyzeDelta( deltas[i] ) ) {
						case PROJECT_ADDED:
							Display.getDefault().syncExec( new RefreshProjectsHandler() );
							break;
						case PROJECT_REMOVED:
							Display.getDefault().syncExec( new RefreshProjectsHandler() );
							break;
						case PROJECT_OPENED:
							Display.getDefault().syncExec( new OpenProjectHandler( (IProject) deltas[i].getResource() ) );
							break;
						case PROJECT_CLOSED:
							Display.getDefault().syncExec( new CloseProjectHandler( (IProject) deltas[i].getResource() ) );
							break;
						case ALGORITHM_ADDED:
							Display.getDefault().syncExec( new RefreshAlgorithmsHandler( ((IFile) deltas[i].getResource()).getProject() ) );
							break;
						case ALGORITHM_CHANGED:
							Display.getDefault().syncExec( new RefreshAlgorithmsHandler( ((IFile) deltas[i].getResource()).getProject() ) );
							break;
						case ALGORITHM_REMOVED:
							Display.getDefault().syncExec( new RefreshAlgorithmsHandler( ((IFile) deltas[i].getResource()).getProject() ) );
							break;
						case OPERATOR_ADDED:
							Display.getDefault().syncExec( new RefreshOperatorsHandler( ((IFile) deltas[i].getResource()).getProject() ) );
							break;
						case OPERATOR_CHANGED:
							Display.getDefault().syncExec( new RefreshOperatorsHandler( ((IFile) deltas[i].getResource()).getProject() ) );
							break;
						case OPERATOR_REMOVED:
							Display.getDefault().syncExec( new RefreshOperatorsHandler( ((IFile) deltas[i].getResource()).getProject() ) );
							break;
						case IRRELEVANT:
							break;
						default:
							break;
					}
				}
				break;
			case IResourceChangeEvent.PRE_AUTO_BUILD:
				System.out.println( "PRE_AUTO_BUILD event occured" );
				break;
			case IResourceChangeEvent.PRE_CLOSE:
				Display.getDefault().syncExec( new PreCloseProjectHandler( (IProject) event.getResource() ) );
				break;
			case IResourceChangeEvent.PRE_DELETE:
				Display.getDefault().syncExec( new PreCloseProjectHandler( (IProject) event.getResource() ) );
				break;
		}
	}
	
	private IResourceDelta[] findAllDeltas( IResourceChangeEvent event ) {
		LinkedList list = new LinkedList();
		LinkedList queue = new LinkedList();
		queue.add( event.getDelta() );
		while ( ! queue.isEmpty() ) {
			IResourceDelta delta = (IResourceDelta) queue.getFirst();
			IResourceDelta[] children = delta.getAffectedChildren();
			for ( int i = 0; i < children.length; i++ ) {
				queue.add( children[i] );
			}
			list.add( delta );
			queue.removeFirst();
		}
		return (IResourceDelta[]) list.toArray( new IResourceDelta[list.size()] );
	}
	
	private int analyzeDelta( IResourceDelta delta ) {
		IResource resource = delta.getResource();
		if ( resource instanceof IWorkspaceRoot ) {
			
		} else if ( resource instanceof IProject ) {
			if ( (delta.getKind() & IResourceDelta.ADDED) != 0 ) {
				return PROJECT_ADDED;
			} else if ( (delta.getKind() & IResourceDelta.CHANGED) != 0 ) {
				if ( ( delta.getFlags() & IResourceDelta.OPEN) != 0 ) {
					if ( ((IProject) resource).isOpen() ) {
						return PROJECT_OPENED;
					} else {
						return PROJECT_CLOSED;
					}
				}
			} else if ( (delta.getKind() & IResourceDelta.REMOVED) != 0 ) {
				return PROJECT_REMOVED;
			}
		} else if ( resource instanceof IFile ) {
			IFile file = (IFile) resource;
			if ( file.getFileExtension().equals( "eva" ) ) {
				if ( (delta.getKind() & IResourceDelta.ADDED) != 0 ) {
					return ALGORITHM_ADDED;
				} else if ( (delta.getKind() & IResourceDelta.CHANGED) != 0 ) {
					return ALGORITHM_CHANGED;
				} else if ( (delta.getKind() & IResourceDelta.REMOVED) != 0 ) {
					return ALGORITHM_REMOVED;
				}
			} else if ( file.getFileExtension().equals( "descriptor" ) ) {
				if ( (delta.getKind() & IResourceDelta.ADDED) != 0 ) {
					return OPERATOR_ADDED;
				} else if ( (delta.getKind() & IResourceDelta.CHANGED) != 0 ) {
					return OPERATOR_CHANGED;
				} else if ( (delta.getKind() & IResourceDelta.REMOVED) != 0 ) {
					return OPERATOR_REMOVED;
				}
			}
		}
		return IRRELEVANT;
	}
	
	private IResourceItem findAssociatedItem( Object object ) {
		LinkedList queue = new LinkedList();
		ResourceTreeContentProvider provider = (ResourceTreeContentProvider) navigator.viewer.getContentProvider();
		Object[] obj = provider.getElements( navigator.viewer.getInput() );
		for ( int i = 0; i < obj.length; i++ ) {
			queue.add( obj[i] );
		}
		while( ! queue.isEmpty() ) {
			System.out.println( queue.getFirst().getClass() );
			IResourceItem item = (IResourceItem) queue.getFirst();
			if ( item.getItem().equals( object ) ) {
				return item;
			}
			Object[] children = item.getChildren();
			for ( int i = 0; i < children.length; i++ ) {
				queue.add( children[i] );
			}
			queue.removeFirst();
		}
		return null;
	}
	
	private AlgorithmFolder findAlgorithmFolder( IProject project ) {
		ResourceTreeContentProvider provider = (ResourceTreeContentProvider) navigator.viewer.getContentProvider();
		ProjectFolder projectFolder = findProjectFolder( project );
		if ( projectFolder != null ) {
			Object[] children = provider.getChildren( projectFolder );
			for ( int i = 0; i < children.length; i++ ) {
				if ( children[i] instanceof AlgorithmFolder ) {
					return (AlgorithmFolder) children[i];
				}
			}
		}
		return null;
	}
	
	private ProjectCategoryFolder findOperatorFolder( IProject project ) {
		ResourceTreeContentProvider provider = (ResourceTreeContentProvider) navigator.viewer.getContentProvider();
		ProjectFolder projectFolder = findProjectFolder( project );
		if ( projectFolder != null ) {
			Object[] children = provider.getChildren( projectFolder );
			for ( int i = 0; i < children.length; i++ ) {
				if ( children[i] instanceof OperatorFolder ) {
					OperatorFolder operatorFolder = (OperatorFolder) children[i];
					Object[] folders = provider.getChildren( operatorFolder );
					for ( int j = 0; j < folders.length; j++ ) {
						if ( folders[j] instanceof ProjectCategoryFolder ) {
							return (ProjectCategoryFolder) folders[j];
						}
					}
				}
			}
		}
		return null;
	}
	
	private ProjectFolder findProjectFolder( IProject project ) {
		ResourceTreeContentProvider provider = (ResourceTreeContentProvider) navigator.viewer.getContentProvider();
		Object[] obj = provider.getElements( navigator.viewer.getInput() );
		for ( int i = 0; i < obj.length; i++ ) {
			if ( obj[i] instanceof ProjectFolder ) {
				if ( ((ProjectFolder) obj[i]).project.equals( project ) ) {
					return (ProjectFolder) obj[i];
				}
			}
		}
		return null;
	}
	
	private class CloseProjectHandler implements Runnable {
		
		private IProject project = null;
		
		private CloseProjectHandler( IProject project ) {
			this.project = project;
		}
		
		public void run() {
			ProjectFolder folder = ResourceChangeHandler.this.findProjectFolder( project );
			ResourceChangeHandler.this.navigator.viewer.refresh( folder, true );
		}
		
	}
	
	private class RefreshAlgorithmsHandler implements Runnable {
		
		private IProject project = null;
		
		private RefreshAlgorithmsHandler( IProject project ) {
			this.project = project;
		}
		
		public void run() {
			AlgorithmFolder folder = ResourceChangeHandler.this.findAlgorithmFolder( project );
			ResourceChangeHandler.this.navigator.viewer.refresh( folder, true );
		}
		
	}
	
	private class RefreshOperatorsHandler implements Runnable {
		
		private IProject project = null;
		
		private RefreshOperatorsHandler( IProject project ) {
			this.project = project;
		}
		
		public void run() {
			ProjectCategoryFolder folder = ResourceChangeHandler.this.findOperatorFolder( project );
			URI classLocation = PathManager.getOutputLocation( folder.getJavaProject() );
			URI[] srcLocation = PathManager.getSourcePath( folder.getJavaProject() );
			DiskCache.refresh( classLocation, srcLocation );
			ResourceChangeHandler.this.navigator.viewer.refresh( folder, true );
		}
		
	}
	
	private class RefreshProjectsHandler implements Runnable {
		
		public void run() {
			ResourceChangeHandler.this.navigator.viewer.refresh();
		}
		
	}
	
	private class OpenProjectHandler implements Runnable {

		private IProject project = null;
		
		private OpenProjectHandler( IProject project ) {
			this.project = project;
		}
		
		public void run() {
			ProjectFolder folder = ResourceChangeHandler.this.findProjectFolder( project );
			ResourceChangeHandler.this.navigator.viewer.refresh( folder, true );
		}
		
	}
	
	private class PreCloseProjectHandler implements Runnable {

		private IProject project = null;
		
		private PreCloseProjectHandler( IProject project ) {
			this.project = project;
		}
		
		public void run() {
			IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
			IEditorReference[] refs = page.getEditorReferences();
			for ( int i = 0; i < refs.length; i++ ) {
				IEditorPart editor = refs[i].getEditor( false );
				if ( editor != null ) {
					IFile file = (IFile) editor.getEditorInput().getAdapter( IFile.class );
					if ( file != null && file.getProject().equals( project ) ) {
						page.closeEditor( editor, true );
					}
				}
			}
		}
		
	}
	
}
