/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:31 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: ResourceLabelProvider.java,v $
 * 
 */
package org.evolvica.ui.views.navigator;

import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;
import org.evolvica.ui.Images;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:53:31 $
 */
public class ResourceLabelProvider extends LabelProvider {
	
	public static final ResourceLabelProvider instance = new ResourceLabelProvider();
	
	public String getText( Object element ) {
		try {
			if ( element instanceof IResourceItem ) {
				IResourceItem resourceItem = (IResourceItem) element;
				if ( resourceItem instanceof ProjectFolder ) {
					return ((ProjectFolder) resourceItem).getProject().getName();
				} else if ( resourceItem instanceof AlgorithmFolder ) {
					return "Algorithms";
				} else if ( resourceItem instanceof AlgorithmItem ) {
					return ((AlgorithmItem) resourceItem).getFile().getName();
				} else if ( resourceItem instanceof CategoryFolder ) {
					return ((CategoryFolder) resourceItem).getCategory();
				} else if ( resourceItem instanceof OperatorFolder ) {
					return "Operators"; 
				} else if ( resourceItem instanceof OperatorItem ) {
					return ((OperatorItem) resourceItem).getDescriptor().getDisplayName();
				}
				return element.toString();
			} else {
				return element.toString();
			}
		} catch ( Exception ex ) {
			return "?UNKNOWN?";
		}
	}

	public Image getImage( Object element ) {
		if ( element instanceof IResourceItem ) {
			IResourceItem resourceItem = (IResourceItem) element;
			if ( resourceItem instanceof ProjectFolder ) {
				if ( ((ProjectFolder) resourceItem ).getProject().isOpen() ) {
					return Images.getImage( "icons/obj16/prj_obj.gif" );
				} else {
					return Images.getImage( "icons/obj16/cprj_obj.gif" );
				}
			} else if ( resourceItem instanceof AlgorithmFolder ) {
				return Images.getImage( "icons/alfolder16.png" );
			} else if ( resourceItem instanceof AlgorithmItem ) {
				return Images.getImage( "icons/algorithm16.png" );
			} else if ( resourceItem instanceof CategoryFolder ) {
				if ( resourceItem instanceof ProjectCategoryFolder ) {
					return Images.getImage( "icons/prj_category16.png" );
				} else {
					return Images.getImage( "icons/category16.png" );
				}
			} else if ( resourceItem instanceof OperatorFolder ) {
				return Images.getImage( "icons/opfolder16.png" );
			} else if ( resourceItem instanceof OperatorItem ) {
				int inputs = ((OperatorItem) resourceItem).getDescriptor().getInputSize();
				int outputs = ((OperatorItem) resourceItem).getDescriptor().getOutputSize();
				boolean isLocal = ((OperatorItem) resourceItem).isLocal();
				if ( isLocal == true ) {
					if ( inputs == 0 && outputs == 1 ) {
						return Images.getImage( "icons/prj_source16.png" );
					} else if ( inputs == 1 && outputs == 0 ) {
						return Images.getImage( "icons/prj_sink16.png" );
					} else if ( inputs == 1 && outputs == 1 ) {
						return Images.getImage( "icons/prj_processor16.png" );
					} else if ( inputs == 1 && outputs > 1 ) {
						return Images.getImage( "icons/prj_fork16.png" );
					} else if ( inputs > 1 && outputs == 1 ) {
						return Images.getImage( "icons/prj_join16.png" );
					} else if ( inputs > 1 && outputs > 1 ) {
						return Images.getImage( "icons/prj_conduit16.png" );
					} else {
						return Images.getImage( "icons/prj_unknown16.png" );
					}
				} else {
					if ( inputs == 0 && outputs == 1 ) {
						return Images.getImage( "icons/source16.png" );
					} else if ( inputs == 1 && outputs == 0 ) {
						return Images.getImage( "icons/sink16.png" );
					} else if ( inputs == 1 && outputs == 1 ) {
						return Images.getImage( "icons/processor16.png" );
					} else if ( inputs == 1 && outputs > 1 ) {
						return Images.getImage( "icons/fork16.png" );
					} else if ( inputs > 1 && outputs == 1 ) {
						return Images.getImage( "icons/join16.png" );
					} else if ( inputs > 1 && outputs > 1 ) {
						return Images.getImage( "icons/conduit16.png" );
					} else {
						return Images.getImage( "icons/unknown16.png" );
					}
				}
			}
		}
		return null;
	}
	
}
