/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:31 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: DeleteAction.java,v $
 * 
 */
package org.evolvica.ui.views.navigator.actions;

import org.apache.commons.lang.StringUtils;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.runtime.Path;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.internal.dialogs.MessageDialogWithToggle;
import org.evolvica.engine.OperatorDescriptor;
import org.evolvica.ui.Images;
import org.evolvica.ui.views.navigator.*;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:53:31 $
 */
public class DeleteAction extends NavigatorAction {
	
	public static final String ID = "org.evolvica.ui.navigator.actions.DeleteAction";
	
	public DeleteAction( ProjectNavigator navigator ) {
		super( navigator, "Delete" );
		setImageDescriptor( Images.getImageDescriptor( "icons/delete16.png" ) );
		setId( ID );
	}
	
	public void runOnSelection( Object selectedItem ) {
		if ( selectedItem != null ) {
			try  {
				if ( selectedItem instanceof ProjectFolder ) {
					MessageDialogWithToggle dialog = MessageDialogWithToggle.openQuestion( navigator.getSite().getShell(), "Confirm", "Really delete project?", "also delete contents of project folder", false );
					if ( dialog.getReturnCode() == MessageDialogWithToggle.OK ) {
						((ProjectFolder) selectedItem).getProject().delete( dialog.getToggleState(), false, null );
					}
				} else if ( selectedItem instanceof AlgorithmItem ) {
					boolean delete = MessageDialog.openQuestion( navigator.getSite().getShell(), "Confirm", "Really delete algorithm?" );
					if ( delete == true ) {
						((AlgorithmItem) selectedItem).getFile().delete( false, true, null );
					}
				} else if ( selectedItem instanceof OperatorItem ) {
					boolean delete = MessageDialog.openQuestion( navigator.getSite().getShell(), "Confirm", "Really delete operator?" );
					if ( delete == true ) {
						OperatorDescriptor desc = ((OperatorItem) selectedItem).getDescriptor();
						String name = desc.getBeanClassName();
						IJavaProject project = ((OperatorItem) selectedItem).getJavaProject();
						IPackageFragmentRoot[] roots = project.getPackageFragmentRoots();
						for ( int i = 0; i < roots.length; i++ ) {
							if ( roots[i].getKind() == IPackageFragmentRoot.K_SOURCE ) {
								IFolder folder = (IFolder) roots[i].getCorrespondingResource();
								IResource resource = folder.findMember( new Path( StringUtils.replace( name, ".", "/" ) + ".java" ) );
								if ( resource != null ) {
									resource.delete( false, null );
								}
								resource = folder.findMember( new Path( StringUtils.replace( name, ".", "/" ) + ".descriptor" ) );
								if ( resource != null ) {
									resource.delete( false, null );
								}
							}
						}
						project.getProject().refreshLocal( IResource.DEPTH_INFINITE, null );
						project.getProject().build( IncrementalProjectBuilder.INCREMENTAL_BUILD, null );
					}
				}
			} catch ( Exception cx ) {
				MessageDialog.openError( navigator.getSite().getShell(), "Error", "Error deleting resource: " + cx.getMessage() );
			}
		}
	}
	
	public boolean isEnabled() {
		IResourceItem selectedItem = navigator.getSelectedItem();
		if ( selectedItem != null ) {
			if ( selectedItem instanceof ProjectFolder ) {
				return true;
			} else if ( selectedItem instanceof AlgorithmItem ) {
				return true;
			} else if ( selectedItem instanceof OperatorItem && ((OperatorItem) selectedItem).isLocal() ) {
				return true;
			}
		}
		return false;
	}

}
