/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:32 $
 * $Revision: 1.3 $
 * $State: Exp $
 * $RCSfile: OpenAction.java,v $
 * 
 */
package org.evolvica.ui.views.navigator.actions;

import org.apache.commons.lang.StringUtils;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jdt.core.IPackageFragmentRoot;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.MultiEditorInput;
import org.evolvica.engine.OperatorDescriptor;
import org.evolvica.ui.views.navigator.*;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.3 $ $Date: 2004/05/26 11:53:32 $
 */
public class OpenAction extends NavigatorAction {
	
	public static final String ID = "org.evolvica.ui.navigator.actions.OpenAction";
	
	public OpenAction( ProjectNavigator navigator ) {
		super( navigator, "Open" );
		setId( ID );
	}
	
	public void runOnSelection( Object item ) {
		try {
			IResourceItem selected = navigator.getSelectedItem();
			if ( selected instanceof ProjectFolder ) {
				selected.getProject().open( null );
			} else if ( selected instanceof AlgorithmItem ) {
				navigator.getSite().getPage().openEditor( ((AlgorithmItem) selected).getFile(), "org.evolvica.ui.AlgorithmEditor", true );
			} else if ( selected instanceof OperatorItem ) {
				navigator.getSite().getPage().openEditor( createMultiEditorInput( (OperatorItem) selected ), "org.evolvica.ui.editors.operator.OperatorEditor", true );
			}
		} catch ( Exception ex ) {
			if ( ex.getMessage() != null ) {
				MessageDialog.openError( navigator.getSite().getShell(), "Error", ex.getMessage() );
			} else {
				MessageDialog.openError( navigator.getSite().getShell(), "Error", "Error opening editor" );
			}
		}
	}
	
	public boolean isEnabled() {
		IResourceItem selected = navigator.getSelectedItem();
		if ( selected != null ) {
			if ( selected instanceof ProjectFolder ) {
				if ( ! ((ProjectFolder) selected).getProject().isOpen() ) {
					return true;
				}
			} else if ( selected instanceof AlgorithmItem ) {
				return true;
			} else if ( selected instanceof OperatorItem && ((OperatorItem) selected).isLocal() ) {
				return true;
			}
		}
		return false;
	}

	private MultiEditorInput createMultiEditorInput( OperatorItem item ) throws PartInitException {
		OperatorDescriptor descriptor = (OperatorDescriptor) item.getItem();
		String[] editorIDs = new String[] { "source editor", "descriptor editor" };
		IEditorInput[] innerEditors = new IEditorInput[2];
		IFile srcFile = null;
		IFile descriptorFile = null;
		try {
			IPackageFragmentRoot[] fragmentRoots = item.getJavaProject().getAllPackageFragmentRoots();
			for ( int i = 0; i < fragmentRoots.length; i++ ) {
				if ( fragmentRoots[i].getKind() == IPackageFragmentRoot.K_SOURCE ) {
					IPath path = fragmentRoots[i].getPath();
					path = path.addTrailingSeparator();
					path = path.append( StringUtils.replace( descriptor.getBeanClassName(), ".", "/" ) );
					path = path.removeFirstSegments( 1 );
					IPath srcPath = path.addFileExtension( "java" );
					IPath descriptorPath = path.addFileExtension( "descriptor" );
					srcFile = item.getProject().getFile( srcPath );
					descriptorFile = item.getProject().getFile( descriptorPath );
					if ( srcFile.exists() && descriptorFile.exists() ) {
						break;
					} else {
						srcFile = null;
						descriptorFile = null;
					}
				}
			}
		} catch ( JavaModelException jmx ) {
			throw new PartInitException( "unable to find Java source or descriptor of operator" );
		}
		if ( srcFile == null || descriptorFile == null ) {
			throw new PartInitException( "unable to find Java source or descriptor of operator" );
		}
		innerEditors[0] = new FileEditorInput( srcFile );
		innerEditors[1] = new FileEditorInput( descriptorFile );
		return new MultiEditorInput( editorIDs, innerEditors );
	}
	
}
