/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:32 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: RefreshAction.java,v $
 * 
 */
package org.evolvica.ui.views.navigator.actions;

import java.lang.reflect.InvocationTargetException;
import java.net.URI;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.evolvica.ui.CacheManager;
import org.evolvica.ui.Images;
import org.evolvica.ui.PathManager;
import org.evolvica.ui.views.navigator.IResourceItem;
import org.evolvica.ui.views.navigator.ProjectNavigator;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:53:32 $
 */
public class RefreshAction extends NavigatorAction {
	
	public static final String ID = "org.evolvica.ui.navigator.actions.RefreshAction";
	
	public RefreshAction( ProjectNavigator navigator ) {
		super( navigator, "Refresh" );
		setImageDescriptor( Images.getImageDescriptor( "icons/refresh16.png" ) );
		setId( ID );
	}
	
	public void runOnSelection( Object selectedItem ) {
		IResourceItem selected = navigator.getSelectedItem();
		IJavaProject javaProject = selected.getJavaProject();
		try {
			ProgressMonitorDialog dialog = new ProgressMonitorDialog( navigator.getSite().getShell() );
			dialog.open();
			dialog.run( true, false, new RefreshRunnable( javaProject ) );
			dialog.close();
		} catch ( Exception ex ) {
			ex.printStackTrace();
		}
		CacheManager.clearCache( PathManager.getOutputLocation( javaProject ) );
		URI[] classpath = PathManager.getClassPath( javaProject );
		for ( int i = 0; i < classpath.length; i++ ) {
			CacheManager.clearCache( classpath[i] );
		}
		navigator.refresh( selected );
	}
	
	public boolean isEnabled() {
		return true;
	}
	
	private class RefreshRunnable implements IRunnableWithProgress {
		
		private IJavaProject javaProject = null;
		
		private RefreshRunnable( IJavaProject project ) {
			this.javaProject = project;
		}
		
		public void run( IProgressMonitor monitor ) throws InvocationTargetException, InterruptedException {
			try {
				javaProject.getProject().refreshLocal( IResource.DEPTH_INFINITE, monitor );
				javaProject.getProject().build( IncrementalProjectBuilder.INCREMENTAL_BUILD, monitor );
			} catch ( CoreException cx ) {
				cx.printStackTrace();
			}
			
		}
		
	}
	
}
