/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:33 $
 * $Revision: 1.4 $
 * $State: Exp $
 * $RCSfile: RenameAction.java,v $
 * 
 */
package org.evolvica.ui.views.navigator.actions;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import org.apache.commons.lang.StringUtils;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.*;
import org.eclipse.jdt.core.*;
import org.eclipse.jdt.ui.refactoring.RenameSupport;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.evolvica.engine.AlgorithmBuilder;
import org.evolvica.engine.AlgorithmWriter;
import org.evolvica.engine.OperatorDescriptor;
import org.evolvica.ui.AlgorithmEditorPlugin;
import org.evolvica.ui.views.navigator.*;
import org.w3c.dom.Document;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.4 $ $Date: 2004/05/26 11:53:33 $
 */
public class RenameAction extends NavigatorAction {
	
	public static final String ID = "org.evolvica.ui.navigator.actions.RenameAction";
	
	public RenameAction( ProjectNavigator explorer ) {
		super( explorer, "Rename" );
		setId( ID );
	}
	
	public void runOnSelection( Object selectedItem ) {
		try {
			if ( selectedItem instanceof AlgorithmItem ) {
				renameAlgorithm( ((AlgorithmItem) selectedItem).getFile() );
			} else if ( selectedItem instanceof ProjectFolder ) {
				RenameSupport.create( ((ProjectFolder) selectedItem).getJavaProject(), null, RenameSupport.NONE ).openDialog( navigator.getSite().getShell() );
				navigator.refresh();
			} else if ( selectedItem instanceof OperatorItem ) {
				renameOperator( ((OperatorItem) selectedItem).getDescriptor() );
			}
		} catch ( CoreException cx ) {
			MessageDialog.openError( navigator.getViewSite().getShell(), "Error", "unable to perform renaming: " + cx.getMessage() );
		}
	}
	
	private void renameAlgorithm( IFile file ) throws CoreException {
		String oldName = file.getName();
		oldName = oldName.substring( 0, oldName.lastIndexOf( "." ) );
		InputDialog dialog = new InputDialog( navigator.getSite().getShell(), "Rename", "Enter new name for algorithm", oldName, null );
		int rcode = dialog.open();
		if ( rcode == InputDialog.OK ) {
			String newName = dialog.getValue();
			if ( newName.length() > 0 && ! newName.equals( oldName ) ) {
				try {
					Document document = new AlgorithmBuilder().buildDocument( file.getContents() );
					document.getDocumentElement().setAttribute( "name", newName );
					ByteArrayOutputStream output = new ByteArrayOutputStream();
					new AlgorithmWriter().write( document, output );
					ByteArrayInputStream input = new ByteArrayInputStream( output.toByteArray() );
					file.setContents( input, true, false, null );
					file.move( file.getProjectRelativePath().removeLastSegments( 1 ).append( newName + ".eva" ), false, true, null );
					file.refreshLocal( IResource.DEPTH_ONE, null );
					input.close();
					output.close();
					navigator.refresh( navigator.getSelectedItem().getParent() );
				} catch ( Exception ex ) {
					throw new CoreException( new Status(
						IStatus.ERROR,
						AlgorithmEditorPlugin.getDefault().getDescriptor().getUniqueIdentifier(), -1,
						ex.getMessage(), ex )
					);
				}
			}
		}
	}
	
	private void renameOperator( OperatorDescriptor descriptor ) throws CoreException {
		String oldName = descriptor.getName();
		InputDialog dialog = new InputDialog( navigator.getSite().getShell(), "Rename", "Enter new class name for operator", oldName, null );
		int rcode = dialog.open();
		if ( rcode == InputDialog.OK ) {
			String newName = dialog.getValue();
			if ( newName.length() > 0 && ! newName.equals( oldName ) ) {
				try {
					String fullOldName = descriptor.getBeanClassName();
					String fullNewName = null;
					if ( fullOldName.indexOf( "." ) > 0 ) {
						fullNewName = fullOldName.substring( 0, fullOldName.lastIndexOf( "." ) ) + "." + newName;
					} else {
						fullNewName = newName;
					}
					IJavaProject project = navigator.getSelectedItem().getJavaProject();
					IPackageFragmentRoot[] roots = project.getPackageFragmentRoots();
					for ( int i = 0; i < roots.length; i++ ) {
						if ( roots[i].getKind() == IPackageFragmentRoot.K_SOURCE ) {
							// rename source file
							IFolder folder = (IFolder) roots[i].getCorrespondingResource();
							IResource resource = folder.findMember( new Path( StringUtils.replace( fullOldName, ".", "/" ) + ".java" ) );
							ICompilationUnit unit = JavaCore.createCompilationUnitFrom( (IFile) resource );
							RenameSupport support = RenameSupport.create( unit, newName, RenameSupport.UPDATE_REFERENCES );
							support.perform( navigator.getSite().getShell(), new ProgressMonitorDialog( navigator.getSite().getShell() ) );
							// edit descriptor file
							resource = folder.findMember( new Path( StringUtils.replace( fullOldName, ".", "/" ) + ".descriptor" ) );
							IFile file = (IFile) resource;
							Document document = new AlgorithmBuilder().buildDocument( file.getContents() );
							document.getDocumentElement().setAttribute( "beanClass", fullNewName );
							document.getDocumentElement().setAttribute( "displayName", newName );
							ByteArrayOutputStream output = new ByteArrayOutputStream();
							new AlgorithmWriter().write( document, output );
							ByteArrayInputStream input = new ByteArrayInputStream( output.toByteArray() );
							file.setContents( input, true, false, null );
							input.close();
							output.close();
							// rename descriptor file
							file.move( file.getFullPath().removeLastSegments( 1 ).append( newName + ".descriptor" ), false, true, null );
							file.refreshLocal( IResource.DEPTH_ONE, null );
						}
					}
				} catch ( Exception ex ) {
					throw new CoreException( new Status(
						IStatus.ERROR,
						AlgorithmEditorPlugin.getDefault().getDescriptor().getUniqueIdentifier(), -1,
						ex.getMessage(), ex )
					);
				}
			}
		}
	}
	
	public boolean isEnabled() {
		IResourceItem selectedItem = navigator.getSelectedItem();
		if ( selectedItem != null ) {
			if ( selectedItem instanceof AlgorithmItem ) {
				return true;
			} else if ( selectedItem instanceof ProjectFolder ) {
				return true;
			} else if ( selectedItem instanceof OperatorItem && ((OperatorItem) selectedItem).isLocal() ) {
				return true;
			}
		}
		return false;
	}
	
}
