/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:33 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: AlgorithmContentProvider.java,v $
 * 
 */
package org.evolvica.ui.views.outline;

import java.util.Arrays;
import java.util.Comparator;
import java.util.Iterator;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;
import org.evolvica.core.population.IPopulation;
import org.evolvica.engine.IAlgorithm;
import org.evolvica.engine.IElement;
import org.evolvica.engine.ISink;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:53:33 $
 */
public class AlgorithmContentProvider implements ITreeContentProvider {
	
	private static final OperatorComparator comparator = new OperatorComparator();
	
	public Object[] getChildren( Object parentElement ) {
		if ( parentElement instanceof IAlgorithm )  {
			IElement[] children = new IElement[((IAlgorithm) parentElement).elementSize()];
			int index = 0;
			for ( Iterator it = ((IAlgorithm) parentElement).iterateElements(); it.hasNext(); ) {
				children[index++] = (IElement) it.next();
			}
			Arrays.sort( children, comparator );
			return children;
		} else if ( parentElement instanceof IElement ) {
			IElement e = (IElement) parentElement;
			if ( e.getOperator() instanceof ISink || e.getOperator() instanceof IPopulation ) {
				return new Object[] { "no content" };
			}
		}
		return new Object[0];
	}

	public Object getParent( Object element ) {
		if ( element instanceof IElement ) {
			return ((IElement) element).getAlgorithm();
		}
		return null;
	}

	public boolean hasChildren( Object element ) {
		if ( element instanceof IAlgorithm ) {
			IAlgorithm algorithm = (IAlgorithm) element;
			if ( algorithm.elementSize() > 0 ) {
				return true;
			}
		} else if ( element instanceof IElement ) {
			IElement e = (IElement) element;
			if ( e.getOperator() instanceof ISink || e.getOperator() instanceof IPopulation ) {
				return true;
			}
		}
		return false;
	}
	
	public Object[] getElements( Object inputElement ) {
		if ( inputElement instanceof OutlineRootItem ) {
			return new Object[] { ((OutlineRootItem) inputElement).getItem() };
		} else {
			return new Object[0];
		}
	}

	public void dispose() {
		
	}

	public void inputChanged( Viewer viewer, Object oldInput, Object newInput ) {
		// do nothing yet
	}
	
	private static class OperatorComparator implements Comparator {

		public int compare( Object o1, Object o2 ) {
			IElement e1 = (IElement) o1;
			IElement e2 = (IElement) o2;
			return e1.getOperator().getName().compareTo( e2.getOperator().getName() );
		}
		
	}
	
}
