/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:34 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: AbstractContentItem.java,v $
 *
 */
package org.evolvica.ui.views.properties;

import java.util.HashSet;
import java.util.Iterator;

/**
 * Documentation missing.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:53:34 $
 * @since RELEASE_NUMBER_NOT_DETERMINED
 */
abstract public class AbstractContentItem implements IContentItem {
	
	protected boolean expert = false;
	
	protected boolean readonly = false;
	
	protected String label = null;
	
	protected IContentItem parent = null;
	
	protected Object value = null;
	
	protected HashSet children = new HashSet();
	
	public AbstractContentItem( IContentItem parent, String label, Object value ) {
		 this.parent = parent;
		 this.label = label;
		 this.value = value;
	}
	
	public void clearCache() {
		children.clear();
	}
	
	public String getLabel() {
		return label;
	}
	
	public IContentItem getParent() {
		return parent;
	}
	
	public Object getValue() {
		return value;
	}
	
	public boolean hasChildren() {
		if ( children.size() > 0 ) {
			return true;
		} else {
			return false;
		}
	}
	
	public boolean isExpert() {
		return expert;
	}
	
	public boolean isReadonly() {
		return readonly;
	}
	
	public void setExpert( boolean expert ) {
		this.expert = expert;
	}
	
	public void setReadonly( boolean readonly ) {
		this.readonly = readonly;
	}
	
	public void setValue( Object value ) throws Exception {
		if ( ! isReadonly() ) {
			setValue( value, true );
		}
	}
	
	public IContentItem[] getChildren() {
		return (IContentItem[]) children.toArray( new IContentItem[children.size()] );
	}
	
	protected ArrayContentItem findArrayItem( String property, int index ) {
		for ( Iterator it = children.iterator(); it.hasNext(); ) {
			IContentItem item = (IContentItem) it.next();
			if ( item instanceof ArrayContentItem ) {
				ArrayContentItem arrayItem = (ArrayContentItem) item;
				if ( property.equals( arrayItem.property ) && index == arrayItem.index ) {
					return arrayItem;
				}
			}
		}
		return null;
	}
	
	protected BeanContentItem findBeanItemByProperty( String property ) {
		for ( Iterator it = children.iterator(); it.hasNext(); ) {
			IContentItem item = (IContentItem) it.next();
			if ( item instanceof BeanContentItem ) {
				if ( ((BeanContentItem) item ).property.equals( property ) ) {
					return (BeanContentItem) item;
				}
			}
		}
		return null;
	}
	
	protected MetaContentItem findMetaItemByLabel( String label ) {
		for ( Iterator it = children.iterator(); it.hasNext(); ) {
			IContentItem item = (IContentItem) it.next();
			if ( item instanceof MetaContentItem && label.equals( ((MetaContentItem) item).getLabel() ) ) {
				return (MetaContentItem) item;
			}
		}
		return null;
	}
	
	public boolean equals( Object arg0 ) {
		if ( this == arg0 ) {
			return true;
		} else {
			return false;
		}
	}
	
	public String toString() {
		return getLabel() + ", value " + getValue();
	}
	
}
