/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:35 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: AlgorithmPropertiesSheetPage.java,v $
 *
 */
package org.evolvica.ui.views.properties;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.TableTree;
import org.eclipse.swt.custom.TableTreeItem;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseTrackListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.evolvica.engine.AttributeDescriptor;
import org.evolvica.engine.IElement;
import org.evolvica.engine.IOperator;
import org.evolvica.ui.views.properties.actions.*;

/**
 * Documentation missing.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:53:35 $
 * @since RELEASE_NUMBER_NOT_DETERMINED
 */
public class AlgorithmPropertiesSheetPage implements IPropertySheetPage {
	
	protected TableTreeViewer viewer = null;
	
	protected TableTree tableTree = null;
	
	protected Table table = null;
	
	protected TableColumn nameColumn = null;
	
	protected TableColumn typeColumn = null;
	
	protected TableColumn valueColumn = null;
	
	protected Action[] actions = new Action[7];
	
	public void createControl( Composite parent ) {
		viewer = new TableTreeViewer( parent, SWT.SINGLE | SWT.FULL_SELECTION );
		tableTree = viewer.getTableTree();
		table = tableTree.getTable();
		nameColumn = new TableColumn( table, SWT.NONE );
		nameColumn.setText( "Name" );
		typeColumn = new TableColumn( table, SWT.NONE );
		typeColumn.setText( "Type" );
		typeColumn.setAlignment( SWT.CENTER );
		valueColumn = new TableColumn( table, SWT.NONE );
		valueColumn.setText( "Value" );
		initPartControl();
	}

	protected void initPartControl() {
		// table appearance
		table.setHeaderVisible( true );
		table.setLinesVisible( true );
		// table column layout
		ColumnLayoutData nameColumnLayout = new ColumnWeightData( 25, false );
		ColumnLayoutData typeColumnLayout = new ColumnWeightData( 5, false );
		ColumnLayoutData valueColumnLayout = new ColumnWeightData( 70, true );
		TableLayout layout = new TableLayout();
		layout.addColumnData( nameColumnLayout );
		layout.addColumnData( typeColumnLayout );
		layout.addColumnData( valueColumnLayout );
		table.setLayout( layout );
		// attach content/label provider
		viewer.setContentProvider( new AttributeContentProvider() );
		viewer.setLabelProvider( new AttributeLabelProvider() );
		// set sorter
		viewer.setSorter( new AttributeContentSorter() );
		// add filter
		viewer.addFilter( ExpertAttributeFilter.instance );
		// attach modifier/editor
		viewer.setColumnProperties( new String[] { "names", "types", "values" } );
		viewer.setCellModifier( new AttributeContentModifier( viewer ) );
		viewer.setCellEditors( new CellEditor[] { null, null, new AttributeContentEditor( table ) } );
		// selection handler
		viewer.addSelectionChangedListener( new ContentSelectionListener() );
		// handler for tooltips
		table.addMouseTrackListener( new TooltipTrackListener() );
	}
	
	public void dispose() {
		
	}
	
	public Control getControl() {
		if ( viewer == null ) {
			return null;
		}
		return viewer.getControl();
	}

	public void setActionBars( IActionBars actionBars ) {
		actions[0] = new AppendArrayItemAsChildAction( viewer );
		actions[1] = new RemoveArrayItemFromChildAction( viewer ); 
		actions[2] = new AppendArrayItemAction( viewer );
		actions[3] = new RemoveArrayItemAction( viewer );
		actions[4] = new SetItemNullAction( viewer );
		actions[5] = new FilterExpertAttributesAction( viewer );
		actions[6] = new FilterReadonlyAttributesAction( viewer );
		for ( int i = 0; i < actions.length; i++ ) {
			actionBars.getToolBarManager().add( actions[i] );
		}
	}

	public void setFocus() {
		
	}

	public void selectionChanged( IWorkbenchPart part, ISelection selection ) {
		if ( selection instanceof IStructuredSelection ) {
			Object object = ((IStructuredSelection) selection).getFirstElement();
			if ( object != null ) {
				viewer.cancelEditing();
				viewer.setInput( object );
			}
		}
	}
	
	protected class ContentSelectionListener implements ISelectionChangedListener {
		
		public void selectionChanged( SelectionChangedEvent event ) {
			ISelection selection = event.getSelection();
			if ( selection != null && selection instanceof IStructuredSelection ) {
				IContentItem item = (IContentItem) ((IStructuredSelection) selection).getFirstElement();
				if ( item instanceof BeanContentItem && ! item.isReadonly() ) {
					actions[4].setEnabled( true );
					if ( ((BeanContentItem) item).getType().isArray() ) {
						actions[0].setEnabled( true );
						actions[1].setEnabled( true );
						actions[2].setEnabled( false );
						actions[3].setEnabled( false );
					} else {
						actions[0].setEnabled( false );
						actions[1].setEnabled( false );
						actions[2].setEnabled( false );
						actions[3].setEnabled( false );
					}
				} else if ( item instanceof ArrayContentItem && ! item.isReadonly() ) {
					actions[4].setEnabled( true );
					if ( ((ArrayContentItem) item).getType().isArray() ) {
						actions[0].setEnabled( true );
						actions[1].setEnabled( true );
						actions[2].setEnabled( true );
						actions[3].setEnabled( true );
					} else {
						actions[0].setEnabled( false );
						actions[1].setEnabled( false );
						actions[2].setEnabled( true );
						actions[3].setEnabled( true );
					}
				} else {
					for ( int i = 0; i < 5; i++ ) {
						actions[i].setEnabled( false );
					}
				}
			}
		}
		
	}
	
	protected class TooltipTrackListener implements MouseTrackListener {

		public void mouseEnter( MouseEvent e ) {
			AlgorithmPropertiesSheetPage.this.table.setToolTipText( null );
		}

		public void mouseExit( MouseEvent e ) {
			AlgorithmPropertiesSheetPage.this.table.setToolTipText( null );
		}

		public void mouseHover( MouseEvent e ) {
			TableItem item = AlgorithmPropertiesSheetPage.this.table.getItem( new Point( e.x, e.y ) );
			if ( item != null ) {
				TableTreeItem treeItem = AlgorithmPropertiesSheetPage.this.tableTree.getItem( new Point( e.x, e.y ) );
				IContentItem contentItem = (IContentItem) treeItem.getData();
				if ( contentItem instanceof BeanContentItem && contentItem.getParent() instanceof RootContentItem ) {
					BeanContentItem beanItem = (BeanContentItem) contentItem;
					RootContentItem root = (RootContentItem) contentItem.getParent();
					if ( root.getValue() instanceof IElement ) {
						IOperator operator = ((IElement) root.getValue()).getOperator();
						AttributeDescriptor ad = operator.descriptor().getAttribute( beanItem.property );
						if ( ad != null ) {
							AlgorithmPropertiesSheetPage.this.table.setToolTipText( ad.getShortDescription() );
						}
					}
				} else if ( contentItem instanceof MetaContentItem && contentItem.getParent() instanceof RootContentItem ) {
					if ( ((MetaContentItem) contentItem).label.equals( "description" ) ) {
						AlgorithmPropertiesSheetPage.this.table.setToolTipText( String.valueOf( ((MetaContentItem) contentItem).value ) );
					}
				}
			}
		}
		
	}
}
