/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:35 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: ArrayContentItem.java,v $
 *
 */
package org.evolvica.ui.views.properties;

import java.beans.PropertyDescriptor;
import java.lang.reflect.Array;
import java.util.HashSet;

import org.apache.commons.beanutils.PropertyUtils;
import org.evolvica.util.Convert;

/**
 * Documentation missing.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:53:35 $
 * @since RELEASE_NUMBER_NOT_DETERMINED
 */
public class ArrayContentItem extends AbstractContentItem {
	
	protected Object array = null;
	
	protected String property = null;
	
	protected int index = -1;
	
	public ArrayContentItem( Object array, IContentItem parent, String property, int index ) {
		super( parent, property + "[" + index + "]", null );
		this.array = array;
		this.property = property;
		this.index = index;
	}
	
	public void synchronize() {
		try {
			Object value = Array.get( array, index );
			if ( value != null ) {
				Class type = value.getClass();
				if ( type.isArray() ) {
					int length = Array.getLength( value );
					HashSet retain = new HashSet();
					for ( int i = 0; i < length; i++ ) {
						ArrayContentItem arrayItem = findArrayItem( property, i );
						if ( arrayItem == null ) {
							arrayItem = new ArrayContentItem( value, this, property, i );
							children.add( arrayItem );
						}
						retain.add( arrayItem );
					}
					children.retainAll( retain );
				} else {
					HashSet retain = new HashSet();
					if ( value instanceof Class && type.equals( Class.class ) ) {
						MetaContentItem metaItem = findMetaItemByLabel( "class" );
						if ( metaItem == null ) {
							metaItem = new MetaContentItem( this, "class", Convert.toClassname( (Class) value ) );
						}
						retain.add( metaItem );
					} else {
						PropertyDescriptor[] pd = PropertyUtils.getPropertyDescriptors( value );
						for ( int i = 0; i < pd.length; i++ ) {
							BeanContentItem beanItem = findBeanItemByProperty( pd[i].getName() );
							if ( beanItem == null ) {
								beanItem = new BeanContentItem( value, this, pd[i].getName() );
								children.add( beanItem );
							}
							retain.add( beanItem );
						}
					}
					children.retainAll( retain );
				}
			}
		} catch ( Exception ex ) {
			// be quiet
		}
		/*
		for ( Iterator it = children.iterator(); it.hasNext(); ) {
			((IContentItem) it.next()).synchronize();
		}
		*/
	}
	
	/*
	public IContentItem[] getChildren() {
		try {
			Object value = Array.get( array, index );
			if ( value != null ) {
				Class type = value.getClass();
				if ( type.isArray() ) {
					int length = Array.getLength( value );
					IContentItem[] children = new IContentItem[length];
					for ( int i = 0; i < length; i++ ) {
						children[i] = new ArrayContentItem( value, this, property, i );
					}
					return children;
				} else {
					PropertyDescriptor[] pd = PropertyUtils.getPropertyDescriptors( value );
					IContentItem[] children = new IContentItem[pd.length];
					for ( int i = 0; i < pd.length; i++ ) {
						children[i] = new BeanContentItem( value, this, pd[i].getName() );
					}
					return children;
				}
			}
		} catch ( Exception ex ) {
			// be quiet
		}
		return new IContentItem[0];
	}
	*/
	
	public Object getArray() {
		return array;
	}
	
	public int getIndex() {
		return index;
	}
	
	public Class getType() {
		try {
			return array.getClass().getComponentType();
		} catch ( Exception ex ) {
			return Object.class;
		}
	}
	
	public Object getValue() {
		try {
			return Array.get( array, index );
		} catch ( Exception ex ) {
			return null;
		}
	}

	public boolean hasChildren() {
		try {
			Object value = Array.get( array, index );
			if ( value != null ) {
				Class type = value.getClass();
				if ( type.isArray() && Array.getLength( value ) > 0 ) {
					return true;
				} else if ( Convert.isConvertibleFromString( type ) ) {
					return false;
				} else {
					return true;
				}
			}
		} catch ( Exception ex ) {
			// be quiet
		}
		return false;
	}
	
	public boolean isReadonly() {
		return parent.isReadonly();
	}
	
	public void resetValue() throws Exception {
		Class type = array.getClass().getComponentType();
		if ( Convert.isConvertibleToBoolean( type ) ) {
			Array.set( array, index, new Boolean( false ) );
		} else if ( type.isPrimitive() ) {
			Array.set( array, index, new Integer( 0 ) );
		} else {
			Array.set( array, index, null );
		}
	}

	public void setValue( Object value, boolean convert ) throws Exception {
		if ( convert == true ) {
			Class type = getType();
			if ( Convert.isConvertibleFromString( type ) ) {
				Array.set( array, index, Convert.toObject( (String) value, type ) );
			} else {
				Array.set( array, index, ((Class) value).newInstance() );
			}
		} else {
			Array.set( array, index, value );
		}
	}
	
}
