/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:35 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: AttributeContentEditor.java,v $
 * 
 */
package org.evolvica.ui.views.properties;

import java.io.File;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IType;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.*;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ElementListSelectionDialog;
import org.evolvica.ui.*;
import org.evolvica.util.Convert;
import org.evolvica.util.Directory;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:53:35 $
 */
public class AttributeContentEditor extends CellEditor {
	
	protected static final int BOOLEAN = 0;
	protected static final int STRING = 1;
	protected static final int DIRECTORY = 2;
	protected static final int FILE = 3;
	protected static final int CLASS = 4;
	
	protected Composite editor;
	
	protected Button button;
	
	protected Text text;
	
	private Class type;
	
	private Object value;
	
	private int mode = STRING;
	
	public AttributeContentEditor( Composite parent ) {
		super( parent );
	}
	
	public void activate() {
		if ( value != null ) {
			text.setText( Convert.toString( value ) );
		} else {
			text.setText( "null" );
		}
		if ( Convert.isConvertibleToBoolean( type ) ) {
			button.setEnabled( false );
			text.setEditable( false );
			text.setEnabled( true );
			text.setToolTipText( "" );
			mode = BOOLEAN;
		} else if ( type.equals( Directory.class ) ) {
			button.setEnabled( true );
			text.setEnabled( false );
			text.setToolTipText( "choose directory" );
			mode = DIRECTORY;
		} else if ( type.equals( File.class ) ) {
			button.setEnabled( true );
			text.setEnabled( false );
			text.setToolTipText( "choose file" );
			mode = FILE;
		} else if ( Convert.isConvertibleFromString( type ) ) {
			button.setEnabled( false );
			text.setEditable( true );
			text.setEnabled( true );
			text.setToolTipText( "" );
			mode = STRING;
		} else {
			if ( type.isArray() ) {
				button.setEnabled( false );
			} else {
				button.setEnabled( true );
			}
			text.setEnabled( false );
			if ( value != null ) {
				text.setText( Convert.toClassname( value ) );
			} else {
				text.setText( "null" );
			}
			text.setToolTipText( "choose class" );
			mode = CLASS;
		}
	}
	
	protected Control createControl( Composite parent ) {
		editor = new Composite( parent, getStyle() );
		editor.setFont( parent.getFont() );
		editor.setBackground( parent.getBackground() );
		editor.setLayout( new ContentCellLayout() );
		text = createText( editor );
//		updateContents(value);
		button = createButton(editor);
		// ESC key listener
		/*
		button.addKeyListener( new KeyAdapter() {
			public void keyReleased( KeyEvent e ) {
				if ( e.character == '\u001b' ) { // Escape
					fireCancelEditor();
				}
			}
		});
		*/
		// selection listener
		button.addSelectionListener( new SelectionAdapter() {
			public void widgetSelected( SelectionEvent event ) {
				value = openDialogBox( editor );
				if ( value != null ) {
					fireApplyEditorValue();
				}
			}
		});
		text.addKeyListener( new KeyAdapter() {
			public void keyPressed( KeyEvent e ) {
				keyReleaseOccured( e );
			}
		} );
		text.addMouseListener( new MouseAdapter() {
			public void mouseDown( MouseEvent e ) {
				if ( mode == BOOLEAN ) {
					if ( text.getText().equals( "true" ) ) {
						text.setText( "false" );
					} else {
						text.setText( "true" );
					}
					fireApplyEditorValue();
				}
			}
		});
		return editor;
 	}

	protected Object openDialogBox( Composite editor ) {
		Object rval = null;
		switch ( mode ) {
			case CLASS:
				ProgressMonitorDialog monitor = new ProgressMonitorDialog( editor.getShell() );
				monitor.setCancelable( false );
				monitor.open();
				IType[] types = findImplementors( monitor.getProgressMonitor() );
				monitor.close();
				ElementListSelectionDialog dialog = new ElementListSelectionDialog( editor.getShell(), new ClassLabelProvider() );
				dialog.setElements( types );
				dialog.setTitle( "Select Class" );
				dialog.open();
				IType itype = (IType) dialog.getFirstResult();
				if ( itype != null ) {
					try {
						rval = Class.forName( itype.getFullyQualifiedName() ); // IMPROVE: use own classloader here?
					} catch ( Exception ex ) {
						UILog.error( "unable to instantiate class " + itype.getElementName(), ex );
					}
				}
				break;
			case DIRECTORY:
				rval = new DirectoryDialog( editor.getShell() ).open();
				break;
			case FILE:
				rval = new FileDialog( editor.getShell() ).open();
				break;
			default:
				break;
		}
		return rval;
	}

	protected Button createButton( Composite parent ) {
		Button result = new Button( parent, SWT.DOWN );
		result.setFont( parent.getFont() );
		result.setText( "!" );
		return result;
	}
	
	protected Text createText( Composite cell ) {
		text = new Text( cell, SWT.LEFT );
		text.setFont( cell.getFont() );
		text.setBackground( cell.getBackground() );
		return text;
	}
	
 	protected Object doGetValue() {
 		switch ( mode ) {
			case BOOLEAN:
				return text.getText();
			case STRING:
				return text.getText();
			case DIRECTORY:
				return value;
			case FILE:
				return value;
			case CLASS:
				return value;
		}
		return null;
	}

	protected void doSetFocus() {
		text.setFocus();
	}

	protected void doSetValue( Object value ) {
		/*
		type = ((AttributeItem) value).getType();
		value = ((AttributeItem) value).getValue();
		*/
		this.value = ((IContentItem) value).getValue();
		if ( value instanceof BeanContentItem ) {
			type = ((BeanContentItem) value).getType();
		} else if ( value instanceof ArrayContentItem ) {
			type = ((ArrayContentItem) value).getType();
		} else {
			type = this.value.getClass();
		}
	}

	protected IType[] findImplementors( IProgressMonitor monitor ) {
		AlgorithmEditor editor = (AlgorithmEditor) PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		try {
			IJavaProject project = editor.getJavaProject();
			IType itype = project.findType( type.getName() );
			IType[] implementors = ClassManager.findImplementors( project, itype, monitor );
			return implementors;
		} catch ( Exception ex ) {
			return new IType[0];
		}
	}
	
	private class ContentCellLayout extends Layout {
		
		public void layout( Composite editor, boolean force ) {
			Rectangle bounds = editor.getClientArea();
			Point size = button.computeSize( SWT.DEFAULT, SWT.DEFAULT, force );
			if ( text != null ) {
				text.setBounds(0, 0, bounds.width-size.x, bounds.height );
			}
			button.setBounds( bounds.width-size.x, 0, size.x, bounds.height );
		}
		
		public Point computeSize( Composite editor, int wHint, int hHint, boolean force ) {
			if ( wHint != SWT.DEFAULT && hHint != SWT.DEFAULT ) {
				return new Point( wHint, hHint );
			}
			Point contentsSize = text.computeSize( SWT.DEFAULT, SWT.DEFAULT, force ); 
			Point buttonSize =  button.computeSize( SWT.DEFAULT, SWT.DEFAULT, force );
			// Just return the button width to ensure the button is not clipped
			// if the label is long.
			// The label will just use whatever extra width there is
			Point result = new Point( buttonSize.x, Math.max( contentsSize.y, buttonSize.y ) );
			return result;
		}
		
	}

}
