/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/06/26 09:34:47 $
 * $Revision: 1.3 $
 * $State: Exp $
 * $RCSfile: BeanContentItem.java,v $
 *
 */
package org.evolvica.ui.views.properties;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyDescriptor;
import java.io.File;
import java.lang.reflect.Array;
import java.lang.reflect.Method;
import java.util.HashSet;
import java.util.Iterator;

import org.apache.commons.beanutils.PropertyUtils;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;
import org.evolvica.engine.IElement;
import org.evolvica.engine.IOperator;
import org.evolvica.ui.AlgorithmEditor;
import org.evolvica.ui.UIEvents;
import org.evolvica.util.Convert;
import org.evolvica.util.Directory;

/**
 * Documentation missing.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.3 $ $Date: 2004/06/26 09:34:47 $
 * @since RELEASE_NUMBER_NOT_DETERMINED
 */
public class BeanContentItem extends AbstractContentItem {
	
	private static final HashSet BLACKLIST = new HashSet();
	
	static {
		BLACKLIST.add( Directory.class );
		BLACKLIST.add( File.class );
	}
	
	protected Object bean = null;
	
	protected String property = null;
	
	public BeanContentItem( Object bean, IContentItem parent, String property ) {
		this( bean, parent, property, property );
	}
	
	public BeanContentItem( Object bean, IContentItem parent, String property, String label ) {
		super( parent, label, null );
		this.bean = bean;
		this.property = property;
	}
	
	public void synchronize() {
		try {
			Object value = PropertyUtils.getProperty( bean, property );
			Class type = PropertyUtils.getPropertyType( bean, property );
			if ( value != null && ! BLACKLIST.contains( type ) ) {
				if ( type.isArray() ) {
					int length = Array.getLength( value );
					HashSet retain = new HashSet();
					for ( int i = 0; i < length; i++ ) {
						ArrayContentItem arrayItem = findArrayItem( property, i );
						if ( arrayItem == null ) {
							arrayItem = new ArrayContentItem( value, this, property, i );
							children.add( arrayItem );
						}
						retain.add( arrayItem );
					}
					children.retainAll( retain );
				} else {
					HashSet retain = new HashSet();
					if ( value instanceof Class && type.equals( Class.class ) ) {
						MetaContentItem metaItem = findMetaItemByLabel( "class" );
						if ( metaItem == null ) {
							metaItem = new MetaContentItem( this, "class", Convert.toClassname( (Class) value ) );
						}
						retain.add( metaItem );
					} else {
						PropertyDescriptor[] pd = PropertyUtils.getPropertyDescriptors( value );
						for ( int i = 0; i < pd.length; i++ ) {
							BeanContentItem beanItem = findBeanItemByProperty( pd[i].getName() );
							if ( beanItem == null ) {
								beanItem = new BeanContentItem( value, this, pd[i].getName() );
								children.add( beanItem );
							}
							retain.add( beanItem );
						}
						children.retainAll( retain );
					}
				}
			}
		} catch ( Exception ex ) {
			// be quiet
		}
		for ( Iterator it = children.iterator(); it.hasNext(); ) {
			((IContentItem) it.next()).synchronize();
		}
	}

	/*
	public IContentItem[] getChildren() {
		try {
			Object value = PropertyUtils.getProperty( bean, property );
			Class type = PropertyUtils.getPropertyType( bean, property );
			if ( value != null ) {
				if ( type.isArray() ) {
					int length = Array.getLength( value );
					IContentItem[] children = new IContentItem[length];
					for ( int i = 0; i < length; i++ ) {
						children[i] = new ArrayContentItem( value, this, property, i );
					}
					return children;
				} else {
					PropertyDescriptor[] pd = PropertyUtils.getPropertyDescriptors( value );
					IContentItem[] children = new IContentItem[pd.length];
					for ( int i = 0; i < pd.length; i++ ) {
						children[i] = new BeanContentItem( value, this, pd[i].getName() );
					}
					return children;
				}
			}
		} catch ( Exception ex ) {
			// be quiet
		}
		return new IContentItem[0];
	}
	*/
	
	public Object getValue() {
		try {
			return PropertyUtils.getProperty( bean, property );
		} catch ( Exception ex ) {
			return null;
		}
	}
	
	public Class getType() {
		try {
			return PropertyUtils.getPropertyType( bean, property );
		} catch ( Exception ex ) {
			return Object.class;
		}
	}
	
	public boolean hasChildren() {
		try {
			Object value = PropertyUtils.getProperty( bean, property );
			Class type = PropertyUtils.getPropertyType( bean, property );
			if ( value != null ) {
				if ( type.isArray() && Array.getLength( value ) > 0 ) {
					return true;
				} else if ( Convert.isConvertibleFromString( type ) ) {
					return false;
				} else {
					return true;
				}
			}
		} catch ( Exception ex ) {
			// be quiet
		}
		return false;
	}

	public boolean isReadonly() {
		try {
			PropertyDescriptor descriptor = PropertyUtils.getPropertyDescriptor( bean, property );
			Method read = PropertyUtils.getReadMethod( descriptor );
			Method write = PropertyUtils.getWriteMethod( descriptor );
			if ( read != null && write != null ) {
				return false;
			}
		} catch ( Exception ex ) {
			// be quiet
		}
		return true;
	}
	
	public void resetValue() throws Exception {
		Class type = getType();
		if ( Convert.isConvertibleToBoolean( type ) ) {
			PropertyUtils.setProperty( bean, property, new Boolean( false ) );
		} else if ( type.isPrimitive() ) {
			PropertyUtils.setProperty( bean, property, new Integer( 0 ) );
		} else {
			PropertyUtils.setProperty( bean, property, null );
		}
	}
	
	public void setValue( Object value, boolean convert ) throws Exception {
		if ( convert == true ) {
			Class type = getType();
			if ( Convert.isConvertibleFromString( type ) ) {
				PropertyUtils.setProperty( bean, property, Convert.toObject( (String) value, type ) );
			} else {
				PropertyUtils.setProperty( bean, property, ((Class) value).newInstance() );
			}
			// synchronize child list
			synchronize();
			// handle a name change separately
			if ( bean instanceof IOperator && property.equals( "name" ) ) {
				IEditorPart active = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
				if ( active instanceof AlgorithmEditor ) {
					IElement element = ((AlgorithmEditor) active).getAlgorithm().getElementById( ((IOperator) bean).getId() );
					if ( element != null ) {
						element.firePropertyChangeEvent( new PropertyChangeEvent( element,UIEvents.ELEMENT_RENAMED, null, value ) );
					}
				}
			}
		} else {
			PropertyUtils.setProperty( bean, property, value );
		}
	}
	
}
