/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:37 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: RootContentItem.java,v $
 *
 */
package org.evolvica.ui.views.properties;

import java.beans.PropertyDescriptor;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.apache.commons.beanutils.PropertyUtils;
import org.evolvica.engine.*;
import org.evolvica.ui.UIEvents;

/**
 * Documentation missing.
 *
 * @author Andreas Rummler
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:53:37 $
 * @since RELEASE_NUMBER_NOT_DETERMINED
 */
public class RootContentItem extends AbstractContentItem {
	
	public RootContentItem( Object root ) {
		super( null, root.toString(), root );
	}
	
	public void synchronize() {
		if ( value instanceof IAlgorithm ) {
			synchronizeAlgorithm();
		} else if ( value instanceof IConnection ) {
			synchronizeConnection();
		} else if ( value instanceof IElement ) {
			synchronizeOperator();
		} else {
			clearCache();
		}
		for ( Iterator it = children.iterator(); it.hasNext(); ) {
			((IContentItem) it.next()).synchronize();
		}
	}
	
	protected void synchronizeAlgorithm() {
		IAlgorithm algorithm = (IAlgorithm) value;
		// name item
		BeanContentItem item = findBeanItemByProperty( "name" );
		if ( item == null ) {
			item = new BeanContentItem( algorithm, this, "name" );
			children.add( item );
		}
		item.bean = algorithm;
		// algorithm item
		item = findBeanItemByProperty( "runner" );
		if ( item == null ) {
			item = new BeanContentItem( algorithm, this, "runner", "algorithm runner" );
			children.add( item );
		}
		item.bean = algorithm;
		// validator item
		item = findBeanItemByProperty( "validator" );
		if ( item == null ) {
			item = new BeanContentItem( algorithm, this, "validator", "algorithm validator" );
			children.add( item );
		}
		item.bean = algorithm;
	}
	
	protected void synchronizeConnection() {
		IConnection connection = (IConnection) value;
		// source
		MetaContentItem item = findMetaItemByLabel( "source" );
		if ( item == null ) {
			item = new MetaContentItem( this, "source", connection.getSource().getOperator().getName() );
			children.add( item );
		}
		item.label = connection.getSource().getOperator().getName();
		// source slot
		item = findMetaItemByLabel( "source slot" );
		if ( item == null ) {
			item = new MetaContentItem( this, "source slot", String.valueOf( connection.getSource().getOutgoingIndex( connection ) ) );
			children.add( item );
		}
		item.label = String.valueOf( connection.getSource().getOutgoingIndex( connection ) );
		// target
		item = findMetaItemByLabel( "target" );
		if ( item == null ) {
			item = new MetaContentItem( this, "target", connection.getTarget().getOperator().getName() );
			children.add( item );
		}
		item.label =  connection.getTarget().getOperator().getName();
		// target slot
		item = findMetaItemByLabel( "target slot" );
		if ( item == null ) {
			item = new MetaContentItem( this, "target slot", String.valueOf( connection.getTarget().getIncomingIndex( connection ) ) );
			children.add( item );
		}
		item.label = String.valueOf( connection.getTarget().getIncomingIndex( connection ) );
	}
	
	protected void synchronizeOperator() {
		IElement element = (IElement) value;
		IOperator operator = element.getOperator();
		OperatorDescriptor desc = operator.descriptor();
		// add information from descriptor
		MetaContentItem metaItem = findMetaItemByLabel( "location" );
		if ( metaItem == null ) {
			String x = element.getMetadata( UIEvents.ELEMENT_X ).getValue();
			String y = element.getMetadata( UIEvents.ELEMENT_Y ).getValue();
			metaItem = new MetaContentItem( this, "location", "(" + x + "," + y + ")", true );
			metaItem.label = "(" + x + "," + y + ")";
			children.add( metaItem );
		}
		metaItem = findMetaItemByLabel( "category" );
		if ( metaItem == null ) {
			metaItem = new MetaContentItem( this, "category", desc.getCategory(), true );
			metaItem.label = desc.getCategory();
			children.add( metaItem );
		}
		metaItem = findMetaItemByLabel( "datatype" );
		if ( metaItem == null ) {
			metaItem = new MetaContentItem( this, "datatype", desc.getDatatypeName(), true );
			metaItem.label = desc.getDatatypeName();
			children.add( metaItem );
		}
		metaItem = findMetaItemByLabel( "description" );
		if ( metaItem == null ) {
			metaItem = new MetaContentItem( this, "description", desc.getShortDescription() );
			metaItem.label = desc.getShortDescription();
			children.add( metaItem );
		}
		metaItem = findMetaItemByLabel( "display name" );
		if ( metaItem == null ) {
			metaItem = new MetaContentItem( this, "display name", desc.getDisplayName(), true );
			metaItem.label = desc.getDisplayName();
			children.add( metaItem );
		}
		metaItem = findMetaItemByLabel( "number of input slots" );
		if ( metaItem == null ) {
			metaItem = new MetaContentItem( this, "number of input slots", String.valueOf( desc.getInputSize() ), true );
			metaItem.label = String.valueOf( desc.getInputSize() );
			children.add( metaItem );
		}
		metaItem = findMetaItemByLabel( "number of output slots" );
		if ( metaItem == null ) {
			metaItem = new MetaContentItem( this, "number of output slots", String.valueOf( desc.getOutputSize() ), true );
			metaItem.label = String.valueOf( desc.getOutputSize() );
			children.add( metaItem );
		}
		metaItem = findMetaItemByLabel( "vendor" );
		if ( metaItem == null ) {
			metaItem = new MetaContentItem( this, "vendor", desc.getVendor(), true );
			metaItem.label = desc.getVendor();
			children.add( metaItem );
		}
		// sync bean items
		PropertyDescriptor[] pd = PropertyUtils.getPropertyDescriptors( operator );
		Set retain = new HashSet();
		for ( int i = 0; i < pd.length; i++ ) {
			BeanContentItem beanItem = findBeanItemByProperty( pd[i].getName() );
			if ( beanItem == null ) {
				beanItem = new BeanContentItem( operator, this, pd[i].getName() );
				children.add( beanItem );
				if ( pd[i].getName().equals( "id" ) ) {
					beanItem.setExpert( true );
					beanItem.setReadonly( true );
				}
			}
			retain.add( beanItem );
		}
		children.retainAll( retain );
	}
	
	/*
	public IContentItem[] getChildren() {
		if ( value instanceof AlgorithmModel ) {
			AlgorithmModel model = (AlgorithmModel) value;
			return new IContentItem[] {
				new BeanContentItem( model.getAlgorithm(), this, "name" ),
				new BeanContentItem( model.getAlgorithm(), this, "runner", "algorithm runner" ),
				new BeanContentItem( model.getAlgorithm(), this, "validator", "algorithm validator" )
			};
		} else if ( value instanceof ConnectionModel ) {
			ConnectionModel model = (ConnectionModel) value;
			return new MetaContentItem[] {
				new MetaContentItem( this, "source", model.getSource().getName() ),
				new MetaContentItem( this, "source slot", String.valueOf( model.getSourceSlot() ) ),
				new MetaContentItem( this, "target", model.getTarget().getName() ),
				new MetaContentItem( this, "target slot", String.valueOf( model.getTargetSlot() ) )
			};
		} else if ( value instanceof OperatorModel ) {
			OperatorModel model = (OperatorModel) value;
			OperatorDescriptor desc = model.getOperator().descriptor();
			LinkedList list = new LinkedList();
			// add information from descriptor
			list.add( new MetaContentItem( this, "location", "(" + model.getLocation().x + "," + model.getLocation().y + ")", true ) );
			list.add( new MetaContentItem( this, "category", desc.getCategory(), true ) );
			list.add( new MetaContentItem( this, "datatype", desc.getDatatypeName(), true ) );
			list.add( new MetaContentItem( this, "description", desc.getShortDescription() ) );
			list.add( new MetaContentItem( this, "display name", desc.getDisplayName(), true ) );
			list.add( new MetaContentItem( this, "number of input slots", String.valueOf( desc.getInputSize() ), true ) );
			list.add( new MetaContentItem( this, "number of output slots", String.valueOf( desc.getOutputSize() ), true ) );
			list.add( new MetaContentItem( this, "vendor", desc.getVendor(), true ) );
			PropertyDescriptor[] pd = PropertyUtils.getPropertyDescriptors( model.getOperator() );
			for ( int i = 0; i < pd.length; i++ ) {
				BeanContentItem beanContentItem = new BeanContentItem( model.getOperator(), this, pd[i].getName() );
				if ( pd[i].getName().equals( "id" ) ) {
					beanContentItem.setExpert( true );
					beanContentItem.setReadonly( true );
				}
				list.add( beanContentItem );
			}
			return (IContentItem[]) list.toArray( new IContentItem[list.size()] );
		} else {
			return new IContentItem[0];
		}
	}
	*/
	
	public void resetValue() {
		// do nothing
	}
	
	public void setValue( Object value, boolean convert ) {
		// ignore ...
	}
	
}
