/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:38 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: AppendArrayItemAction.java,v $
 * 
 */
package org.evolvica.ui.views.properties.actions;

import java.beans.PropertyVetoException;
import java.lang.reflect.Array;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableTreeViewer;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;
import org.evolvica.ui.AlgorithmEditor;
import org.evolvica.ui.Images;
import org.evolvica.ui.views.properties.ArrayContentItem;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:53:38 $
 */
public class AppendArrayItemAction extends Action {
	
	protected TableTreeViewer viewer = null;
	
	public AppendArrayItemAction( TableTreeViewer viewer ) {
		super( "Add Item to Array", IAction.AS_PUSH_BUTTON );
		this.viewer = viewer;
		setEnabled( false );
		setImageDescriptor( Images.getImageDescriptor( "icons/add_item16.png" ) );
		setHoverImageDescriptor( Images.getImageDescriptor( "icons/add_item16hover.png" ) );
		setToolTipText( "add item to array" );
	}
	
	public void run() {
		IEditorPart editor = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		try {
			viewer.cancelEditing();
			ArrayContentItem contentItem = (ArrayContentItem) ((IStructuredSelection) viewer.getSelection()).getFirstElement();
			addToArrayContentItem( contentItem );
			viewer.refresh( contentItem.getParent() );
			if ( editor instanceof AlgorithmEditor ) {
				((AlgorithmEditor) editor).setDirty( true );
			}
		} catch ( Exception ex ) {
			String message = ex.getMessage();
			if ( ex instanceof InvocationTargetException ) {
				Throwable cause = ((InvocationTargetException) ex).getCause();
				if ( cause instanceof PropertyVetoException ) {
					message = ((PropertyVetoException) cause).getMessage();
				}
			}
			MessageDialog.openError( editor.getSite().getShell(), "Error adding item to array", message );
		}
	}
	
	protected void addToArrayContentItem( ArrayContentItem item ) throws Exception {
		Object array = item.getArray();
		Class type = array.getClass();
		int length = Array.getLength( array );
		int index = item.getIndex();
		Object newArray = Array.newInstance( type.getComponentType(), length + 1 );
		System.arraycopy( array, 0, newArray, 0, index + 1 );
		System.arraycopy( array, index + 1, newArray, index + 2, length - index - 1 );
		item.getParent().setValue( newArray, false );
		item.getParent().clearCache();
		item.getParent().synchronize();
		viewer.refresh();
	}
	
}
