/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:38 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: AppendArrayItemAsChildAction.java,v $
 * 
 */
package org.evolvica.ui.views.properties.actions;

import java.beans.PropertyVetoException;
import java.lang.reflect.Array;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableTreeViewer;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;
import org.evolvica.ui.AlgorithmEditor;
import org.evolvica.ui.Images;
import org.evolvica.ui.views.properties.ArrayContentItem;
import org.evolvica.ui.views.properties.BeanContentItem;
import org.evolvica.ui.views.properties.IContentItem;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:53:38 $
 */
public class AppendArrayItemAsChildAction extends Action {
	
	protected TableTreeViewer viewer = null;
	
	public AppendArrayItemAsChildAction( TableTreeViewer viewer ) {
		super( "Add New Item as Child", IAction.AS_PUSH_BUTTON );
		this.viewer = viewer;
		setEnabled( false );
		setImageDescriptor( Images.getImageDescriptor( "icons/add_child16.png" ) );
		setHoverImageDescriptor( Images.getImageDescriptor( "icons/add_child16hover.png" ) );
		setToolTipText( "add new item to array as a child" );
	}
	
	public void run() {
		IEditorPart editor = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		try {
			viewer.cancelEditing();
			IContentItem contentItem = (IContentItem) ((IStructuredSelection) viewer.getSelection()).getFirstElement();
			addToContentItem( contentItem );
			viewer.refresh( contentItem );
			if ( editor instanceof AlgorithmEditor ) {
				((AlgorithmEditor) editor).setDirty( true );
			}
		} catch ( Exception ex ) {
			String message = ex.getMessage();
			if ( ex instanceof InvocationTargetException ) {
				Throwable cause = ((InvocationTargetException) ex).getCause();
				if ( cause instanceof PropertyVetoException ) {
					message = ((PropertyVetoException) cause).getMessage();
				}
			}
			MessageDialog.openError( editor.getSite().getShell(), "Error adding item as a child", message );
		}
	}
	
	protected void addToContentItem( IContentItem item ) throws Exception {
		boolean refreshNeeded = false;
		if ( item instanceof ArrayContentItem ) {
			Class type = ((ArrayContentItem) item).getType();
			Object array = item.getValue();
			Object newArray = null;
			if ( array != null ) {
				int length = Array.getLength( array );
				newArray = Array.newInstance( type.getComponentType(), length + 1 );
				System.arraycopy( array, 0, newArray, 1, length );
			} else {
				newArray = Array.newInstance( type.getComponentType(), 1 );
			}
			item.setValue( newArray, false );
			refreshNeeded = true;
		} else if ( item instanceof BeanContentItem ) {
			Class type = ((BeanContentItem) item).getType();
			Object array = item.getValue();
			Object newArray = null;
			if ( array != null ) {
				int length = Array.getLength( array );
				newArray = Array.newInstance( type.getComponentType(), length + 1 );
				System.arraycopy( array, 0, newArray, 1, length );
			} else {
				newArray = Array.newInstance( type.getComponentType(), 1 );
			}
			item.setValue( newArray, false );
			refreshNeeded = true;
		}
		if ( refreshNeeded == true ) {
			item.clearCache();
			item.synchronize();
			viewer.refresh();
		}
	}
	
}
