/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:39 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: RemoveArrayItemFromChildAction.java,v $
 * 
 */
package org.evolvica.ui.views.properties.actions;

import java.beans.PropertyVetoException;
import java.lang.reflect.Array;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableTreeViewer;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;
import org.evolvica.ui.AlgorithmEditor;
import org.evolvica.ui.Images;
import org.evolvica.ui.views.properties.ArrayContentItem;
import org.evolvica.ui.views.properties.BeanContentItem;
import org.evolvica.ui.views.properties.IContentItem;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:53:39 $
 */
public class RemoveArrayItemFromChildAction extends Action {
	
	protected TableTreeViewer viewer = null;
	
	public RemoveArrayItemFromChildAction( TableTreeViewer viewer ) {
		super( "Remove First Child", IAction.AS_PUSH_BUTTON );
		this.viewer = viewer;
		setEnabled( false );
		setImageDescriptor( Images.getImageDescriptor( "icons/remove_child16.png" ) );
		setHoverImageDescriptor( Images.getImageDescriptor( "icons/remove_child16hover.png" ) );
		setToolTipText( "remove first child item from array" );
	}
	
	public void run() {
		IEditorPart editor = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		try {
			viewer.cancelEditing();
			IContentItem contentItem = (IContentItem) ((IStructuredSelection) viewer.getSelection()).getFirstElement();
			removeFromContentItem( contentItem );
			viewer.refresh( contentItem );
			if ( editor instanceof AlgorithmEditor ) {
				((AlgorithmEditor) editor).setDirty( true );
			}
		} catch ( Exception ex ) {
			String message = ex.getMessage();
			if ( ex instanceof InvocationTargetException ) {
				Throwable cause = ((InvocationTargetException) ex).getCause();
				if ( cause instanceof PropertyVetoException ) {
					message = ((PropertyVetoException) cause).getMessage();
				}
			}
			MessageDialog.openError( editor.getSite().getShell(), "Error removing child item", message );
		}
	}
	
	protected void removeFromContentItem( IContentItem item ) throws Exception {
		boolean refreshNeeded = false;
		if ( item instanceof ArrayContentItem ) {
			Object array = item.getValue();
			if ( array != null ) {
				int length = Array.getLength( array );
				if ( length > 0 ) {
					Object newArray = Array.newInstance( ((ArrayContentItem) item).getType().getComponentType(), length - 1 );
					System.arraycopy( array, 1, newArray, 0, length - 1 );
					item.setValue( newArray, false );
					refreshNeeded = true;
				}
			}
		} else if ( item instanceof BeanContentItem ){
			Object array = item.getValue();
			Class type = ((BeanContentItem) item).getType();
			if ( type.isArray() ) { // type should ALWAYS be an array
				int length = Array.getLength( array );
				if ( length > 0 ) {
					Object newArray = Array.newInstance( type.getComponentType(), length - 1 );
					System.arraycopy( array, 1, newArray, 0, length - 1 );
					item.setValue( newArray, false );
					refreshNeeded = true;
				}
			}
		}
		if ( refreshNeeded == true ) {
			item.clearCache();
			item.synchronize();
			viewer.refresh();
		}
	}
	
}
