/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:39 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: SetItemNullAction.java,v $
 * 
 */
package org.evolvica.ui.views.properties.actions;

import java.beans.PropertyVetoException;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableTreeViewer;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;
import org.evolvica.ui.AlgorithmEditor;
import org.evolvica.ui.Images;
import org.evolvica.ui.views.properties.ArrayContentItem;
import org.evolvica.ui.views.properties.IContentItem;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:53:39 $
 */
public class SetItemNullAction extends Action {
	
	protected TableTreeViewer viewer = null;
	
	public SetItemNullAction( TableTreeViewer viewer ) {
		super( "Reset Value", IAction.AS_PUSH_BUTTON );
		this.viewer = viewer;
		setEnabled( false );
		setImageDescriptor( Images.getImageDescriptor( "icons/setnull16.png" ) );
		setHoverImageDescriptor( Images.getImageDescriptor( "icons/setnull16hover.png" ) );
		setToolTipText( "reset value/set value to NULL" );
	}
	
	public void run() {
		IEditorPart editor = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		try {
			viewer.cancelEditing();
			IContentItem contentItem = (IContentItem) ((IStructuredSelection) viewer.getSelection()).getFirstElement();
			contentItem.resetValue();
			contentItem.clearCache();
			contentItem.synchronize();
			viewer.refresh( contentItem );
			if ( contentItem instanceof ArrayContentItem ) {
				viewer.refresh( contentItem.getParent() );
			}
			if ( editor instanceof AlgorithmEditor ) {
				((AlgorithmEditor) editor).setDirty( true );
			}
		} catch ( Exception ex ) {
			String message = ex.getMessage();
			if ( ex instanceof InvocationTargetException ) {
				Throwable cause = ((InvocationTargetException) ex).getCause();
				if ( cause instanceof PropertyVetoException ) {
					message = ((PropertyVetoException) cause).getMessage();
				}
			}
			MessageDialog.openError( editor.getSite().getShell(), "Error resetting property", message );
		}
	}

}
