/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:40 $
 * $Revision: 1.4 $
 * $State: Exp $
 * $RCSfile: NewAlgorithmCreationWizard.java,v $
 *
 */
package org.evolvica.ui.wizards;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.*;
import org.eclipse.core.runtime.*;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.*;

/** Documentation missing.
 *
 * @author rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.4 $
 */
public class NewAlgorithmCreationWizard extends Wizard implements INewWizard {
	
	private NewAlgorithmCreationWizardPage page;
	private ISelection selection;
	private String algorithmName; 
	
	/**
	 * Constructor for SampleNewWizard.
	 */
	public NewAlgorithmCreationWizard() {
		super();
		setNeedsProgressMonitor( true );
	}
	
	/**
	 * Adding the page to the wizard.
	 */

	public void addPages() {
		page = new NewAlgorithmCreationWizardPage( selection );
		addPage( page );
	}

	/**
	 * This method is called when 'Finish' button is pressed in
	 * the wizard. We will create an operation and run it
	 * using wizard as execution context.
	 */
	public boolean performFinish() {
//		final String containerName = page.getContainerName();
		final String algorithmName = page.getAlgorithmName();
		IRunnableWithProgress op = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException {
				try {
//					doFinish(containerName, fileName, monitor);
					doFinish( algorithmName, monitor);
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} finally {
					monitor.done();
				}
			}
		};
		try {
			getContainer().run( true, false, op );
		} catch (InterruptedException e) {
			return false;
		} catch (InvocationTargetException e) {
			Throwable realException = e.getTargetException();
			MessageDialog.openError( getShell(), "Error", realException.getMessage() );
			return false;
		}
		return true;
	}
	
	/**
	 * The worker method. It will find the container, create the
	 * file if missing or just replace its contents, and open
	 * the editor on the newly created file.
	 */

	private void doFinish( String algorithmName, IProgressMonitor monitor ) throws CoreException {
		String algorithmFileName = algorithmName + ".eva";
		monitor.beginTask( "creating " + algorithmName, 2 );
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		if ( selection != null ) {
			Object resource = ((StructuredSelection) selection).getFirstElement();
			if ( resource instanceof IResource ) {
				IContainer container = null;
				if ( resource instanceof IContainer ) {
					container = (IContainer) resource;
				} else {
					container = ((IResource) resource).getParent();
				}
				final IFile file = container.getFile( new Path( algorithmFileName ) );
				try {
					InputStream stream = openContentStream( algorithmName );
					if ( file.exists() ) {
						file.setContents( stream, true, true, monitor );
					} else {
						file.create( stream, true, monitor );
					}
					stream.close();
				} catch ( IOException e ) {
					// do nothing
				}
				monitor.worked( 1 );
				monitor.setTaskName( "Opening file for editing..." );
				getShell().getDisplay().asyncExec( new Runnable() {
					public void run() {
						IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
						try {
							page.openEditor( file );
						} catch (PartInitException e) {
							// do nothing
						}
					}
				});
			} else {
				throwCoreException( "invalid parent resource selection - can't create configuration file" );
			}
		}
		/*
		*/
		monitor.worked( 1 );
	}
	
	/**
	 * We will initialize file contents with a sample text.
	 */

	protected InputStream openContentStream( String algorithmName ) {
		StringBuffer contents = new StringBuffer();
		contents.append( "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" );
		contents.append( "<algorithm name=\"" + algorithmName + "\" version=\"0.2\">\n" );
		contents.append( "\t<engine></engine>\n" );
		contents.append( "\t<structure></structure>\n" );
		contents.append( "\t<connectivity></connectivity>\n" );
		contents.append( "\t<view></view>\n" );
		contents.append( "</algorithm>\n" );
		return new ByteArrayInputStream( contents.toString().getBytes() );
	}

	private void throwCoreException(String message) throws CoreException {
		IStatus status = new Status(IStatus.ERROR, "org.evolvica.ui", IStatus.OK, message, null );
		throw new CoreException( status );
	}

	/**
	 * We will accept the selection in the workbench to see if
	 * we can initialize from it.
	 * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.selection = selection;
	}
}