/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:53:40 $
 * $Revision: 1.3 $
 * $State: Exp $
 * $RCSfile: NewAlgorithmCreationWizardPage.java,v $
 *
 */
package org.evolvica.ui.wizards;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.*;

/** Documentation missing.
 *
 * @author rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.3 $
 */
public class NewAlgorithmCreationWizardPage extends WizardPage {
	
	private Text algorithmName;
	private ISelection selection;

	/**
	 * Constructor for SampleNewWizardPage.
	 * @param pageName
	 */
	public NewAlgorithmCreationWizardPage(ISelection selection) {
		super( "wizardPage" );
		setTitle( "Create Evolutionary Algorithm" );
		setDescription( "This wizard creates an empty evolutionary algorithm." );
		this.selection = selection;
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl( Composite parent ) {
		Composite container = new Composite( parent, SWT.NULL );
		GridLayout layout = new GridLayout();
		container.setLayout( layout );
		layout.numColumns = 2;
		layout.verticalSpacing = 9;
		
		// configuration file name
		Label label = new Label( container, SWT.NULL );
		label.setText("&Algorithm name:");
		algorithmName = new Text( container, SWT.BORDER | SWT.SINGLE );
		GridData gd = new GridData( GridData.FILL_HORIZONTAL );
		algorithmName.setLayoutData( gd );
		algorithmName.addModifyListener( new ModifyListener() {
			public void modifyText( ModifyEvent e ) {
				dialogChanged();
			}
		});
		
		initialize();
		dialogChanged();
		setControl(container);
	}
	
	/**
	 * Tests if the current workbench selection is a suitable
	 * container to use.
	 */
	
	private void initialize() {
		if (selection!=null && selection.isEmpty()==false && selection instanceof IStructuredSelection) {
			IStructuredSelection ssel = (IStructuredSelection)selection;
			if (ssel.size()>1) return;
			Object obj = ssel.getFirstElement();
			if (obj instanceof IResource) {
				IContainer container;
				if (obj instanceof IContainer) {
					container = (IContainer)obj;
				} else {
					container = ((IResource)obj).getParent();
				}
			}
		}
		algorithmName.setText( "myAlgorithm" );
	}
	
	/**
	 * Ensures that the text field is set.
	 */
	private void dialogChanged() {
		String fileName = getAlgorithmName();
		if (fileName.length() == 0) {
			updateStatus( "Algorithm name must be specified" );
			return;
		}
		updateStatus( null );
		
	}

	private void updateStatus( String message ) {
		setErrorMessage( message );
		setPageComplete( message == null );
	}

	public String getAlgorithmName() {
		return algorithmName.getText();
	}
	
}