/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/06/14 10:57:28 $
 * $Revision: 1.4 $
 * $State: Exp $
 * $RCSfile: NewOperatorCreationWizard.java,v $
 * 
 */
package org.evolvica.ui.wizards;

import java.net.URI;

import org.apache.commons.lang.StringUtils;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.*;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.internal.corext.util.JavaModelUtil;
import org.eclipse.jdt.internal.ui.JavaPlugin;
import org.eclipse.jdt.internal.ui.JavaPluginImages;
import org.eclipse.jdt.internal.ui.wizards.NewElementWizard;
import org.eclipse.jdt.internal.ui.wizards.NewWizardMessages;
import org.eclipse.jdt.ui.wizards.NewClassWizardPage;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.evolvica.engine.DescriptorWriter;
import org.evolvica.engine.OperatorDescriptor;
import org.evolvica.ui.AlgorithmEditorPlugin;
import org.evolvica.ui.PathManager;
import org.evolvica.ui.SourceManager;

/** Documentation missing.
 *
 * @author Andreas Rummler
 * @since RELEASE_NUMBER_NOT_DETERMINED
 * @version $Revision: 1.4 $ $Date: 2004/06/14 10:57:28 $
 */
public class NewOperatorCreationWizard extends NewElementWizard {

	private NewClassWizardPage fPage;

	public NewOperatorCreationWizard() {
		super();
		setDefaultPageImageDescriptor(JavaPluginImages.DESC_WIZBAN_NEWCLASS);
		setDialogSettings(JavaPlugin.getDefault().getDialogSettings());
		setWindowTitle( NewWizardMessages.getString( "NewClassCreationWizard.title" ) );
	}
	
	public void addPages() {
		super.addPages();
		fPage= new NewClassWizardPage();
		addPage(fPage);
		fPage.init(getSelection());
	}

	protected void finishPage(IProgressMonitor monitor) throws InterruptedException, CoreException {
		fPage.createType( monitor ); // use the full progress monitor
		ICompilationUnit unit = JavaModelUtil.toOriginal( fPage.getCreatedType().getCompilationUnit() );
		if ( unit != null ) {
			if ( SourceManager.isOperator( unit ) ) {
				// create descriptor resource
				try {
					String fullName = PathManager.getFullyQualifiedName( unit );
					URI sourceLocation = PathManager.getLocation( unit );
					URI writeLocation = new URI( sourceLocation.toString() + StringUtils.replace( fullName, ".", "/" ) + ".descriptor" );
					OperatorDescriptor desc = new OperatorDescriptor( fullName, null, null ); // we don't need the location of the descriptor
					DescriptorWriter.write( desc, writeLocation );
					IContainer container = unit.getResource().getParent();
					IFile file = container.getFile( new Path( StringUtils.replace( unit.getElementName(), ".java", ".descriptor" ) ) );
					file.refreshLocal( 0, monitor );
				} catch ( Exception ex ) {
					throw new CoreException( new Status(
						IStatus.ERROR,
						AlgorithmEditorPlugin.getDefault().getDescriptor().getUniqueIdentifier(),
						-1,
						"unable to create descriptor file",
						ex
					) );
				}
			}
			// open operator in editor
			IResource resource= unit.getResource();
			selectAndReveal( resource );
			openResource( (IFile) resource );
		}	
	}
	
	protected void openResource( final IFile resource ) {
		final IWorkbenchPage activePage= JavaPlugin.getActivePage();
		if ( activePage != null ) {
			final Display display= getShell().getDisplay();
			if ( display != null ) {
				display.asyncExec(new Runnable() {
					public void run() {
						try {
							activePage.openEditor( resource, "org.evolvica.ui.editors.operator.OperatorEditor", true );
						} catch (PartInitException e) {
							JavaPlugin.log(e);
						}
					}
				});
			}
		}
	}
	
	public boolean performFinish() {
		warnAboutTypeCommentDeprecation();
		return super.performFinish();
	}

}
