/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:36 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: Digester.java,v $
 * 
 */
package org.evolvica.util;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import org.apache.commons.lang.StringUtils;

import sun.misc.BASE64Encoder;

/** A simple class which generates message digests for given strings/bytearrays.
 *
 * @author Andreas Rummler
 * @since 0.6.0
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:58:36 $
 */
public class Digester {
	
	private static final BASE64Encoder BASE64 = new BASE64Encoder();
	
	private static MessageDigest MD5 = null;
	
	private static MessageDigest SHA = null;
	
	static {
		try {
			MD5 = MessageDigest.getInstance( "MD5" );
			SHA = MessageDigest.getInstance( "SHA" );
		} catch ( NoSuchAlgorithmException nsax ) {
			// be quiet
		}
	}
	
	private static final String toAlphanumeric( String s ) {
		String rval = StringUtils.replace( s, "=", "" );
		String plus = "0";
		String slash = "a";
		for ( int i = 0; i < rval.length(); i++ ) {
			if ( StringUtils.isNumeric( String.valueOf( rval.charAt( i ) ) ) ) {
				plus = String.valueOf( rval.charAt( i ) );
			}
			if ( StringUtils.isAlpha( String.valueOf( rval.charAt( i ) ) ) ) {
				slash = String.valueOf( rval.charAt( i ) );
			}
		}
		rval = StringUtils.replace( rval, "+", plus );
		rval = StringUtils.replace( rval, "/", slash );
		StringBuffer sb = new StringBuffer( rval );
		sb.insert( 8, "-" );
		sb.insert( 13, "-" );
		sb.insert( 18, "-" );
		sb.insert( 23, "-" );
		return sb.toString().toLowerCase();
	}
	
	/**
	 * Generates a digest which only contains alphanumeric characters and minus-signs (similiar to an UUID).
	 * 
	 * @param array array of bytes
	 * @return generated message digest
	 */
	public static final String getDigest( byte[] array ) {
		byte[] hash = null;
		if ( SHA != null ) {
			hash = getSHADigest( array );
		} else {
			hash = getMD5Digest( array );
		}
		return toAlphanumeric( BASE64.encode( hash ) );
	}
	
	/**
	 * Generates a digest which only contains alphanumeric characters and minus-signs (similiar to an UUID).
	 * 
	 * @param s string from which the digest should be generated
	 * @return generated message digest
	 */
	public static final String getDigest( String s ) {
		return getDigest( s.getBytes() );
	}
	
	/**
	 * Generates a MD5 digest for the given string.
	 * 
	 * @param s string to be used for digest generation
	 * @return generated message digest
	 */
	public static final byte[] getMD5Digest( String s ) {
		return getMD5Digest( s.getBytes() );
	}
	
	/**
	 * Generates a MD5 digest for the given bytearray.
	 * 
	 * @param byteArray byte array to be used for digest generation
	 * @return generated message digest
	 */
	public static final byte[] getMD5Digest( byte[] byteArray ) {
		if ( MD5 != null ) {
			return MD5.digest( byteArray );
		} else {
			return new byte[] { 0 };
		}
	}
	
	/**
	 * Generates a SHA-1 digest for the given string.
	 * 
	 * @param s string to be used for digest generation
	 * @return generated message digest
	 */
	public static final byte[] getSHADigest( String s ) {
		return getSHADigest( s.getBytes() );
	}
	
	/**
	 * Generates a SHA-1 digest for the given string.
	 * 
	 * @param byteArray byte array to be used for digest generation
	 * @return generated message digest
	 */
	public static final byte[] getSHADigest( byte[] byteArray ) {
		if ( SHA != null ) {
			return SHA.digest( byteArray );
		} else {
			return new byte[] { 0 };
		}
	}
	
}
