/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:36 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: Directory.java,v $
 * 
 */
package org.evolvica.util;

import java.io.File;
import java.net.URI;

/** Class for a directory.
 * This is just a helper to differentiate between files and directories.
 * Use this class for operator properties that are directories. Use a normal
 * File instance for regular files.
 *
 * @author Andreas Rummler
 * @since 0.5.0
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:36 $
 */
public class Directory extends File {

	/**
	 * Constructor.
	 * 
	 * @param pathname directory path
	 * @throws IllegalArgumentException thrown in case the path is not a directory
	 */
	public Directory( String pathname ) {
		super( pathname );
		if ( ! isDirectory() ) {
			throw new IllegalArgumentException( "pathname is not a directory" );
		}
	}
	
	/**
	 * Constructor.
	 * 
	 * @param uri path URI
	 * @throws IllegalArgumentException thrown in case the path is not a directory
	 */
	public Directory( URI uri ) {
		super( uri );
		if ( ! isDirectory() ) {
			throw new IllegalArgumentException( "pathname is not a directory" );
		}
	}

}
