
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:36 $
 * $Revision: 1.9 $
 * $State: Exp $
 * $RCSfile: EVLog.java,v $
 *
 */

package org.evolvica.util;

import java.io.File;
import java.util.Enumeration;

import org.apache.commons.lang.SystemUtils;
import org.apache.log4j.*;
import org.apache.log4j.xml.DOMConfigurator;

/** Logging of events.
 * This class requires the Log4J package.
 *
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.9 $ $Date: 2004/05/26 11:58:36 $
 */
public class EVLog {
	
	/** Evolvica generic logger.
	* Use this logger for info and warning messages.
	*
	*/
	public static final Logger def = Logger.getLogger( "evolvica" ); //$NON-NLS-1$
	
	/** Logger for algorithm engine related stuff.
	* Use this logger for debug messages related to the algorithm engine.
	*
	*/
	public static final Logger engine = Logger.getLogger( "evolvica.engine" ); //$NON-NLS-1$
	
	/** Logger for event handling.
	* Use this logger for debug messages related to event handling.
	*
	*/
	public static final Logger event = Logger.getLogger( "evolvica.event" ); //$NON-NLS-1$
	
	static {
		try {
			File configFile = new File( SystemUtils.USER_HOME + SystemUtils.FILE_SEPARATOR + ".log4j.xml" );
			if ( configFile.exists() ) {
				DOMConfigurator.configure( configFile.toURL() );
				def.info( "using Log4J configuration from " + configFile.toString() );
			} else {
				throw new Exception( "Log4J user configuration file not found or not readable, using default configuration" );
			}
		} catch ( Exception ex ) {
			System.err.println( ex.getMessage() );
			// make default configuration
			Logger root = LogManager.getLoggerRepository().getRootLogger();
			root.setLevel( Level.INFO );
			boolean append = true;
			for( Enumeration enum = root.getAllAppenders(); enum.hasMoreElements(); ) {
				Appender appender = (Appender) enum.nextElement();
				if ( appender instanceof ConsoleAppender ) {
					append = false;
					appender.setLayout( new PatternLayout( "%d [%-5p] %m%n" ) );
				}
			}
			if ( append == true ) {
				root.addAppender( new ConsoleAppender( new PatternLayout( "%d [%-5p] %m%n" ) ) );
			}
		}
	}
	
	/** Logs a debug message.
	*
	* @param message message
	*/
	public static void debug( String message ) {
		def.debug( message );
	}

	/** Logs a debug message.
	*
	* @param message message
	* @param throwable throwable causing this message
	*/
	public static void debug( String message, Throwable throwable ) {
		def.debug( message, throwable );
	}

	/** Logs an error message.
	*
	* @param message message
	*/
	public static void error( String message ) {
		def.error( message );
	}

	/** Logs an error message.
	*
	* @param message message
	* @param throwable throwable causing this message
	*/
	public static void error( String message, Throwable throwable ) {
		def.error( message, throwable );
	}

	/** Logs a fatal condition.
	*
	* @param message message
	*/
	public static void fatal( String message ) {
		def.fatal( message );
	}

	/** Logs a fatal condition.
	*
	* @param message message
	* @param throwable throwable causing this message
	*/
	public static void fatal( String message, Throwable throwable ) {
		def.fatal( message, throwable );
	}

	/** Logs an info message.
	*
	* @param message message
	*/
	public static void info( String message ) {
		def.info( message );
	}

	/** Logs an info message.
	*
	* @param message message
	* @param throwable throwable causing this message
	*/
	public static void info( String message, Throwable throwable ) {
		def.info( message, throwable );
	}

	/** Logs a warning message.
	*
	* @param message message
	*/
	public static void warn( String message ) {
		def.warn( message );
	}

	/** Logs a warning message.
	*
	* @param message message
	* @param throwable throwable causing this message
	*/
	public static void warn( String message, Throwable throwable ) {
		def.warn( message, throwable );
	}
	
}
