/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:37 $
 * $Revision: 1.2 $
 * $State: Exp $
 * $RCSfile: Metadata.java,v $
 * 
 */
package org.evolvica.util;

/** A metadata object.
 *
 * @author Andreas Rummler
 * @since 0.6.0
 * @version $Revision: 1.2 $ $Date: 2004/05/26 11:58:37 $
 */
public class Metadata {
	
	/**
	 * Separator used in metadata paths.
	 */
	public static final String SEPARATOR = "/";
	
	/**
	 * ID for metadata change events.
	 */
	public static final String METADATA_CHANGE = "METADATA_CHANGE";
	
	private String path = null;
	
	private String value = null;

	/**
	 * Constructor.
	 * 
	 * @param path metadata path
	 * @param value metadata value
	 */
	public Metadata( String path, String value ) {
		this.path = path;
		this.value = value;
	}
	
	/**
	 * Constructor.
	 * 
	 * @param path metadata path
	 * @param value metadata value (boolean)
	 */
	public Metadata( String path, boolean value ) {
		this( path, Boolean.toString( value ) );
	}
	
	/**
	 * Constructor.
	 * 
	 * @param path metadata path
	 * @param value metadata value (float)
	 */
	public Metadata( String path, float value ) {
		this( path, Float.toString( value ) );
	}
	
	/**
	 * Constructor.
	 * 
	 * @param path metadata path
	 * @param value metadata value (int)
	 */
	public Metadata( String path, int value ) {
		this( path, Integer.toString( value ) );
	}
	
	/**
	 * Returns the path of this metadata object.
	 * 
	 * @return path
	 */
	public String getPath() {
		return path;
	}
	
	/**
	 * Returns the value of this metadata object.
	 * 
	 * @return value
	 */
	public String getValue() {
		return value;
	}
	
	/**
	 * Returns the value of this metadata object as a boolean value.
	 * 
	 * @return metadata value
	 */
	public boolean getValueAsBoolean() {
		return Boolean.valueOf( value ).booleanValue();
	}
	
	/**
	 * Returns the value of this metadata object as a float value.
	 * 
	 * @return metadata value
	 */
	public float getValueAsFloat() {
		return Float.parseFloat( value );
	}
	
	/**
	 * Returns the value of this metadata object as an integer value.
	 * 
	 * @return metadata value
	 */
	public int getValueAsInteger() {
		return Integer.parseInt( value );
	}
	
	/**
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return path + " = " + value;
	}
	
}
