
/*
 * Copyright (C) 2000-2004 Technical University of Ilmenau, Germany
 *                         Department of Electronic Circuits & Systems
 *
 * You may only use this file according to the terms of the license agreement included
 * within the file LICENSE which is distributed together with the software package.
 *
 * CVS Info:
 * $Author: rummler $
 * $Date: 2004/05/26 11:58:39 $
 * $Revision: 1.5 $
 * $State: Exp $
 * $RCSfile: UniqueID.java,v $
 *
 */

package org.evolvica.util;

/** Class for generating unique identifiers.
 * As identifier there can be either integers, long integers or strings.
 * Every request for an ID will increment an internal
 * counter, so there is no way for getting an ID twice.
 * 
 * @since 0.4.0
 * @author Andreas Rummler
 * @version $Revision: 1.5 $ $Date: 2004/05/26 11:58:39 $
 */
public class UniqueID {
	
	/** Internal integer ID.
	* 
	*/
	static protected int intID = 0;
	
	/** Internal long ID.
	* 
	*/
	static protected long longID = 0;
	
	/** Internal string ID.
	* 
	*/
	static protected StringBuffer stringID = new StringBuffer( "a" );
	
	/** Gets a new integer ID.
	* 
	* @return new ID
	*/
	static public int nextInt() {
		return intID++;
	}
	
	/** Gets a new long ID.
	* 
	* @return new ID
	*/
	static public long nextLong() {
		return longID++;
	}
	
	/** Gets a new string ID.
	* 
	* @return new string ID
	*/
	static public String nextString() {
		String rt = stringID.toString();
		incString();
		return rt;
	}
	
	/** Decrements a string.
	* Decrementing a string means subtracting 1 on a character. Example: dec( k ) = j, dec( aa ) = z, and so on.
	*/
	static protected void decString() {
		int index = stringID.length() - 1;
		boolean return_bit = false;
		char indexChar;
		while ( index != -1 ) {
			indexChar = stringID.charAt( index );
			switch ( indexChar ) {
				case 'a' :
					if ( stringID.length() == 1 ) {
						return_bit = true;
					} else {
						stringID.setCharAt( index, 'z' );
						if ( index == 1 && stringID.charAt( index - 1 ) == 'a' ) {
							stringID.deleteCharAt( index - 1 );
							return_bit = true;
						}
					}
				break;
				default :
					stringID.setCharAt( index, --indexChar );
					return_bit = true;
				break;
			}
			if ( return_bit == true ) {
				return;
			} else {
				--index;
			}
		}
	}
	
	/** Increments a string.
	* Incrementing a string means adding 1 to a character. Example: inc( k ) = l, inc( z ) = aa, and so on.
	*/
	static protected void incString() {
		int index = stringID.length() - 1;
		boolean return_bit = false;
		char indexChar;
		while ( index != -1 ) {
			indexChar = stringID.charAt( index );
			switch ( indexChar ) {
				case 'z' :
					stringID.setCharAt( index, 'a' );
				break;
				default :
					stringID.setCharAt( index, ++indexChar );
					return_bit = true;
				break;
			}
			if ( return_bit == true ) {
				return;
			} else {
				index--;
				if ( index == -1 ) {
					stringID.insert( 0, 'a' );
				}
			}
		}
	}
	
}
